@@ -19,56 +19,63 @@
 package org.apache.paimon.catalog;
 
 import org.apache.paimon.CoreOptions;
-import org.apache.paimon.TableType;
+import org.apache.paimon.PagedList;
+import org.apache.paimon.Snapshot;
 import org.apache.paimon.factories.FactoryUtil;
 import org.apache.paimon.fs.FileIO;
 import org.apache.paimon.fs.FileStatus;
 import org.apache.paimon.fs.Path;
-import org.apache.paimon.metastore.MetastoreClient;
-import org.apache.paimon.operation.FileStoreCommit;
-import org.apache.paimon.operation.Lock;
 import org.apache.paimon.options.Options;
 import org.apache.paimon.partition.Partition;
+import org.apache.paimon.partition.PartitionStatistics;
 import org.apache.paimon.schema.Schema;
 import org.apache.paimon.schema.SchemaChange;
 import org.apache.paimon.schema.SchemaManager;
 import org.apache.paimon.schema.TableSchema;
-import org.apache.paimon.table.CatalogEnvironment;
 import org.apache.paimon.table.FileStoreTable;
-import org.apache.paimon.table.FileStoreTableFactory;
 import org.apache.paimon.table.FormatTable;
+import org.apache.paimon.table.Instant;
 import org.apache.paimon.table.Table;
+import org.apache.paimon.table.TableSnapshot;
 import org.apache.paimon.table.object.ObjectTable;
-import org.apache.paimon.table.sink.BatchWriteBuilder;
+import org.apache.paimon.table.sink.BatchTableCommit;
 import org.apache.paimon.table.system.SystemTableLoader;
 import org.apache.paimon.types.RowType;
-import org.apache.paimon.utils.Preconditions;
+
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 import javax.annotation.Nullable;
 
 import java.io.IOException;
 import java.util.ArrayList;
-import java.util.Collections;
 import java.util.HashMap;
 import java.util.HashSet;
-import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.Objects;
 import java.util.Optional;
 import java.util.stream.Collectors;
 
+import static org.apache.paimon.CoreOptions.OBJECT_LOCATION;
+import static org.apache.paimon.CoreOptions.PATH;
 import static org.apache.paimon.CoreOptions.TYPE;
-import static org.apache.paimon.CoreOptions.createCommitUser;
+import static org.apache.paimon.catalog.CatalogUtils.checkNotBranch;
+import static org.apache.paimon.catalog.CatalogUtils.checkNotSystemDatabase;
+import static org.apache.paimon.catalog.CatalogUtils.checkNotSystemTable;
+import static org.apache.paimon.catalog.CatalogUtils.isSystemDatabase;
 import static org.apache.paimon.catalog.CatalogUtils.listPartitionsFromFileSystem;
+import static org.apache.paimon.catalog.CatalogUtils.validateAutoCreateClose;
 import static org.apache.paimon.options.CatalogOptions.LOCK_ENABLED;
 import static org.apache.paimon.options.CatalogOptions.LOCK_TYPE;
 import static org.apache.paimon.utils.BranchManager.DEFAULT_MAIN_BRANCH;
 import static org.apache.paimon.utils.Preconditions.checkArgument;
-import static org.apache.paimon.utils.Preconditions.checkNotNull;
 
 /** Common implementation of {@link Catalog}. */
 public abstract class AbstractCatalog implements Catalog {
 
+    private static final Logger LOG = LoggerFactory.getLogger(AbstractCatalog.class);
+
     protected final FileIO fileIO;
     protected final Map<String, String> tableDefaultOptions;
     protected final Options catalogOptions;
@@ -90,11 +97,16 @@ public abstract class AbstractCatalog implements Catalog {
         return catalogOptions.toMap();
     }
 
-    @Override
+    public abstract String warehouse();
+
     public FileIO fileIO() {
         return fileIO;
     }
 
+    protected FileIO fileIO(Path path) {
+        return fileIO;
+    }
+
     public Optional<CatalogLockFactory> lockFactory() {
         if (!lockEnabled()) {
             return Optional.empty();
@@ -126,6 +138,11 @@ public abstract class AbstractCatalog implements Catalog {
         return false;
     }
 
+    @Override
+    public PagedList<String> listDatabasesPaged(Integer maxResults, String pageToken) {
+        return new PagedList<>(listDatabases(), null);
+    }
+
     @Override
     public void createDatabase(String name, boolean ignoreIfExists, Map<String, String> properties)
             throws DatabaseAlreadyExistException {
@@ -152,52 +169,21 @@ public abstract class AbstractCatalog implements Catalog {
     protected abstract Database getDatabaseImpl(String name) throws DatabaseNotExistException;
 
     @Override
-    public void createPartition(Identifier identifier, Map<String, String> partitionSpec)
-            throws TableNotExistException {
-        Identifier tableIdentifier =
-                Identifier.create(identifier.getDatabaseName(), identifier.getTableName());
-        FileStoreTable table = (FileStoreTable) getTable(tableIdentifier);
-
-        if (table.partitionKeys().isEmpty() || !table.coreOptions().partitionedTableInMetastore()) {
-            throw new UnsupportedOperationException(
-                    "The table is not partitioned table in metastore.");
-        }
-
-        MetastoreClient.Factory metastoreFactory =
-                table.catalogEnvironment().metastoreClientFactory();
-        if (metastoreFactory == null) {
-            throw new UnsupportedOperationException(
-                    "The catalog must have metastore to create partition.");
-        }
-
-        try (MetastoreClient client = metastoreFactory.create()) {
-            client.addPartition(new LinkedHashMap<>(partitionSpec));
-        } catch (Exception e) {
-            throw new RuntimeException(e);
-        }
-    }
-
-    @Override
-    public void dropPartition(Identifier identifier, Map<String, String> partitionSpec)
-            throws TableNotExistException {
-        checkNotSystemTable(identifier, "dropPartition");
-        Table table = getTable(identifier);
-        FileStoreTable fileStoreTable = (FileStoreTable) table;
-        try (FileStoreCommit commit =
-                fileStoreTable
-                        .store()
-                        .newCommit(
-                                createCommitUser(fileStoreTable.coreOptions().toConfiguration()))) {
-            commit.dropPartitions(
-                    Collections.singletonList(partitionSpec), BatchWriteBuilder.COMMIT_IDENTIFIER);
-        }
-    }
+    public void markDonePartitions(Identifier identifier, List<Map<String, String>> partitions)
+            throws TableNotExistException {}
 
     @Override
     public List<Partition> listPartitions(Identifier identifier) throws TableNotExistException {
         return listPartitionsFromFileSystem(getTable(identifier));
     }
 
+    @Override
+    public PagedList<Partition> listPartitionsPaged(
+            Identifier identifier, Integer maxResults, String pageToken)
+            throws TableNotExistException {
+        return new PagedList<>(listPartitions(identifier), null);
+    }
+
     protected abstract void createDatabaseImpl(String name, Map<String, String> properties);
 
     @Override
@@ -230,7 +216,7 @@ public abstract class AbstractCatalog implements Catalog {
             if (changes == null || changes.isEmpty()) {
                 return;
             }
-            alterDatabaseImpl(name, changes);
+            this.alterDatabaseImpl(name, changes);
         } catch (DatabaseNotExistException e) {
             if (ignoreIfNotExists) {
                 return;
@@ -254,16 +240,82 @@ public abstract class AbstractCatalog implements Catalog {
         return listTablesImpl(databaseName).stream().sorted().collect(Collectors.toList());
     }
 
+    @Override
+    public PagedList<String> listTablesPaged(
+            String databaseName, Integer maxResults, String pageToken)
+            throws DatabaseNotExistException {
+        return new PagedList<>(listTables(databaseName), null);
+    }
+
+    @Override
+    public PagedList<Table> listTableDetailsPaged(
+            String databaseName, Integer maxResults, String pageToken)
+            throws DatabaseNotExistException {
+        if (isSystemDatabase(databaseName)) {
+            List<Table> systemTables =
+                    SystemTableLoader.loadGlobalTableNames().stream()
+                            .map(
+                                    tableName -> {
+                                        try {
+                                            return getTable(
+                                                    Identifier.create(databaseName, tableName));
+                                        } catch (TableNotExistException ignored) {
+                                            LOG.warn(
+                                                    "system table {}.{} does not exist",
+                                                    databaseName,
+                                                    tableName);
+                                            return null;
+                                        }
+                                    })
+                            .filter(Objects::nonNull)
+                            .collect(Collectors.toList());
+            return new PagedList<>(systemTables, null);
+        }
+
+        // check db exists
+        getDatabase(databaseName);
+
+        return listTableDetailsPagedImpl(databaseName, maxResults, pageToken);
+    }
+
     protected abstract List<String> listTablesImpl(String databaseName);
 
+    protected PagedList<Table> listTableDetailsPagedImpl(
+            String databaseName, Integer maxResults, String pageToken)
+            throws DatabaseNotExistException {
+        PagedList<String> pagedTableNames = listTablesPaged(databaseName, maxResults, pageToken);
+        return new PagedList<>(
+                pagedTableNames.getElements().stream()
+                        .map(
+                                tableName -> {
+                                    try {
+                                        return getTable(Identifier.create(databaseName, tableName));
+                                    } catch (TableNotExistException ignored) {
+                                        LOG.warn(
+                                                "table {}.{} does not exist",
+                                                databaseName,
+                                                tableName);
+                                        return null;
+                                    }
+                                })
+                        .filter(Objects::nonNull)
+                        .collect(Collectors.toList()),
+                pagedTableNames.getNextPageToken());
+    }
+
     @Override
     public void dropTable(Identifier identifier, boolean ignoreIfNotExists)
             throws TableNotExistException {
         checkNotBranch(identifier, "dropTable");
         checkNotSystemTable(identifier, "dropTable");
 
+        List<Path> externalPaths = new ArrayList<>();
         try {
-            getTable(identifier);
+            Table table = getTable(identifier);
+            if (table instanceof FileStoreTable) {
+                FileStoreTable fileStoreTable = (FileStoreTable) table;
+                externalPaths = fileStoreTable.store().pathFactory().getExternalPaths();
+            }
         } catch (TableNotExistException e) {
             if (ignoreIfNotExists) {
                 return;
@@ -271,10 +323,10 @@ public abstract class AbstractCatalog implements Catalog {
             throw new TableNotExistException(identifier);
         }
 
-        dropTableImpl(identifier);
+        dropTableImpl(identifier, externalPaths);
     }
 
-    protected abstract void dropTableImpl(Identifier identifier);
+    protected abstract void dropTableImpl(Identifier identifier, List<Path> externalPaths);
 
     @Override
     public void createTable(Identifier identifier, Schema schema, boolean ignoreIfExists)
@@ -322,7 +374,7 @@ public abstract class AbstractCatalog implements Catalog {
                 ObjectTable.SCHEMA,
                 rowType);
         checkArgument(
-                schema.options().containsKey(CoreOptions.OBJECT_LOCATION.key()),
+                schema.options().containsKey(OBJECT_LOCATION.key()),
                 "Object table should have object-location option.");
         createTableImpl(identifier, schema.copy(ObjectTable.SCHEMA));
     }
@@ -380,78 +432,82 @@ public abstract class AbstractCatalog implements Catalog {
 
     @Override
     public Table getTable(Identifier identifier) throws TableNotExistException {
-        if (isSystemDatabase(identifier.getDatabaseName())) {
-            String tableName = identifier.getTableName();
-            Table table =
-                    SystemTableLoader.loadGlobal(
-                            tableName, fileIO, this::allTablePaths, catalogOptions);
-            if (table == null) {
-                throw new TableNotExistException(identifier);
-            }
-            return table;
-        } else if (identifier.isSystemTable()) {
-            Table originTable =
-                    getDataOrFormatTable(
-                            new Identifier(
-                                    identifier.getDatabaseName(),
-                                    identifier.getTableName(),
-                                    identifier.getBranchName(),
-                                    null));
-            if (!(originTable instanceof FileStoreTable)) {
-                throw new UnsupportedOperationException(
-                        String.format(
-                                "Only data table support system tables, but this table %s is %s.",
-                                identifier, originTable.getClass()));
-            }
-            Table table =
-                    SystemTableLoader.load(
-                            Preconditions.checkNotNull(identifier.getSystemTableName()),
-                            (FileStoreTable) originTable);
-            if (table == null) {
-                throw new TableNotExistException(identifier);
-            }
-            return table;
-        } else {
-            return getDataOrFormatTable(identifier);
+        return CatalogUtils.loadTable(
+                this,
+                identifier,
+                p -> fileIO(),
+                this::fileIO,
+                this::loadTableMetadata,
+                lockFactory().orElse(null),
+                lockContext().orElse(null));
+    }
+
+    @Override
+    public void createBranch(Identifier identifier, String branch, @Nullable String fromTag)
+            throws TableNotExistException, BranchAlreadyExistException, TagNotExistException {
+        throw new UnsupportedOperationException();
+    }
+
+    @Override
+    public void dropBranch(Identifier identifier, String branch) throws BranchNotExistException {
+        throw new UnsupportedOperationException();
+    }
+
+    @Override
+    public void fastForward(Identifier identifier, String branch) throws BranchNotExistException {
+        throw new UnsupportedOperationException();
+    }
+
+    @Override
+    public List<String> listBranches(Identifier identifier) throws TableNotExistException {
+        throw new UnsupportedOperationException();
+    }
+
+    @Override
+    public boolean commitSnapshot(
+            Identifier identifier, Snapshot snapshot, List<PartitionStatistics> statistics) {
+        throw new UnsupportedOperationException();
+    }
+
+    @Override
+    public Optional<TableSnapshot> loadSnapshot(Identifier identifier) {
+        throw new UnsupportedOperationException();
+    }
+
+    @Override
+    public void rollbackTo(Identifier identifier, Instant instant)
+            throws Catalog.TableNotExistException {
+        throw new UnsupportedOperationException();
+    }
+
+    @Override
+    public boolean supportsListObjectsPaged() {
+        return false;
+    }
+
+    @Override
+    public boolean supportsVersionManagement() {
+        return false;
+    }
+
+    @Override
+    public void createPartitions(Identifier identifier, List<Map<String, String>> partitions)
+            throws TableNotExistException {}
+
+    @Override
+    public void dropPartitions(Identifier identifier, List<Map<String, String>> partitions)
+            throws TableNotExistException {
+        Table table = getTable(identifier);
+        try (BatchTableCommit commit = table.newBatchWriteBuilder().newCommit()) {
+            commit.truncatePartitions(partitions);
+        } catch (Exception e) {
+            throw new RuntimeException(e);
         }
     }
 
-    protected Table getDataOrFormatTable(Identifier identifier) throws TableNotExistException {
-        Preconditions.checkArgument(identifier.getSystemTableName() == null);
-        TableMeta tableMeta = getDataTableMeta(identifier);
-        FileStoreTable table =
-                FileStoreTableFactory.create(
-                        fileIO,
-                        getTableLocation(identifier),
-                        tableMeta.schema,
-                        new CatalogEnvironment(
-                                identifier,
-                                tableMeta.uuid,
-                                Lock.factory(
-                                        lockFactory().orElse(null),
-                                        lockContext().orElse(null),
-                                        identifier),
-                                metastoreClientFactory(identifier).orElse(null)));
-        CoreOptions options = table.coreOptions();
-        if (options.type() == TableType.OBJECT_TABLE) {
-            String objectLocation = options.objectLocation();
-            checkNotNull(objectLocation, "Object location should not be null for object table.");
-            table =
-                    ObjectTable.builder()
-                            .underlyingTable(table)
-                            .objectLocation(objectLocation)
-                            .objectFileIO(objectFileIO(objectLocation))
-                            .build();
-        }
-        return table;
-    }
-
-    /**
-     * Catalog implementation may override this method to provide {@link FileIO} to object table.
-     */
-    protected FileIO objectFileIO(String objectLocation) {
-        return fileIO;
-    }
+    @Override
+    public void alterPartitions(Identifier identifier, List<PartitionStatistics> partitions)
+            throws TableNotExistException {}
 
     /**
      * Create a {@link FormatTable} identified by the given {@link Identifier}.
@@ -475,48 +531,17 @@ public abstract class AbstractCatalog implements Catalog {
         return newDatabasePath(warehouse(), database);
     }
 
-    public Map<String, Map<String, Path>> allTablePaths() {
-        try {
-            Map<String, Map<String, Path>> allPaths = new HashMap<>();
-            for (String database : listDatabases()) {
-                Map<String, Path> tableMap =
-                        allPaths.computeIfAbsent(database, d -> new HashMap<>());
-                for (String table : listTables(database)) {
-                    tableMap.put(table, getTableLocation(Identifier.create(database, table)));
-                }
-            }
-            return allPaths;
-        } catch (DatabaseNotExistException e) {
-            throw new RuntimeException("Database is deleted while listing", e);
-        }
+    protected TableMetadata loadTableMetadata(Identifier identifier) throws TableNotExistException {
+        return new TableMetadata(loadTableSchema(identifier), false, null);
     }
 
-    protected TableMeta getDataTableMeta(Identifier identifier) throws TableNotExistException {
-        return new TableMeta(getDataTableSchema(identifier), null);
-    }
-
-    protected abstract TableSchema getDataTableSchema(Identifier identifier)
+    protected abstract TableSchema loadTableSchema(Identifier identifier)
             throws TableNotExistException;
 
-    /** Get metastore client factory for the table specified by {@code identifier}. */
-    public Optional<MetastoreClient.Factory> metastoreClientFactory(Identifier identifier) {
-        return Optional.empty();
-    }
-
     public Path getTableLocation(Identifier identifier) {
         return new Path(newDatabasePath(identifier.getDatabaseName()), identifier.getTableName());
     }
 
-    protected static void checkNotBranch(Identifier identifier, String method) {
-        if (identifier.getBranchName() != null) {
-            throw new IllegalArgumentException(
-                    String.format(
-                            "Cannot '%s' for branch table '%s', "
-                                    + "please modify the table with the default branch.",
-                            method, identifier));
-        }
-    }
-
     protected void assertMainBranch(Identifier identifier) {
         if (identifier.getBranchName() != null
                 && !DEFAULT_MAIN_BRANCH.equals(identifier.getBranchName())) {
@@ -525,23 +550,6 @@ public abstract class AbstractCatalog implements Catalog {
         }
     }
 
-    protected static boolean isTableInSystemDatabase(Identifier identifier) {
-        return isSystemDatabase(identifier.getDatabaseName()) || identifier.isSystemTable();
-    }
-
-    protected static void checkNotSystemTable(Identifier identifier, String method) {
-        if (isTableInSystemDatabase(identifier)) {
-            throw new IllegalArgumentException(
-                    String.format(
-                            "Cannot '%s' for system table '%s', please use data table.",
-                            method, identifier));
-        }
-    }
-
-    private void copyTableDefaultOptions(Map<String, String> options) {
-        tableDefaultOptions.forEach(options::putIfAbsent);
-    }
-
     public static Path newTableLocation(String warehouse, Identifier identifier) {
         checkNotBranch(identifier, "newTableLocation");
         checkNotSystemTable(identifier, "newTableLocation");
@@ -554,26 +562,8 @@ public abstract class AbstractCatalog implements Catalog {
         return new Path(warehouse, database + DB_SUFFIX);
     }
 
-    public static boolean isSystemDatabase(String database) {
-        return SYSTEM_DATABASE_NAME.equals(database);
-    }
-
-    /** Validate database cannot be a system database. */
-    protected void checkNotSystemDatabase(String database) {
-        if (isSystemDatabase(database)) {
-            throw new ProcessSystemDatabaseException();
-        }
-    }
-
-    private void validateAutoCreateClose(Map<String, String> options) {
-        checkArgument(
-                !Boolean.parseBoolean(
-                        options.getOrDefault(
-                                CoreOptions.AUTO_CREATE.key(),
-                                CoreOptions.AUTO_CREATE.defaultValue().toString())),
-                String.format(
-                        "The value of %s property should be %s.",
-                        CoreOptions.AUTO_CREATE.key(), Boolean.FALSE));
+    private void copyTableDefaultOptions(Map<String, String> options) {
+        tableDefaultOptions.forEach(options::putIfAbsent);
     }
 
     private void validateCustomTablePath(Map<String, String> options) {
@@ -623,38 +613,17 @@ public abstract class AbstractCatalog implements Catalog {
     }
 
     public Optional<TableSchema> tableSchemaInFileSystem(Path tablePath, String branchName) {
-        return new SchemaManager(fileIO, tablePath, branchName)
-                .latest()
-                .map(
-                        s -> {
-                            if (!DEFAULT_MAIN_BRANCH.equals(branchName)) {
+        Optional<TableSchema> schema = new SchemaManager(fileIO, tablePath, branchName).latest();
+        if (!DEFAULT_MAIN_BRANCH.equals(branchName)) {
+            schema =
+                    schema.map(
+                            s -> {
                                 Options branchOptions = new Options(s.options());
                                 branchOptions.set(CoreOptions.BRANCH, branchName);
                                 return s.copy(branchOptions.toMap());
-                            } else {
-                                return s;
-                            }
-                        });
-    }
-
-    /** Table metadata. */
-    protected static class TableMeta {
-
-        private final TableSchema schema;
-        @Nullable private final String uuid;
-
-        public TableMeta(TableSchema schema, @Nullable String uuid) {
-            this.schema = schema;
-            this.uuid = uuid;
-        }
-
-        public TableSchema schema() {
-            return schema;
-        }
-
-        @Nullable
-        public String uuid() {
-            return uuid;
+                            });
         }
+        schema.ifPresent(s -> s.options().put(PATH.key(), tablePath.toString()));
+        return schema;
     }
 }
