@@ -74,7 +74,7 @@ public abstract class AbstractFileStoreWrite<T> implements FileStoreWrite<T> {
     private final int writerNumberMax;
     @Nullable private final IndexMaintainer.Factory<T> indexFactory;
     @Nullable private final DeletionVectorsMaintainer.Factory dvMaintainerFactory;
-    private final int totalBuckets;
+    private final int numBuckets;
     private final RowType partitionType;
 
     @Nullable protected IOManager ioManager;
@@ -84,12 +84,13 @@ public abstract class AbstractFileStoreWrite<T> implements FileStoreWrite<T> {
     private ExecutorService lazyCompactExecutor;
     private boolean closeCompactExecutorWhenLeaving = true;
     private boolean ignorePreviousFiles = false;
+    private boolean ignoreNumBucketCheck = false;
     protected boolean isStreamingMode = false;
 
     protected CompactionMetrics compactionMetrics = null;
     protected final String tableName;
     private boolean isInsertOnly;
-    private boolean legacyPartitionName;
+    private final boolean legacyPartitionName;
 
     protected AbstractFileStoreWrite(
             SnapshotManager snapshotManager,
@@ -98,10 +99,7 @@ public abstract class AbstractFileStoreWrite<T> implements FileStoreWrite<T> {
             @Nullable DeletionVectorsMaintainer.Factory dvMaintainerFactory,
             String tableName,
             CoreOptions options,
-            int totalBuckets,
-            RowType partitionType,
-            int writerNumberMax,
-            boolean legacyPartitionName) {
+            RowType partitionType) {
         this.snapshotManager = snapshotManager;
         this.scan = scan;
         // Statistic is useless in writer
@@ -112,12 +110,12 @@ public abstract class AbstractFileStoreWrite<T> implements FileStoreWrite<T> {
         }
         this.indexFactory = indexFactory;
         this.dvMaintainerFactory = dvMaintainerFactory;
-        this.totalBuckets = totalBuckets;
+        this.numBuckets = options.bucket();
         this.partitionType = partitionType;
         this.writers = new HashMap<>();
         this.tableName = tableName;
-        this.writerNumberMax = writerNumberMax;
-        this.legacyPartitionName = legacyPartitionName;
+        this.writerNumberMax = options.writeMaxWritersToSpill();
+        this.legacyPartitionName = options.legacyPartitionName();
     }
 
     @Override
@@ -136,6 +134,11 @@ public abstract class AbstractFileStoreWrite<T> implements FileStoreWrite<T> {
         this.ignorePreviousFiles = ignorePreviousFiles;
     }
 
+    @Override
+    public void withIgnoreNumBucketCheck(boolean ignoreNumBucketCheck) {
+        this.ignoreNumBucketCheck = ignoreNumBucketCheck;
+    }
+
     @Override
     public void withCompactExecutor(ExecutorService compactExecutor) {
         this.lazyCompactExecutor = compactExecutor;
@@ -228,6 +231,7 @@ public abstract class AbstractFileStoreWrite<T> implements FileStoreWrite<T> {
                         new CommitMessageImpl(
                                 partition,
                                 bucket,
+                                writerContainer.totalBuckets,
                                 increment.newFilesIncrement(),
                                 increment.compactIncrement(),
                                 new IndexIncrement(newIndexFiles));
@@ -290,9 +294,11 @@ public abstract class AbstractFileStoreWrite<T> implements FileStoreWrite<T> {
         //
         // Condition 2: No compaction is in progress. That is, no more changelog will be
         // produced.
+        //
+        // Condition 3: The writer has no postponed compaction like gentle lookup compaction.
         return writerContainer ->
                 writerContainer.lastModifiedCommitIdentifier < latestCommittedIdentifier
-                        && !writerContainer.writer.isCompacting();
+                        && !writerContainer.writer.compactNotCompleted();
     }
 
     protected static <T>
@@ -346,6 +352,7 @@ public abstract class AbstractFileStoreWrite<T> implements FileStoreWrite<T> {
                         new State<>(
                                 partition,
                                 bucket,
+                                writerContainer.totalBuckets,
                                 writerContainer.baseSnapshotId,
                                 writerContainer.lastModifiedCommitIdentifier,
                                 dataFiles,
@@ -367,7 +374,6 @@ public abstract class AbstractFileStoreWrite<T> implements FileStoreWrite<T> {
         for (State<T> state : states) {
             RecordWriter<T> writer =
                     createWriter(
-                            state.baseSnapshotId,
                             state.partition,
                             state.bucket,
                             state.dataFiles,
@@ -379,6 +385,7 @@ public abstract class AbstractFileStoreWrite<T> implements FileStoreWrite<T> {
             WriterContainer<T> writerContainer =
                     new WriterContainer<>(
                             writer,
+                            state.totalBuckets,
                             state.indexMaintainer,
                             state.deletionVectorsMaintainer,
                             state.baseSnapshotId);
@@ -408,7 +415,7 @@ public abstract class AbstractFileStoreWrite<T> implements FileStoreWrite<T> {
     }
 
     private long writerNumber() {
-        return writers.values().stream().mapToLong(e -> e.values().size()).sum();
+        return writers.values().stream().mapToLong(Map::size).sum();
     }
 
     @VisibleForTesting
@@ -426,24 +433,27 @@ public abstract class AbstractFileStoreWrite<T> implements FileStoreWrite<T> {
             }
         }
 
-        Long latestSnapshotId = snapshotManager.latestSnapshotId();
+        Snapshot latestSnapshot = snapshotManager.latestSnapshot();
         List<DataFileMeta> restoreFiles = new ArrayList<>();
-        if (!ignorePreviousFiles && latestSnapshotId != null) {
-            restoreFiles = scanExistingFileMetas(latestSnapshotId, partition, bucket);
+        int totalBuckets;
+        if (!ignorePreviousFiles && latestSnapshot != null) {
+            totalBuckets = scanExistingFileMetas(latestSnapshot, partition, bucket, restoreFiles);
+        } else {
+            totalBuckets = getDefaultBucketNum(partition);
         }
+
         IndexMaintainer<T> indexMaintainer =
                 indexFactory == null
                         ? null
                         : indexFactory.createOrRestore(
-                                ignorePreviousFiles ? null : latestSnapshotId, partition, bucket);
+                                ignorePreviousFiles ? null : latestSnapshot, partition, bucket);
         DeletionVectorsMaintainer deletionVectorsMaintainer =
                 dvMaintainerFactory == null
                         ? null
                         : dvMaintainerFactory.createOrRestore(
-                                ignorePreviousFiles ? null : latestSnapshotId, partition, bucket);
+                                ignorePreviousFiles ? null : latestSnapshot, partition, bucket);
         RecordWriter<T> writer =
                 createWriter(
-                        latestSnapshotId,
                         partition.copy(),
                         bucket,
                         restoreFiles,
@@ -454,7 +464,11 @@ public abstract class AbstractFileStoreWrite<T> implements FileStoreWrite<T> {
         writer.withInsertOnly(isInsertOnly);
         notifyNewWriter(writer);
         return new WriterContainer<>(
-                writer, indexMaintainer, deletionVectorsMaintainer, latestSnapshotId);
+                writer,
+                totalBuckets,
+                indexMaintainer,
+                deletionVectorsMaintainer,
+                latestSnapshot == null ? null : latestSnapshot.id());
     }
 
     @Override
@@ -468,13 +482,16 @@ public abstract class AbstractFileStoreWrite<T> implements FileStoreWrite<T> {
         return this;
     }
 
-    private List<DataFileMeta> scanExistingFileMetas(
-            long snapshotId, BinaryRow partition, int bucket) {
-        List<DataFileMeta> existingFileMetas = new ArrayList<>();
+    private int scanExistingFileMetas(
+            Snapshot snapshot,
+            BinaryRow partition,
+            int bucket,
+            List<DataFileMeta> existingFileMetas) {
         List<ManifestEntry> files =
-                scan.withSnapshot(snapshotId).withPartitionBucket(partition, bucket).plan().files();
+                scan.withSnapshot(snapshot).withPartitionBucket(partition, bucket).plan().files();
+        int totalBuckets = getDefaultBucketNum(partition);
         for (ManifestEntry entry : files) {
-            if (entry.totalBuckets() != totalBuckets) {
+            if (!ignoreNumBucketCheck && entry.totalBuckets() != numBuckets) {
                 String partInfo =
                         partitionType.getFieldCount() > 0
                                 ? "partition "
@@ -488,11 +505,18 @@ public abstract class AbstractFileStoreWrite<T> implements FileStoreWrite<T> {
                         String.format(
                                 "Try to write %s with a new bucket num %d, but the previous bucket num is %d. "
                                         + "Please switch to batch mode, and perform INSERT OVERWRITE to rescale current data layout first.",
-                                partInfo, totalBuckets, entry.totalBuckets()));
+                                partInfo, numBuckets, entry.totalBuckets()));
             }
+            totalBuckets = entry.totalBuckets();
             existingFileMetas.add(entry.file());
         }
-        return existingFileMetas;
+        return totalBuckets;
+    }
+
+    // TODO see comments on FileStoreWrite#withIgnoreNumBucketCheck for what is needed to support
+    //  writing partitions with different buckets
+    public int getDefaultBucketNum(BinaryRow partition) {
+        return numBuckets;
     }
 
     private ExecutorService compactExecutor() {
@@ -513,7 +537,6 @@ public abstract class AbstractFileStoreWrite<T> implements FileStoreWrite<T> {
     protected void notifyNewWriter(RecordWriter<T> writer) {}
 
     protected abstract RecordWriter<T> createWriter(
-            @Nullable Long snapshotId,
             BinaryRow partition,
             int bucket,
             List<DataFileMeta> restoreFiles,
@@ -532,6 +555,7 @@ public abstract class AbstractFileStoreWrite<T> implements FileStoreWrite<T> {
     @VisibleForTesting
     public static class WriterContainer<T> {
         public final RecordWriter<T> writer;
+        public final int totalBuckets;
         @Nullable public final IndexMaintainer<T> indexMaintainer;
         @Nullable public final DeletionVectorsMaintainer deletionVectorsMaintainer;
         protected final long baseSnapshotId;
@@ -539,10 +563,12 @@ public abstract class AbstractFileStoreWrite<T> implements FileStoreWrite<T> {
 
         protected WriterContainer(
                 RecordWriter<T> writer,
+                int totalBuckets,
                 @Nullable IndexMaintainer<T> indexMaintainer,
                 @Nullable DeletionVectorsMaintainer deletionVectorsMaintainer,
                 Long baseSnapshotId) {
             this.writer = writer;
+            this.totalBuckets = totalBuckets;
             this.indexMaintainer = indexMaintainer;
             this.deletionVectorsMaintainer = deletionVectorsMaintainer;
             this.baseSnapshotId =
@@ -555,4 +581,9 @@ public abstract class AbstractFileStoreWrite<T> implements FileStoreWrite<T> {
     Map<BinaryRow, Map<Integer, WriterContainer<T>>> writers() {
         return writers;
     }
+
+    @VisibleForTesting
+    public CompactionMetrics compactionMetrics() {
+        return compactionMetrics;
+    }
 }
