@@ -21,10 +21,10 @@ package org.apache.paimon.operation;
 import org.apache.paimon.CoreOptions;
 import org.apache.paimon.Snapshot;
 import org.apache.paimon.annotation.VisibleForTesting;
+import org.apache.paimon.catalog.SnapshotCommit;
 import org.apache.paimon.data.BinaryRow;
 import org.apache.paimon.data.InternalRow;
 import org.apache.paimon.fs.FileIO;
-import org.apache.paimon.fs.Path;
 import org.apache.paimon.io.DataFileMeta;
 import org.apache.paimon.io.DataFilePathFactory;
 import org.apache.paimon.manifest.FileEntry;
@@ -36,11 +36,13 @@ import org.apache.paimon.manifest.ManifestEntry;
 import org.apache.paimon.manifest.ManifestFile;
 import org.apache.paimon.manifest.ManifestFileMeta;
 import org.apache.paimon.manifest.ManifestList;
+import org.apache.paimon.manifest.PartitionEntry;
 import org.apache.paimon.manifest.SimpleFileEntry;
 import org.apache.paimon.operation.metrics.CommitMetrics;
 import org.apache.paimon.operation.metrics.CommitStats;
 import org.apache.paimon.options.MemorySize;
 import org.apache.paimon.partition.PartitionPredicate;
+import org.apache.paimon.partition.PartitionStatistics;
 import org.apache.paimon.predicate.Predicate;
 import org.apache.paimon.predicate.PredicateBuilder;
 import org.apache.paimon.schema.SchemaManager;
@@ -52,8 +54,10 @@ import org.apache.paimon.table.sink.CommitMessage;
 import org.apache.paimon.table.sink.CommitMessageImpl;
 import org.apache.paimon.table.source.ScanMode;
 import org.apache.paimon.types.RowType;
+import org.apache.paimon.utils.DataFilePathFactories;
 import org.apache.paimon.utils.FileStorePathFactory;
 import org.apache.paimon.utils.IOUtils;
+import org.apache.paimon.utils.InternalRowPartitionComputer;
 import org.apache.paimon.utils.Pair;
 import org.apache.paimon.utils.Preconditions;
 import org.apache.paimon.utils.SnapshotManager;
@@ -75,9 +79,9 @@ import java.util.Map;
 import java.util.Objects;
 import java.util.Optional;
 import java.util.Set;
-import java.util.concurrent.Callable;
 import java.util.stream.Collectors;
 
+import static java.util.Collections.emptyList;
 import static org.apache.paimon.deletionvectors.DeletionVectorsIndexFile.DELETION_VECTORS_INDEX;
 import static org.apache.paimon.index.HashIndexFile.HASH_INDEX;
 import static org.apache.paimon.manifest.ManifestEntry.recordCount;
@@ -85,7 +89,6 @@ import static org.apache.paimon.manifest.ManifestEntry.recordCountAdd;
 import static org.apache.paimon.manifest.ManifestEntry.recordCountDelete;
 import static org.apache.paimon.partition.PartitionPredicate.createBinaryPartitions;
 import static org.apache.paimon.partition.PartitionPredicate.createPartitionPredicate;
-import static org.apache.paimon.utils.BranchManager.DEFAULT_MAIN_BRANCH;
 import static org.apache.paimon.utils.InternalRowPartitionComputer.partToSimpleString;
 
 /**
@@ -94,12 +97,12 @@ import static org.apache.paimon.utils.InternalRowPartitionComputer.partToSimpleS
  * <p>This class provides an atomic commit method to the user.
  *
  * <ol>
- *   <li>Before calling {@link FileStoreCommitImpl#commit}, if user cannot determine if this commit
- *       is done before, user should first call {@link FileStoreCommitImpl#filterCommitted}.
+ *   <li>Before calling {@link #commit(ManifestCommittable, Map)}, if user cannot determine if this
+ *       commit is done before, user should first call {@link #filterCommitted}.
  *   <li>Before committing, it will first check for conflicts by checking if all files to be removed
  *       currently exists, and if modified files have overlapping key ranges with existing files.
- *   <li>After that it use the external {@link FileStoreCommitImpl#lock} (if provided) or the atomic
- *       rename of the file system to ensure atomicity.
+ *   <li>After that it use the external {@link SnapshotCommit} (if provided) or the atomic rename of
+ *       the file system to ensure atomicity.
  *   <li>If commit fails due to conflicts or exception it tries its best to clean up and aborts.
  *   <li>If atomic rename fails it tries again after reading the latest snapshot from step 2.
  * </ol>
@@ -112,6 +115,7 @@ public class FileStoreCommitImpl implements FileStoreCommit {
 
     private static final Logger LOG = LoggerFactory.getLogger(FileStoreCommitImpl.class);
 
+    private final SnapshotCommit snapshotCommit;
     private final FileIO fileIO;
     private final SchemaManager schemaManager;
     private final String tableName;
@@ -135,15 +139,16 @@ public class FileStoreCommitImpl implements FileStoreCommit {
     private final List<CommitCallback> commitCallbacks;
     private final StatsFileHandler statsFileHandler;
     private final BucketMode bucketMode;
-    private long commitTimeout;
+    private final long commitTimeout;
     private final int commitMaxRetries;
+    private final InternalRowPartitionComputer partitionComputer;
 
-    @Nullable private Lock lock;
     private boolean ignoreEmptyCommit;
     private CommitMetrics commitMetrics;
     @Nullable private PartitionExpire partitionExpire;
 
     public FileStoreCommitImpl(
+            SnapshotCommit snapshotCommit,
             FileIO fileIO,
             SchemaManager schemaManager,
             String tableName,
@@ -170,6 +175,7 @@ public class FileStoreCommitImpl implements FileStoreCommit {
             List<CommitCallback> commitCallbacks,
             int commitMaxRetries,
             long commitTimeout) {
+        this.snapshotCommit = snapshotCommit;
         this.fileIO = fileIO;
         this.schemaManager = schemaManager;
         this.tableName = tableName;
@@ -197,20 +203,19 @@ public class FileStoreCommitImpl implements FileStoreCommit {
         this.commitCallbacks = commitCallbacks;
         this.commitMaxRetries = commitMaxRetries;
         this.commitTimeout = commitTimeout;
+        this.partitionComputer =
+                new InternalRowPartitionComputer(
+                        options.partitionDefaultName(),
+                        partitionType,
+                        partitionType.getFieldNames().toArray(new String[0]),
+                        options.legacyPartitionName());
 
-        this.lock = null;
         this.ignoreEmptyCommit = true;
         this.commitMetrics = null;
         this.statsFileHandler = statsFileHandler;
         this.bucketMode = bucketMode;
     }
 
-    @Override
-    public FileStoreCommit withLock(Lock lock) {
-        this.lock = lock;
-        return this;
-    }
-
     @Override
     public FileStoreCommit ignoreEmptyCommit(boolean ignoreEmptyCommit) {
         this.ignoreEmptyCommit = ignoreEmptyCommit;
@@ -324,7 +329,6 @@ public class FileStoreCommitImpl implements FileStoreCommit {
                                 committable.logOffsets(),
                                 Snapshot.CommitKind.APPEND,
                                 noConflictCheck(),
-                                branchName,
                                 null);
                 generatedSnapshot += 1;
             }
@@ -359,7 +363,6 @@ public class FileStoreCommitImpl implements FileStoreCommit {
                                 committable.logOffsets(),
                                 Snapshot.CommitKind.COMPACT,
                                 hasConflictChecked(safeLatestSnapshotId),
-                                branchName,
                                 null);
                 generatedSnapshot += 1;
             }
@@ -498,14 +501,13 @@ public class FileStoreCommitImpl implements FileStoreCommit {
                 attempts +=
                         tryCommit(
                                 compactTableFiles,
-                                Collections.emptyList(),
+                                emptyList(),
                                 compactDvIndexFiles,
                                 committable.identifier(),
                                 committable.watermark(),
                                 committable.logOffsets(),
                                 Snapshot.CommitKind.COMPACT,
                                 mustConflictCheck(),
-                                branchName,
                                 null);
                 generatedSnapshot += 1;
             }
@@ -514,9 +516,9 @@ public class FileStoreCommitImpl implements FileStoreCommit {
             if (this.commitMetrics != null) {
                 reportCommit(
                         appendTableFiles,
-                        Collections.emptyList(),
+                        emptyList(),
                         compactTableFiles,
-                        Collections.emptyList(),
+                        emptyList(),
                         commitDuration,
                         generatedSnapshot,
                         attempts);
@@ -556,35 +558,19 @@ public class FileStoreCommitImpl implements FileStoreCommit {
         }
 
         tryOverwrite(
-                partitionFilter,
-                Collections.emptyList(),
-                Collections.emptyList(),
-                commitIdentifier,
-                null,
-                new HashMap<>());
+                partitionFilter, emptyList(), emptyList(), commitIdentifier, null, new HashMap<>());
     }
 
     @Override
     public void truncateTable(long commitIdentifier) {
-        tryOverwrite(
-                null,
-                Collections.emptyList(),
-                Collections.emptyList(),
-                commitIdentifier,
-                null,
-                new HashMap<>());
+        tryOverwrite(null, emptyList(), emptyList(), commitIdentifier, null, new HashMap<>());
     }
 
     @Override
     public void abort(List<CommitMessage> commitMessages) {
-        Map<Pair<BinaryRow, Integer>, DataFilePathFactory> factoryMap = new HashMap<>();
+        DataFilePathFactories factories = new DataFilePathFactories(pathFactory);
         for (CommitMessage message : commitMessages) {
-            DataFilePathFactory pathFactory =
-                    factoryMap.computeIfAbsent(
-                            Pair.of(message.partition(), message.bucket()),
-                            k ->
-                                    this.pathFactory.createDataFilePathFactory(
-                                            k.getKey(), k.getValue()));
+            DataFilePathFactory pathFactory = factories.get(message.partition(), message.bucket());
             CommitMessageImpl commitMessage = (CommitMessageImpl) message;
             List<DataFileMeta> toDelete = new ArrayList<>();
             toDelete.addAll(commitMessage.newFilesIncrement().newFiles());
@@ -608,15 +594,14 @@ public class FileStoreCommitImpl implements FileStoreCommit {
     public void commitStatistics(Statistics stats, long commitIdentifier) {
         String statsFileName = statsFileHandler.writeStats(stats);
         tryCommit(
-                Collections.emptyList(),
-                Collections.emptyList(),
-                Collections.emptyList(),
+                emptyList(),
+                emptyList(),
+                emptyList(),
                 commitIdentifier,
                 null,
                 Collections.emptyMap(),
                 Snapshot.CommitKind.ANALYZE,
                 noConflictCheck(),
-                branchName,
                 statsFileName);
     }
 
@@ -719,8 +704,13 @@ public class FileStoreCommitImpl implements FileStoreCommit {
     }
 
     private ManifestEntry makeEntry(FileKind kind, CommitMessage commitMessage, DataFileMeta file) {
+        Integer totalBuckets = commitMessage.totalBuckets();
+        if (totalBuckets == null) {
+            totalBuckets = numBucket;
+        }
+
         return new ManifestEntry(
-                kind, commitMessage.partition(), commitMessage.bucket(), numBucket, file);
+                kind, commitMessage.partition(), commitMessage.bucket(), totalBuckets, file);
     }
 
     private int tryCommit(
@@ -732,7 +722,6 @@ public class FileStoreCommitImpl implements FileStoreCommit {
             Map<Integer, Long> logOffsets,
             Snapshot.CommitKind commitKind,
             ConflictCheck conflictCheck,
-            String branchName,
             @Nullable String statsFileName) {
         int retryCount = 0;
         RetryResult retryResult = null;
@@ -751,7 +740,6 @@ public class FileStoreCommitImpl implements FileStoreCommit {
                             commitKind,
                             latestSnapshot,
                             conflictCheck,
-                            branchName,
                             statsFileName);
 
             if (result.isSuccess()) {
@@ -786,12 +774,14 @@ public class FileStoreCommitImpl implements FileStoreCommit {
         List<ManifestEntry> changesWithOverwrite = new ArrayList<>();
         List<IndexManifestEntry> indexChangesWithOverwrite = new ArrayList<>();
         if (latestSnapshot != null) {
-            List<ManifestEntry> currentEntries =
-                    scan.withSnapshot(latestSnapshot)
-                            .withPartitionFilter(partitionFilter)
-                            .withKind(ScanMode.ALL)
-                            .plan()
-                            .files();
+            scan.withSnapshot(latestSnapshot)
+                    .withPartitionFilter(partitionFilter)
+                    .withKind(ScanMode.ALL);
+            if (numBucket != BucketMode.POSTPONE_BUCKET) {
+                // bucket = -2 can only be overwritten in postpone bucket tables
+                scan.withBucketFilter(bucket -> bucket >= 0);
+            }
+            List<ManifestEntry> currentEntries = scan.plan().files();
             for (ManifestEntry entry : currentEntries) {
                 changesWithOverwrite.add(
                         new ManifestEntry(
@@ -818,14 +808,13 @@ public class FileStoreCommitImpl implements FileStoreCommit {
 
         return tryCommit(
                 changesWithOverwrite,
-                Collections.emptyList(),
+                emptyList(),
                 indexChangesWithOverwrite,
                 identifier,
                 watermark,
                 logOffsets,
                 Snapshot.CommitKind.OVERWRITE,
                 mustConflictCheck(),
-                branchName,
                 null);
     }
 
@@ -841,15 +830,10 @@ public class FileStoreCommitImpl implements FileStoreCommit {
             Snapshot.CommitKind commitKind,
             @Nullable Snapshot latestSnapshot,
             ConflictCheck conflictCheck,
-            String branchName,
             @Nullable String newStatsFileName) {
         long startMillis = System.currentTimeMillis();
         long newSnapshotId =
                 latestSnapshot == null ? Snapshot.FIRST_SNAPSHOT_ID : latestSnapshot.id() + 1;
-        Path newSnapshotPath =
-                branchName.equals(DEFAULT_MAIN_BRANCH)
-                        ? snapshotManager.snapshotPath(newSnapshotId)
-                        : snapshotManager.copyWithBranch(branchName).snapshotPath(newSnapshotId);
 
         if (LOG.isDebugEnabled()) {
             LOG.debug("Ready to commit table files to snapshot {}", newSnapshotId);
@@ -898,9 +882,10 @@ public class FileStoreCommitImpl implements FileStoreCommit {
         }
 
         Snapshot newSnapshot;
-        String baseManifestList = null;
-        String deltaManifestList = null;
-        String changelogManifestList = null;
+        Pair<String, Long> baseManifestList = null;
+        Pair<String, Long> deltaManifestList = null;
+        List<PartitionEntry> deltaStatistics;
+        Pair<String, Long> changelogManifestList = null;
         String oldIndexManifest = null;
         String indexManifest = null;
         List<ManifestFileMeta> mergeBeforeManifests = new ArrayList<>();
@@ -946,6 +931,7 @@ public class FileStoreCommitImpl implements FileStoreCommit {
 
             boolean rewriteIndexManifest = true;
             if (retryResult != null) {
+                deltaStatistics = retryResult.deltaStatistics;
                 deltaManifestList = retryResult.deltaManifestList;
                 changelogManifestList = retryResult.changelogManifestList;
                 if (Objects.equals(oldIndexManifest, retryResult.oldIndexManifest)) {
@@ -957,6 +943,7 @@ public class FileStoreCommitImpl implements FileStoreCommit {
                 }
             } else {
                 // write new delta files into manifest files
+                deltaStatistics = new ArrayList<>(PartitionEntry.merge(deltaFiles));
                 deltaManifestList = manifestList.write(manifestFile.write(deltaFiles));
 
                 // write changelog into manifest files
@@ -992,9 +979,12 @@ public class FileStoreCommitImpl implements FileStoreCommit {
                     new Snapshot(
                             newSnapshotId,
                             latestSchemaId,
-                            baseManifestList,
-                            deltaManifestList,
-                            changelogManifestList,
+                            baseManifestList.getLeft(),
+                            baseManifestList.getRight(),
+                            deltaManifestList.getKey(),
+                            deltaManifestList.getRight(),
+                            changelogManifestList == null ? null : changelogManifestList.getKey(),
+                            changelogManifestList == null ? null : changelogManifestList.getRight(),
                             indexManifest,
                             commitUser,
                             identifier,
@@ -1016,27 +1006,19 @@ public class FileStoreCommitImpl implements FileStoreCommit {
             cleanUpNoReuseTmpManifests(baseManifestList, mergeBeforeManifests, mergeAfterManifests);
             throw new RuntimeException(
                     String.format(
-                            "Exception occurs when preparing snapshot #%d (path %s) by user %s "
+                            "Exception occurs when preparing snapshot #%d by user %s "
                                     + "with hash %s and kind %s. Clean up.",
-                            newSnapshotId,
-                            newSnapshotPath.toString(),
-                            commitUser,
-                            identifier,
-                            commitKind.name()),
+                            newSnapshotId, commitUser, identifier, commitKind.name()),
                     e);
         }
 
-        if (commitSnapshotImpl(newSnapshot, newSnapshotPath)) {
+        if (commitSnapshotImpl(newSnapshot, deltaStatistics)) {
             if (LOG.isDebugEnabled()) {
                 LOG.debug(
                         String.format(
-                                "Successfully commit snapshot #%d (path %s) by user %s "
+                                "Successfully commit snapshot #%d by user %s "
                                         + "with identifier %s and kind %s.",
-                                newSnapshotId,
-                                newSnapshotPath,
-                                commitUser,
-                                identifier,
-                                commitKind.name()));
+                                newSnapshotId, commitUser, identifier, commitKind.name()));
             }
             commitCallbacks.forEach(callback -> callback.call(deltaFiles, newSnapshot));
             return new SuccessResult();
@@ -1046,17 +1028,13 @@ public class FileStoreCommitImpl implements FileStoreCommit {
         long commitTime = (System.currentTimeMillis() - startMillis) / 1000;
         LOG.warn(
                 String.format(
-                        "Atomic commit failed for snapshot #%d (path %s) by user %s "
+                        "Atomic commit failed for snapshot #%d by user %s "
                                 + "with identifier %s and kind %s after %s seconds. "
                                 + "Clean up and try again.",
-                        newSnapshotId,
-                        newSnapshotPath,
-                        commitUser,
-                        identifier,
-                        commitKind.name(),
-                        commitTime));
+                        newSnapshotId, commitUser, identifier, commitKind.name(), commitTime));
         cleanUpNoReuseTmpManifests(baseManifestList, mergeBeforeManifests, mergeAfterManifests);
         return new RetryResult(
+                deltaStatistics,
                 deltaManifestList,
                 changelogManifestList,
                 oldIndexManifest,
@@ -1140,16 +1118,19 @@ public class FileStoreCommitImpl implements FileStoreCommit {
             }
         }
 
-        String baseManifestList = manifestList.write(mergeAfterManifests);
-        String deltaManifestList = manifestList.write(Collections.emptyList());
+        Pair<String, Long> baseManifestList = manifestList.write(mergeAfterManifests);
+        Pair<String, Long> deltaManifestList = manifestList.write(emptyList());
 
         // prepare snapshot file
         Snapshot newSnapshot =
                 new Snapshot(
                         latestSnapshot.id() + 1,
                         latestSnapshot.schemaId(),
-                        baseManifestList,
-                        deltaManifestList,
+                        baseManifestList.getLeft(),
+                        baseManifestList.getRight(),
+                        deltaManifestList.getLeft(),
+                        deltaManifestList.getRight(),
+                        null,
                         null,
                         latestSnapshot.indexManifest(),
                         commitUser,
@@ -1163,12 +1144,7 @@ public class FileStoreCommitImpl implements FileStoreCommit {
                         latestSnapshot.watermark(),
                         latestSnapshot.statistics());
 
-        Path newSnapshotPath =
-                branchName.equals(DEFAULT_MAIN_BRANCH)
-                        ? snapshotManager.snapshotPath(newSnapshot.id())
-                        : snapshotManager.copyWithBranch(branchName).snapshotPath(newSnapshot.id());
-
-        if (!commitSnapshotImpl(newSnapshot, newSnapshotPath)) {
+        if (!commitSnapshotImpl(newSnapshot, emptyList())) {
             return new ManifestCompactResult(
                     baseManifestList, deltaManifestList, mergeBeforeManifests, mergeAfterManifests);
         } else {
@@ -1176,39 +1152,22 @@ public class FileStoreCommitImpl implements FileStoreCommit {
         }
     }
 
-    private boolean commitSnapshotImpl(Snapshot newSnapshot, Path newSnapshotPath) {
+    private boolean commitSnapshotImpl(Snapshot newSnapshot, List<PartitionEntry> deltaStatistics) {
         try {
-            Callable<Boolean> callable =
-                    () -> {
-                        boolean committed =
-                                fileIO.tryToWriteAtomic(newSnapshotPath, newSnapshot.toJson());
-                        if (committed) {
-                            snapshotManager.commitLatestHint(newSnapshot.id());
-                        }
-                        return committed;
-                    };
-            if (lock != null) {
-                return lock.runWithLock(
-                        () ->
-                                // fs.rename may not returns false if target file
-                                // already exists, or even not atomic
-                                // as we're relying on external locking, we can first
-                                // check if file exist then rename to work around this
-                                // case
-                                !fileIO.exists(newSnapshotPath) && callable.call());
-            } else {
-                return callable.call();
+            List<PartitionStatistics> statistics = new ArrayList<>(deltaStatistics.size());
+            for (PartitionEntry entry : deltaStatistics) {
+                statistics.add(entry.toPartitionStatistics(partitionComputer));
             }
+            return snapshotCommit.commit(newSnapshot, branchName, statistics);
         } catch (Throwable e) {
             // exception when performing the atomic rename,
             // we cannot clean up because we can't determine the success
             throw new RuntimeException(
                     String.format(
-                            "Exception occurs when committing snapshot #%d (path %s) by user %s "
+                            "Exception occurs when committing snapshot #%d by user %s "
                                     + "with identifier %s and kind %s. "
                                     + "Cannot clean up because we can't determine the success.",
                             newSnapshot.id(),
-                            newSnapshotPath,
                             commitUser,
                             newSnapshot.commitIdentifier(),
                             newSnapshot.commitKind().name()),
@@ -1455,11 +1414,11 @@ public class FileStoreCommitImpl implements FileStoreCommit {
     }
 
     private void cleanUpNoReuseTmpManifests(
-            String baseManifestList,
+            Pair<String, Long> baseManifestList,
             List<ManifestFileMeta> mergeBeforeManifests,
             List<ManifestFileMeta> mergeAfterManifests) {
         if (baseManifestList != null) {
-            manifestList.delete(baseManifestList);
+            manifestList.delete(baseManifestList.getKey());
         }
         Set<String> oldMetaSet =
                 mergeBeforeManifests.stream()
@@ -1473,22 +1432,22 @@ public class FileStoreCommitImpl implements FileStoreCommit {
     }
 
     private void cleanUpReuseTmpManifests(
-            String deltaManifestList,
-            String changelogManifestList,
+            Pair<String, Long> deltaManifestList,
+            Pair<String, Long> changelogManifestList,
             String oldIndexManifest,
             String newIndexManifest) {
         if (deltaManifestList != null) {
-            for (ManifestFileMeta manifest : manifestList.read(deltaManifestList)) {
+            for (ManifestFileMeta manifest : manifestList.read(deltaManifestList.getKey())) {
                 manifestFile.delete(manifest.fileName());
             }
-            manifestList.delete(deltaManifestList);
+            manifestList.delete(deltaManifestList.getKey());
         }
 
         if (changelogManifestList != null) {
-            for (ManifestFileMeta manifest : manifestList.read(changelogManifestList)) {
+            for (ManifestFileMeta manifest : manifestList.read(changelogManifestList.getKey())) {
                 manifestFile.delete(manifest.fileName());
             }
-            manifestList.delete(changelogManifestList);
+            manifestList.delete(changelogManifestList.getKey());
         }
 
         cleanIndexManifest(oldIndexManifest, newIndexManifest);
@@ -1505,6 +1464,7 @@ public class FileStoreCommitImpl implements FileStoreCommit {
         for (CommitCallback callback : commitCallbacks) {
             IOUtils.closeQuietly(callback);
         }
+        IOUtils.closeQuietly(snapshotCommit);
     }
 
     private static class LevelIdentifier {
@@ -1568,8 +1528,9 @@ public class FileStoreCommitImpl implements FileStoreCommit {
 
     private class RetryResult implements CommitResult {
 
-        private final String deltaManifestList;
-        private final String changelogManifestList;
+        private final List<PartitionEntry> deltaStatistics;
+        private final Pair<String, Long> deltaManifestList;
+        private final Pair<String, Long> changelogManifestList;
 
         private final String oldIndexManifest;
         private final String newIndexManifest;
@@ -1578,12 +1539,14 @@ public class FileStoreCommitImpl implements FileStoreCommit {
         private final List<SimpleFileEntry> baseDataFiles;
 
         private RetryResult(
-                String deltaManifestList,
-                String changelogManifestList,
+                List<PartitionEntry> deltaStatistics,
+                Pair<String, Long> deltaManifestList,
+                Pair<String, Long> changelogManifestList,
                 String oldIndexManifest,
                 String newIndexManifest,
                 Snapshot latestSnapshot,
                 List<SimpleFileEntry> baseDataFiles) {
+            this.deltaStatistics = deltaStatistics;
             this.deltaManifestList = deltaManifestList;
             this.changelogManifestList = changelogManifestList;
             this.oldIndexManifest = oldIndexManifest;
@@ -1605,14 +1568,14 @@ public class FileStoreCommitImpl implements FileStoreCommit {
 
     private class ManifestCompactResult implements CommitResult {
 
-        private final String baseManifestList;
-        private final String deltaManifestList;
+        private final Pair<String, Long> baseManifestList;
+        private final Pair<String, Long> deltaManifestList;
         private final List<ManifestFileMeta> mergeBeforeManifests;
         private final List<ManifestFileMeta> mergeAfterManifests;
 
         public ManifestCompactResult(
-                String baseManifestList,
-                String deltaManifestList,
+                Pair<String, Long> baseManifestList,
+                Pair<String, Long> deltaManifestList,
                 List<ManifestFileMeta> mergeBeforeManifests,
                 List<ManifestFileMeta> mergeAfterManifests) {
             this.baseManifestList = baseManifestList;
@@ -1622,7 +1585,7 @@ public class FileStoreCommitImpl implements FileStoreCommit {
         }
 
         public void cleanAll() {
-            manifestList.delete(deltaManifestList);
+            manifestList.delete(deltaManifestList.getKey());
             cleanUpNoReuseTmpManifests(baseManifestList, mergeBeforeManifests, mergeAfterManifests);
         }
 
