@@ -19,7 +19,6 @@
 package org.apache.paimon.operation;
 
 import org.apache.paimon.Snapshot;
-import org.apache.paimon.catalog.Catalog;
 import org.apache.paimon.data.Timestamp;
 import org.apache.paimon.fs.FileIO;
 import org.apache.paimon.fs.FileStatus;
@@ -32,11 +31,11 @@ import org.apache.paimon.manifest.ManifestList;
 import org.apache.paimon.schema.SchemaManager;
 import org.apache.paimon.table.FileStoreTable;
 import org.apache.paimon.utils.BranchManager;
+import org.apache.paimon.utils.ChangelogManager;
 import org.apache.paimon.utils.DateTimeUtils;
 import org.apache.paimon.utils.FileStorePathFactory;
 import org.apache.paimon.utils.Pair;
 import org.apache.paimon.utils.Preconditions;
-import org.apache.paimon.utils.SerializableConsumer;
 import org.apache.paimon.utils.SnapshotManager;
 import org.apache.paimon.utils.SupplierWithIOException;
 import org.apache.paimon.utils.TagManager;
@@ -59,9 +58,14 @@ import java.util.TimeZone;
 import java.util.concurrent.TimeUnit;
 import java.util.function.Consumer;
 import java.util.function.Predicate;
+import java.util.stream.Collectors;
 
 import static java.util.Collections.emptyList;
+import static org.apache.paimon.utils.ChangelogManager.CHANGELOG_PREFIX;
 import static org.apache.paimon.utils.FileStorePathFactory.BUCKET_PATH_PREFIX;
+import static org.apache.paimon.utils.HintFileUtils.EARLIEST;
+import static org.apache.paimon.utils.HintFileUtils.LATEST;
+import static org.apache.paimon.utils.SnapshotManager.SNAPSHOT_PREFIX;
 import static org.apache.paimon.utils.StringUtils.isNullOrWhitespaceOnly;
 
 /**
@@ -89,18 +93,17 @@ public abstract class OrphanFilesClean implements Serializable {
     protected final FileStoreTable table;
     protected final FileIO fileIO;
     protected final long olderThanMillis;
-    protected final SerializableConsumer<Path> fileCleaner;
+    protected final boolean dryRun;
     protected final int partitionKeysNum;
     protected final Path location;
 
-    public OrphanFilesClean(
-            FileStoreTable table, long olderThanMillis, SerializableConsumer<Path> fileCleaner) {
+    public OrphanFilesClean(FileStoreTable table, long olderThanMillis, boolean dryRun) {
         this.table = table;
         this.fileIO = table.fileIO();
         this.partitionKeysNum = table.partitionKeys().size();
         this.location = table.location();
         this.olderThanMillis = olderThanMillis;
-        this.fileCleaner = fileCleaner;
+        this.dryRun = dryRun;
     }
 
     protected List<String> validBranches() {
@@ -129,31 +132,92 @@ public abstract class OrphanFilesClean implements Serializable {
             Consumer<Path> deletedFilesConsumer,
             Consumer<Long> deletedFilesLenInBytesConsumer) {
         for (String branch : branches) {
-            FileStoreTable branchTable = table.switchToBranch(branch);
-            SnapshotManager snapshotManager = branchTable.snapshotManager();
-
-            // specially handle the snapshot directory
-            List<Pair<Path, Long>> nonSnapshotFiles =
-                    snapshotManager.tryGetNonSnapshotFiles(this::oldEnough);
-            nonSnapshotFiles.forEach(
-                    nonSnapshotFile ->
-                            cleanFile(
-                                    nonSnapshotFile,
-                                    deletedFilesConsumer,
-                                    deletedFilesLenInBytesConsumer));
-
-            // specially handle the changelog directory
-            List<Pair<Path, Long>> nonChangelogFiles =
-                    snapshotManager.tryGetNonChangelogFiles(this::oldEnough);
-            nonChangelogFiles.forEach(
-                    nonChangelogFile ->
-                            cleanFile(
-                                    nonChangelogFile,
-                                    deletedFilesConsumer,
-                                    deletedFilesLenInBytesConsumer));
+            cleanBranchSnapshotDir(branch, deletedFilesConsumer, deletedFilesLenInBytesConsumer);
         }
     }
 
+    protected void cleanBranchSnapshotDir(
+            String branch,
+            Consumer<Path> deletedFilesConsumer,
+            Consumer<Long> deletedFilesLenInBytesConsumer) {
+        LOG.info("Start to clean snapshot directory of branch {}.", branch);
+        FileStoreTable branchTable = table.switchToBranch(branch);
+        SnapshotManager snapshotManager = branchTable.snapshotManager();
+        ChangelogManager changelogManager = branchTable.changelogManager();
+
+        // specially handle the snapshot directory
+        List<Pair<Path, Long>> nonSnapshotFiles =
+                tryGetNonSnapshotFiles(snapshotManager.snapshotDirectory(), this::oldEnough);
+        nonSnapshotFiles.forEach(
+                nonSnapshotFile ->
+                        cleanFile(
+                                nonSnapshotFile,
+                                deletedFilesConsumer,
+                                deletedFilesLenInBytesConsumer));
+
+        // specially handle the changelog directory
+        List<Pair<Path, Long>> nonChangelogFiles =
+                tryGetNonChangelogFiles(changelogManager.changelogDirectory(), this::oldEnough);
+        nonChangelogFiles.forEach(
+                nonChangelogFile ->
+                        cleanFile(
+                                nonChangelogFile,
+                                deletedFilesConsumer,
+                                deletedFilesLenInBytesConsumer));
+        LOG.info("End to clean snapshot directory of branch {}.", branch);
+    }
+
+    private List<Pair<Path, Long>> tryGetNonSnapshotFiles(
+            Path snapshotDirectory, Predicate<FileStatus> fileStatusFilter) {
+        return listPathWithFilter(
+                fileIO, snapshotDirectory, fileStatusFilter, nonSnapshotFileFilter());
+    }
+
+    private List<Pair<Path, Long>> tryGetNonChangelogFiles(
+            Path changelogDirectory, Predicate<FileStatus> fileStatusFilter) {
+        return listPathWithFilter(
+                fileIO, changelogDirectory, fileStatusFilter, nonChangelogFileFilter());
+    }
+
+    private static List<Pair<Path, Long>> listPathWithFilter(
+            FileIO fileIO,
+            Path directory,
+            Predicate<FileStatus> fileStatusFilter,
+            Predicate<Path> fileFilter) {
+        try {
+            FileStatus[] statuses = fileIO.listStatus(directory);
+            if (statuses == null) {
+                return Collections.emptyList();
+            }
+
+            return Arrays.stream(statuses)
+                    .filter(fileStatusFilter)
+                    .filter(status -> fileFilter.test(status.getPath()))
+                    .map(status -> Pair.of(status.getPath(), status.getLen()))
+                    .collect(Collectors.toList());
+        } catch (IOException ignored) {
+            return Collections.emptyList();
+        }
+    }
+
+    private static Predicate<Path> nonSnapshotFileFilter() {
+        return path -> {
+            String name = path.getName();
+            return !name.startsWith(SNAPSHOT_PREFIX)
+                    && !name.equals(EARLIEST)
+                    && !name.equals(LATEST);
+        };
+    }
+
+    private static Predicate<Path> nonChangelogFileFilter() {
+        return path -> {
+            String name = path.getName();
+            return !name.startsWith(CHANGELOG_PREFIX)
+                    && !name.equals(EARLIEST)
+                    && !name.equals(LATEST);
+        };
+    }
+
     private void cleanFile(
             Pair<Path, Long> deleteFileInfo,
             Consumer<Path> deletedFilesConsumer,
@@ -162,16 +226,30 @@ public abstract class OrphanFilesClean implements Serializable {
         Long fileSize = deleteFileInfo.getRight();
         deletedFilesConsumer.accept(filePath);
         deletedFilesLenInBytesConsumer.accept(fileSize);
-        fileCleaner.accept(filePath);
+        cleanFile(filePath);
+    }
+
+    protected void cleanFile(Path path) {
+        if (!dryRun) {
+            try {
+                if (fileIO.isDir(path)) {
+                    fileIO.deleteDirectoryQuietly(path);
+                } else {
+                    fileIO.deleteQuietly(path);
+                }
+            } catch (IOException ignored) {
+            }
+        }
     }
 
     protected Set<Snapshot> safelyGetAllSnapshots(String branch) throws IOException {
         FileStoreTable branchTable = table.switchToBranch(branch);
         SnapshotManager snapshotManager = branchTable.snapshotManager();
+        ChangelogManager changelogManager = branchTable.changelogManager();
         TagManager tagManager = branchTable.tagManager();
         Set<Snapshot> readSnapshots = new HashSet<>(snapshotManager.safelyGetAllSnapshots());
         readSnapshots.addAll(tagManager.taggedSnapshots());
-        readSnapshots.addAll(snapshotManager.safelyGetAllChangelogs());
+        readSnapshots.addAll(changelogManager.safelyGetAllChangelogs());
         return readSnapshots;
     }
 
@@ -254,14 +332,44 @@ public abstract class OrphanFilesClean implements Serializable {
     /** List directories that contains data files and manifest files. */
     protected List<Path> listPaimonFileDirs() {
         FileStorePathFactory pathFactory = table.store().pathFactory();
+        return listPaimonFileDirs(
+                table.fullName(),
+                pathFactory.manifestPath().toString(),
+                pathFactory.indexPath().toString(),
+                pathFactory.statisticsPath().toString(),
+                pathFactory.dataFilePath().toString(),
+                partitionKeysNum,
+                table.store().options().dataFileExternalPaths());
+    }
 
+    protected List<Path> listPaimonFileDirs(
+            String tableName,
+            String manifestPath,
+            String indexPath,
+            String statisticsPath,
+            String dataFilePath,
+            int partitionKeysNum,
+            String dataFileExternalPaths) {
+        LOG.info("Start: listing paimon file directories for table [{}]", tableName);
+        long start = System.currentTimeMillis();
         List<Path> paimonFileDirs = new ArrayList<>();
 
-        paimonFileDirs.add(pathFactory.manifestPath());
-        paimonFileDirs.add(pathFactory.indexPath());
-        paimonFileDirs.add(pathFactory.statisticsPath());
-        paimonFileDirs.addAll(listFileDirs(pathFactory.dataFilePath(), partitionKeysNum));
+        paimonFileDirs.add(new Path(manifestPath));
+        paimonFileDirs.add(new Path(indexPath));
+        paimonFileDirs.add(new Path(statisticsPath));
+        paimonFileDirs.addAll(listFileDirs(new Path(dataFilePath), partitionKeysNum));
 
+        // add external data paths
+        if (dataFileExternalPaths != null) {
+            String[] externalPathArr = dataFileExternalPaths.split(",");
+            for (String externalPath : externalPathArr) {
+                paimonFileDirs.addAll(listFileDirs(new Path(externalPath), partitionKeysNum));
+            }
+        }
+        LOG.info(
+                "End list paimon file directories for table [{}] spend [{}] ms",
+                tableName,
+                System.currentTimeMillis() - start);
         return paimonFileDirs;
     }
 
@@ -353,28 +461,6 @@ public abstract class OrphanFilesClean implements Serializable {
         return status.getModificationTime() < olderThanMillis;
     }
 
-    public static SerializableConsumer<Path> createFileCleaner(
-            Catalog catalog, @Nullable Boolean dryRun) {
-        SerializableConsumer<Path> fileCleaner;
-        if (Boolean.TRUE.equals(dryRun)) {
-            fileCleaner = path -> {};
-        } else {
-            FileIO fileIO = catalog.fileIO();
-            fileCleaner =
-                    path -> {
-                        try {
-                            if (fileIO.isDir(path)) {
-                                fileIO.deleteDirectoryQuietly(path);
-                            } else {
-                                fileIO.deleteQuietly(path);
-                            }
-                        } catch (IOException ignored) {
-                        }
-                    };
-        }
-        return fileCleaner;
-    }
-
     public static long olderThanMillis(@Nullable String olderThan) {
         if (isNullOrWhitespaceOnly(olderThan)) {
             return System.currentTimeMillis() - TimeUnit.DAYS.toMillis(1);
@@ -390,4 +476,33 @@ public abstract class OrphanFilesClean implements Serializable {
             return parsedTimestampData.getMillisecond();
         }
     }
+
+    /** Try to clean empty data directories. */
+    protected void tryCleanDataDirectory(Set<Path> dataDirs, int maxLevel) {
+        for (int level = 0; level < maxLevel; level++) {
+            dataDirs =
+                    dataDirs.stream()
+                            .filter(this::tryDeleteEmptyDirectory)
+                            .map(Path::getParent)
+                            .collect(Collectors.toSet());
+        }
+    }
+
+    public boolean tryDeleteEmptyDirectory(Path path) {
+        if (dryRun) {
+            return false;
+        }
+
+        try {
+            return fileIO.delete(path, false);
+        } catch (IOException e) {
+            return false;
+        }
+    }
+
+    /** Cleaner to clean files. */
+    public interface FileCleaner extends Serializable {
+
+        void clean(String table, Path path);
+    }
 }
