@@ -25,7 +25,6 @@ import org.apache.paimon.catalog.Catalog;
 import org.apache.paimon.catalog.Identifier;
 import org.apache.paimon.fs.FileIO;
 import org.apache.paimon.fs.Path;
-import org.apache.paimon.operation.Lock;
 import org.apache.paimon.schema.SchemaChange.AddColumn;
 import org.apache.paimon.schema.SchemaChange.DropColumn;
 import org.apache.paimon.schema.SchemaChange.RemoveOption;
@@ -36,6 +35,7 @@ import org.apache.paimon.schema.SchemaChange.UpdateColumnNullability;
 import org.apache.paimon.schema.SchemaChange.UpdateColumnPosition;
 import org.apache.paimon.schema.SchemaChange.UpdateColumnType;
 import org.apache.paimon.schema.SchemaChange.UpdateComment;
+import org.apache.paimon.table.BucketMode;
 import org.apache.paimon.table.FileStoreTableFactory;
 import org.apache.paimon.types.ArrayType;
 import org.apache.paimon.types.DataField;
@@ -45,6 +45,7 @@ import org.apache.paimon.types.MapType;
 import org.apache.paimon.types.ReassignFieldId;
 import org.apache.paimon.types.RowType;
 import org.apache.paimon.utils.BranchManager;
+import org.apache.paimon.utils.LazyField;
 import org.apache.paimon.utils.Preconditions;
 import org.apache.paimon.utils.SnapshotManager;
 import org.apache.paimon.utils.StringUtils;
@@ -67,7 +68,6 @@ import java.util.List;
 import java.util.Map;
 import java.util.Objects;
 import java.util.Optional;
-import java.util.concurrent.Callable;
 import java.util.concurrent.atomic.AtomicInteger;
 import java.util.function.Function;
 import java.util.function.Predicate;
@@ -91,8 +91,6 @@ public class SchemaManager implements Serializable {
     private final FileIO fileIO;
     private final Path tableRoot;
 
-    @Nullable private transient Lock lock;
-
     private final String branch;
 
     public SchemaManager(FileIO fileIO, Path tableRoot) {
@@ -110,11 +108,6 @@ public class SchemaManager implements Serializable {
         return new SchemaManager(fileIO, tableRoot, branchName);
     }
 
-    public SchemaManager withLock(@Nullable Lock lock) {
-        this.lock = lock;
-        return this;
-    }
-
     public Optional<TableSchema> latest() {
         try {
             return listVersionedFiles(fileIO, schemaDirectory(), SCHEMA_PREFIX)
@@ -222,21 +215,7 @@ public class SchemaManager implements Serializable {
                 }
             }
 
-            List<DataField> fields = schema.fields();
-            List<String> partitionKeys = schema.partitionKeys();
-            List<String> primaryKeys = schema.primaryKeys();
-            Map<String, String> options = schema.options();
-            int highestFieldId = RowType.currentHighestFieldId(fields);
-
-            TableSchema newSchema =
-                    new TableSchema(
-                            0,
-                            fields,
-                            highestFieldId,
-                            partitionKeys,
-                            primaryKeys,
-                            options,
-                            schema.comment());
+            TableSchema newSchema = TableSchema.create(0, schema);
 
             // validate table from creating table
             FileStoreTableFactory.create(fileIO, tableRoot, newSchema).store();
@@ -292,8 +271,10 @@ public class SchemaManager implements Serializable {
     public TableSchema commitChanges(List<SchemaChange> changes)
             throws Catalog.TableNotExistException, Catalog.ColumnAlreadyExistException,
                     Catalog.ColumnNotExistException {
-        SnapshotManager snapshotManager = new SnapshotManager(fileIO, tableRoot, branch);
-        boolean hasSnapshots = (snapshotManager.latestSnapshotId() != null);
+        SnapshotManager snapshotManager =
+                new SnapshotManager(fileIO, tableRoot, branch, null, null);
+        LazyField<Boolean> hasSnapshots =
+                new LazyField<>(() -> snapshotManager.latestSnapshot() != null);
 
         while (true) {
             TableSchema oldTableSchema =
@@ -310,7 +291,7 @@ public class SchemaManager implements Serializable {
             for (SchemaChange change : changes) {
                 if (change instanceof SetOption) {
                     SetOption setOption = (SetOption) change;
-                    if (hasSnapshots) {
+                    if (hasSnapshots.get()) {
                         checkAlterTableOption(
                                 setOption.key(),
                                 oldOptions.get(setOption.key()),
@@ -320,7 +301,7 @@ public class SchemaManager implements Serializable {
                     newOptions.put(setOption.key(), setOption.value());
                 } else if (change instanceof RemoveOption) {
                     RemoveOption removeOption = (RemoveOption) change;
-                    if (hasSnapshots) {
+                    if (hasSnapshots.get()) {
                         checkResetTableOption(removeOption.key());
                     }
                     newOptions.remove(removeOption.key());
@@ -792,16 +773,11 @@ public class SchemaManager implements Serializable {
     }
 
     @VisibleForTesting
-    boolean commit(TableSchema newSchema) throws Exception {
+    public boolean commit(TableSchema newSchema) throws Exception {
         SchemaValidation.validateTableSchema(newSchema);
         SchemaValidation.validateFallbackBranch(this, newSchema);
         Path schemaPath = toSchemaPath(newSchema.id());
-        Callable<Boolean> callable =
-                () -> fileIO.tryToWriteAtomic(schemaPath, newSchema.toString());
-        if (lock == null) {
-            return callable.call();
-        }
-        return lock.runWithLock(callable);
+        return fileIO.tryToWriteAtomic(schemaPath, newSchema.toString());
     }
 
     /** Read schema for schema id. */
@@ -875,6 +851,10 @@ public class SchemaManager implements Serializable {
             if (newBucket == -1) {
                 throw new UnsupportedOperationException("Cannot change bucket to -1.");
             }
+            if (oldBucket == BucketMode.POSTPONE_BUCKET) {
+                throw new UnsupportedOperationException(
+                        "Cannot change bucket for postpone bucket tables.");
+            }
         }
     }
 
