@@ -25,6 +25,7 @@ import org.apache.paimon.manifest.ExpireFileEntry;
 import org.apache.paimon.operation.ChangelogDeletion;
 import org.apache.paimon.operation.SnapshotDeletion;
 import org.apache.paimon.operation.TagDeletion;
+import org.apache.paimon.utils.ChangelogManager;
 import org.apache.paimon.utils.SnapshotManager;
 import org.apache.paimon.utils.TagManager;
 
@@ -49,6 +50,7 @@ public class RollbackHelper {
     private static final Logger LOG = LoggerFactory.getLogger(RollbackHelper.class);
 
     private final SnapshotManager snapshotManager;
+    private final ChangelogManager changelogManager;
     private final TagManager tagManager;
     private final FileIO fileIO;
     private final SnapshotDeletion snapshotDeletion;
@@ -57,12 +59,14 @@ public class RollbackHelper {
 
     public RollbackHelper(
             SnapshotManager snapshotManager,
+            ChangelogManager changelogManager,
             TagManager tagManager,
             FileIO fileIO,
             SnapshotDeletion snapshotDeletion,
             ChangelogDeletion changelogDeletion,
             TagDeletion tagDeletion) {
         this.snapshotManager = snapshotManager;
+        this.changelogManager = changelogManager;
         this.tagManager = tagManager;
         this.fileIO = fileIO;
         this.snapshotDeletion = snapshotDeletion;
@@ -70,10 +74,10 @@ public class RollbackHelper {
         this.tagDeletion = tagDeletion;
     }
 
-    /** Clean snapshots and tags whose id is larger than given snapshot's. */
-    public void cleanLargerThan(Snapshot retainedSnapshot) {
+    /** Clean snapshots and tags whose id is larger than given snapshot's and update latest hit. */
+    public void updateLatestAndCleanLargerThan(Snapshot retainedSnapshot) {
         // clean data files
-        List<Snapshot> cleanedSnapshots = cleanSnapshotsDataFiles(retainedSnapshot);
+        List<Snapshot> cleanedSnapshots = updateLatestAndCleanSnapshotsDataFiles(retainedSnapshot);
         List<Changelog> cleanedChangelogs = cleanLongLivedChangelogDataFiles(retainedSnapshot);
         List<Snapshot> cleanedTags = cleanTagsDataFiles(retainedSnapshot);
         Set<Long> cleanedIds = new HashSet<>();
@@ -100,7 +104,7 @@ public class RollbackHelper {
         }
     }
 
-    private List<Snapshot> cleanSnapshotsDataFiles(Snapshot retainedSnapshot) {
+    private List<Snapshot> updateLatestAndCleanSnapshotsDataFiles(Snapshot retainedSnapshot) {
         long earliest =
                 checkNotNull(
                         snapshotManager.earliestSnapshotId(), "Cannot find earliest snapshot.");
@@ -113,7 +117,6 @@ public class RollbackHelper {
         } catch (IOException e) {
             throw new UncheckedIOException(e);
         }
-
         // delete snapshot files first, cannot be read now
         // it is possible that some snapshots have been expired
         List<Snapshot> toBeCleaned = new ArrayList<>();
@@ -143,8 +146,8 @@ public class RollbackHelper {
     }
 
     private List<Changelog> cleanLongLivedChangelogDataFiles(Snapshot retainedSnapshot) {
-        Long earliest = snapshotManager.earliestLongLivedChangelogId();
-        Long latest = snapshotManager.latestLongLivedChangelogId();
+        Long earliest = changelogManager.earliestLongLivedChangelogId();
+        Long latest = changelogManager.latestLongLivedChangelogId();
         if (earliest == null || latest == null) {
             return Collections.emptyList();
         }
@@ -153,7 +156,7 @@ public class RollbackHelper {
         List<Changelog> toBeCleaned = new ArrayList<>();
         long to = Math.max(earliest, retainedSnapshot.id() + 1);
         for (long i = latest; i >= to; i--) {
-            toBeCleaned.add(snapshotManager.changelog(i));
+            toBeCleaned.add(changelogManager.changelog(i));
         }
 
         // modify the latest hint
@@ -162,9 +165,9 @@ public class RollbackHelper {
                 if (to == earliest) {
                     // all changelog has been cleaned, so we do not know the actual latest id
                     // set to -1
-                    snapshotManager.commitLongLivedChangelogLatestHint(-1);
+                    changelogManager.commitLongLivedChangelogLatestHint(-1);
                 } else {
-                    snapshotManager.commitLongLivedChangelogLatestHint(to - 1);
+                    changelogManager.commitLongLivedChangelogLatestHint(to - 1);
                 }
             }
         } catch (IOException e) {
@@ -174,7 +177,7 @@ public class RollbackHelper {
         // delete data files of changelog
         for (Changelog changelog : toBeCleaned) {
             // delete changelog files first, cannot be read now
-            fileIO.deleteQuietly(snapshotManager.longLivedChangelogPath(changelog.id()));
+            fileIO.deleteQuietly(changelogManager.longLivedChangelogPath(changelog.id()));
             // clean the deleted file
             changelogDeletion.cleanUnusedDataFiles(changelog, manifestEntry -> false);
         }
