@@ -19,6 +19,7 @@
 package org.apache.paimon.table.source;
 
 import org.apache.paimon.data.BinaryRow;
+import org.apache.paimon.data.InternalRow;
 import org.apache.paimon.io.DataFileMeta;
 import org.apache.paimon.io.DataFileMeta08Serializer;
 import org.apache.paimon.io.DataFileMeta09Serializer;
@@ -28,7 +29,12 @@ import org.apache.paimon.io.DataInputView;
 import org.apache.paimon.io.DataInputViewStreamWrapper;
 import org.apache.paimon.io.DataOutputView;
 import org.apache.paimon.io.DataOutputViewStreamWrapper;
+import org.apache.paimon.predicate.CompareUtils;
+import org.apache.paimon.stats.SimpleStatsEvolution;
+import org.apache.paimon.stats.SimpleStatsEvolutions;
+import org.apache.paimon.types.DataField;
 import org.apache.paimon.utils.FunctionWithIOException;
+import org.apache.paimon.utils.InternalRowUtils;
 import org.apache.paimon.utils.SerializationUtils;
 
 import javax.annotation.Nullable;
@@ -52,12 +58,13 @@ public class DataSplit implements Split {
 
     private static final long serialVersionUID = 7L;
     private static final long MAGIC = -2394839472490812314L;
-    private static final int VERSION = 5;
+    private static final int VERSION = 6;
 
     private long snapshotId = 0;
     private BinaryRow partition;
     private int bucket = -1;
     private String bucketPath;
+    @Nullable private Integer totalBuckets;
 
     private List<DataFileMeta> beforeFiles = new ArrayList<>();
     @Nullable private List<DeletionFile> beforeDeletionFiles;
@@ -86,6 +93,10 @@ public class DataSplit implements Split {
         return bucketPath;
     }
 
+    public @Nullable Integer totalBuckets() {
+        return totalBuckets;
+    }
+
     public List<DataFileMeta> beforeFiles() {
         return beforeFiles;
     }
@@ -141,6 +152,44 @@ public class DataSplit implements Split {
         return partialMergedRowCount();
     }
 
+    public Object minValue(int fieldIndex, DataField dataField, SimpleStatsEvolutions evolutions) {
+        Object minValue = null;
+        for (DataFileMeta dataFile : dataFiles) {
+            SimpleStatsEvolution evolution = evolutions.getOrCreate(dataFile.schemaId());
+            InternalRow minValues =
+                    evolution.evolution(
+                            dataFile.valueStats().minValues(), dataFile.valueStatsCols());
+            Object other = InternalRowUtils.get(minValues, fieldIndex, dataField.type());
+            if (minValue == null) {
+                minValue = other;
+            } else if (other != null) {
+                if (CompareUtils.compareLiteral(dataField.type(), minValue, other) > 0) {
+                    minValue = other;
+                }
+            }
+        }
+        return minValue;
+    }
+
+    public Object maxValue(int fieldIndex, DataField dataField, SimpleStatsEvolutions evolutions) {
+        Object maxValue = null;
+        for (DataFileMeta dataFile : dataFiles) {
+            SimpleStatsEvolution evolution = evolutions.getOrCreate(dataFile.schemaId());
+            InternalRow maxValues =
+                    evolution.evolution(
+                            dataFile.valueStats().maxValues(), dataFile.valueStatsCols());
+            Object other = InternalRowUtils.get(maxValues, fieldIndex, dataField.type());
+            if (maxValue == null) {
+                maxValue = other;
+            } else if (other != null) {
+                if (CompareUtils.compareLiteral(dataField.type(), maxValue, other) < 0) {
+                    maxValue = other;
+                }
+            }
+        }
+        return maxValue;
+    }
+
     /**
      * Obtain merged row count as much as possible. There are two scenarios where accurate row count
      * can be calculated:
@@ -232,6 +281,7 @@ public class DataSplit implements Split {
                 && rawConvertible == dataSplit.rawConvertible
                 && Objects.equals(partition, dataSplit.partition)
                 && Objects.equals(bucketPath, dataSplit.bucketPath)
+                && Objects.equals(totalBuckets, dataSplit.totalBuckets)
                 && Objects.equals(beforeFiles, dataSplit.beforeFiles)
                 && Objects.equals(beforeDeletionFiles, dataSplit.beforeDeletionFiles)
                 && Objects.equals(dataFiles, dataSplit.dataFiles)
@@ -245,6 +295,7 @@ public class DataSplit implements Split {
                 partition,
                 bucket,
                 bucketPath,
+                totalBuckets,
                 beforeFiles,
                 beforeDeletionFiles,
                 dataFiles,
@@ -266,6 +317,7 @@ public class DataSplit implements Split {
         this.partition = other.partition;
         this.bucket = other.bucket;
         this.bucketPath = other.bucketPath;
+        this.totalBuckets = other.totalBuckets;
         this.beforeFiles = other.beforeFiles;
         this.beforeDeletionFiles = other.beforeDeletionFiles;
         this.dataFiles = other.dataFiles;
@@ -281,6 +333,12 @@ public class DataSplit implements Split {
         SerializationUtils.serializeBinaryRow(partition, out);
         out.writeInt(bucket);
         out.writeUTF(bucketPath);
+        if (totalBuckets != null) {
+            out.writeBoolean(true);
+            out.writeInt(totalBuckets);
+        } else {
+            out.writeBoolean(false);
+        }
 
         DataFileMetaSerializer dataFileSer = new DataFileMetaSerializer();
         out.writeInt(beforeFiles.size());
@@ -310,6 +368,7 @@ public class DataSplit implements Split {
         BinaryRow partition = SerializationUtils.deserializeBinaryRow(in);
         int bucket = in.readInt();
         String bucketPath = in.readUTF();
+        Integer totalBuckets = version >= 6 && in.readBoolean() ? in.readInt() : null;
 
         FunctionWithIOException<DataInputView, DataFileMeta> dataFileSer =
                 getFileMetaSerde(version);
@@ -341,6 +400,7 @@ public class DataSplit implements Split {
                         .withPartition(partition)
                         .withBucket(bucket)
                         .withBucketPath(bucketPath)
+                        .withTotalBuckets(totalBuckets)
                         .withBeforeFiles(beforeFiles)
                         .withDataFiles(dataFiles)
                         .isStreaming(isStreaming)
@@ -414,6 +474,11 @@ public class DataSplit implements Split {
             return this;
         }
 
+        public Builder withTotalBuckets(Integer totalBuckets) {
+            this.split.totalBuckets = totalBuckets;
+            return this;
+        }
+
         public Builder withBeforeFiles(List<DataFileMeta> beforeFiles) {
             this.split.beforeFiles = new ArrayList<>(beforeFiles);
             return this;
