@@ -46,6 +46,7 @@ import org.apache.paimon.table.source.DeletionFile;
 import org.apache.paimon.table.source.PlanImpl;
 import org.apache.paimon.table.source.ScanMode;
 import org.apache.paimon.table.source.SplitGenerator;
+import org.apache.paimon.utils.ChangelogManager;
 import org.apache.paimon.utils.FileStorePathFactory;
 import org.apache.paimon.utils.Filter;
 import org.apache.paimon.utils.Pair;
@@ -64,6 +65,7 @@ import java.util.Map;
 import java.util.Optional;
 import java.util.Set;
 import java.util.function.BiConsumer;
+import java.util.stream.Collectors;
 
 import static org.apache.paimon.Snapshot.FIRST_SNAPSHOT_ID;
 import static org.apache.paimon.deletionvectors.DeletionVectorsIndexFile.DELETION_VECTORS_INDEX;
@@ -79,6 +81,7 @@ public class SnapshotReaderImpl implements SnapshotReader {
     private final CoreOptions options;
     private final boolean deletionVectors;
     private final SnapshotManager snapshotManager;
+    private final ChangelogManager changelogManager;
     private final ConsumerManager consumerManager;
     private final SplitGenerator splitGenerator;
     private final BiConsumer<FileStoreScan, Predicate> nonPartitionFilterConsumer;
@@ -95,6 +98,7 @@ public class SnapshotReaderImpl implements SnapshotReader {
             TableSchema tableSchema,
             CoreOptions options,
             SnapshotManager snapshotManager,
+            ChangelogManager changelogManager,
             SplitGenerator splitGenerator,
             BiConsumer<FileStoreScan, Predicate> nonPartitionFilterConsumer,
             DefaultValueAssigner defaultValueAssigner,
@@ -106,6 +110,7 @@ public class SnapshotReaderImpl implements SnapshotReader {
         this.options = options;
         this.deletionVectors = options.deletionVectorsEnabled();
         this.snapshotManager = snapshotManager;
+        this.changelogManager = changelogManager;
         this.consumerManager =
                 new ConsumerManager(
                         snapshotManager.fileIO(),
@@ -130,6 +135,11 @@ public class SnapshotReaderImpl implements SnapshotReader {
         return snapshotManager;
     }
 
+    @Override
+    public ChangelogManager changelogManager() {
+        return changelogManager;
+    }
+
     @Override
     public ManifestsReader manifestsReader() {
         return scan.manifestsReader();
@@ -192,6 +202,12 @@ public class SnapshotReaderImpl implements SnapshotReader {
         return this;
     }
 
+    @Override
+    public SnapshotReader withPartitionsFilter(List<Map<String, String>> partitions) {
+        scan.withPartitionsFilter(partitions);
+        return this;
+    }
+
     @Override
     public SnapshotReader withFilter(Predicate predicate) {
         List<String> partitionKeys = tableSchema.partitionKeys();
@@ -229,6 +245,12 @@ public class SnapshotReaderImpl implements SnapshotReader {
         return this;
     }
 
+    @Override
+    public SnapshotReader withLevel(int level) {
+        scan.withLevel(level);
+        return this;
+    }
+
     @Override
     public SnapshotReader withLevelFilter(Filter<Integer> levelFilter) {
         scan.withLevelFilter(levelFilter);
@@ -296,17 +318,17 @@ public class SnapshotReaderImpl implements SnapshotReader {
         FileStoreScan.Plan plan = scan.plan();
         @Nullable Snapshot snapshot = plan.snapshot();
 
-        Map<BinaryRow, Map<Integer, List<DataFileMeta>>> files =
+        Map<BinaryRow, Map<Integer, List<ManifestEntry>>> grouped =
                 groupByPartFiles(plan.files(FileKind.ADD));
         if (options.scanPlanSortPartition()) {
-            Map<BinaryRow, Map<Integer, List<DataFileMeta>>> newFiles = new LinkedHashMap<>();
-            files.entrySet().stream()
+            Map<BinaryRow, Map<Integer, List<ManifestEntry>>> sorted = new LinkedHashMap<>();
+            grouped.entrySet().stream()
                     .sorted((o1, o2) -> partitionComparator().compare(o1.getKey(), o2.getKey()))
-                    .forEach(entry -> newFiles.put(entry.getKey(), entry.getValue()));
-            files = newFiles;
+                    .forEach(entry -> sorted.put(entry.getKey(), entry.getValue()));
+            grouped = sorted;
         }
         List<DataSplit> splits =
-                generateSplits(snapshot, scanMode != ScanMode.ALL, splitGenerator, files);
+                generateSplits(snapshot, scanMode != ScanMode.ALL, splitGenerator, grouped);
         return new PlanImpl(
                 plan.watermark(), snapshot == null ? null : snapshot.id(), (List) splits);
     }
@@ -315,44 +337,54 @@ public class SnapshotReaderImpl implements SnapshotReader {
             @Nullable Snapshot snapshot,
             boolean isStreaming,
             SplitGenerator splitGenerator,
-            Map<BinaryRow, Map<Integer, List<DataFileMeta>>> groupedDataFiles) {
+            Map<BinaryRow, Map<Integer, List<ManifestEntry>>> groupedManifestEntries) {
         List<DataSplit> splits = new ArrayList<>();
         // Read deletion indexes at once to reduce file IO
-        Map<Pair<BinaryRow, Integer>, List<IndexFileMeta>> deletionIndexFilesMap =
-                deletionVectors && snapshot != null
-                        ? indexFileHandler.scan(
-                                snapshot, DELETION_VECTORS_INDEX, groupedDataFiles.keySet())
-                        : Collections.emptyMap();
-        for (Map.Entry<BinaryRow, Map<Integer, List<DataFileMeta>>> entry :
-                groupedDataFiles.entrySet()) {
+        Map<Pair<BinaryRow, Integer>, List<IndexFileMeta>> deletionIndexFilesMap = null;
+        if (!isStreaming) {
+            deletionIndexFilesMap =
+                    deletionVectors && snapshot != null
+                            ? indexFileHandler.scan(
+                                    snapshot,
+                                    DELETION_VECTORS_INDEX,
+                                    groupedManifestEntries.keySet())
+                            : Collections.emptyMap();
+        }
+        for (Map.Entry<BinaryRow, Map<Integer, List<ManifestEntry>>> entry :
+                groupedManifestEntries.entrySet()) {
             BinaryRow partition = entry.getKey();
-            Map<Integer, List<DataFileMeta>> buckets = entry.getValue();
-            for (Map.Entry<Integer, List<DataFileMeta>> bucketEntry : buckets.entrySet()) {
+            Map<Integer, List<ManifestEntry>> buckets = entry.getValue();
+            for (Map.Entry<Integer, List<ManifestEntry>> bucketEntry : buckets.entrySet()) {
                 int bucket = bucketEntry.getKey();
-                List<DataFileMeta> bucketFiles = bucketEntry.getValue();
+                List<DataFileMeta> bucketFiles =
+                        bucketEntry.getValue().stream()
+                                .map(ManifestEntry::file)
+                                .collect(Collectors.toList());
                 DataSplit.Builder builder =
                         DataSplit.builder()
                                 .withSnapshot(
                                         snapshot == null ? FIRST_SNAPSHOT_ID - 1 : snapshot.id())
                                 .withPartition(partition)
                                 .withBucket(bucket)
+                                .withTotalBuckets(bucketEntry.getValue().get(0).totalBuckets())
                                 .isStreaming(isStreaming);
                 List<SplitGenerator.SplitGroup> splitGroups =
                         isStreaming
                                 ? splitGenerator.splitForStreaming(bucketFiles)
                                 : splitGenerator.splitForBatch(bucketFiles);
-                List<IndexFileMeta> deletionIndexFiles =
-                        deletionIndexFilesMap.getOrDefault(
-                                Pair.of(partition, bucket), Collections.emptyList());
                 for (SplitGenerator.SplitGroup splitGroup : splitGroups) {
                     List<DataFileMeta> dataFiles = splitGroup.files;
                     String bucketPath = pathFactory.bucketPath(partition, bucket).toString();
                     builder.withDataFiles(dataFiles)
                             .rawConvertible(splitGroup.rawConvertible)
                             .withBucketPath(bucketPath);
-                    if (deletionVectors) {
+                    if (deletionVectors && deletionIndexFilesMap != null) {
                         builder.withDataDeletionFiles(
-                                getDeletionFiles(dataFiles, deletionIndexFiles));
+                                getDeletionFiles(
+                                        dataFiles,
+                                        deletionIndexFilesMap.getOrDefault(
+                                                Pair.of(partition, bucket),
+                                                Collections.emptyList())));
                     }
 
                     splits.add(builder.build());
@@ -387,9 +419,9 @@ public class SnapshotReaderImpl implements SnapshotReader {
         withMode(ScanMode.DELTA);
         FileStoreScan.Plan plan = scan.plan();
 
-        Map<BinaryRow, Map<Integer, List<DataFileMeta>>> beforeFiles =
+        Map<BinaryRow, Map<Integer, List<ManifestEntry>>> beforeFiles =
                 groupByPartFiles(plan.files(FileKind.DELETE));
-        Map<BinaryRow, Map<Integer, List<DataFileMeta>>> dataFiles =
+        Map<BinaryRow, Map<Integer, List<ManifestEntry>>> dataFiles =
                 groupByPartFiles(plan.files(FileKind.ADD));
         Snapshot beforeSnapshot = snapshotManager.snapshot(plan.snapshot().id() - 1);
         return toChangesPlan(true, plan, beforeSnapshot, beforeFiles, dataFiles);
@@ -399,8 +431,8 @@ public class SnapshotReaderImpl implements SnapshotReader {
             boolean isStreaming,
             FileStoreScan.Plan plan,
             Snapshot beforeSnapshot,
-            Map<BinaryRow, Map<Integer, List<DataFileMeta>>> beforeFiles,
-            Map<BinaryRow, Map<Integer, List<DataFileMeta>>> dataFiles) {
+            Map<BinaryRow, Map<Integer, List<ManifestEntry>>> beforeFiles,
+            Map<BinaryRow, Map<Integer, List<ManifestEntry>>> dataFiles) {
         Snapshot snapshot = plan.snapshot();
         List<DataSplit> splits = new ArrayList<>();
         Map<BinaryRow, Set<Integer>> buckets = new HashMap<>();
@@ -413,42 +445,63 @@ public class SnapshotReaderImpl implements SnapshotReader {
                         buckets.computeIfAbsent(part, k -> new HashSet<>())
                                 .addAll(bucketMap.keySet()));
         // Read deletion indexes at once to reduce file IO
-        Map<Pair<BinaryRow, Integer>, List<IndexFileMeta>> beforDeletionIndexFilesMap =
-                deletionVectors
-                        ? indexFileHandler.scan(
-                                beforeSnapshot, DELETION_VECTORS_INDEX, beforeFiles.keySet())
-                        : Collections.emptyMap();
-        Map<Pair<BinaryRow, Integer>, List<IndexFileMeta>> deletionIndexFilesMap =
-                deletionVectors
-                        ? indexFileHandler.scan(
-                                snapshot, DELETION_VECTORS_INDEX, dataFiles.keySet())
-                        : Collections.emptyMap();
+        Map<Pair<BinaryRow, Integer>, List<IndexFileMeta>> beforDeletionIndexFilesMap = null;
+        Map<Pair<BinaryRow, Integer>, List<IndexFileMeta>> deletionIndexFilesMap = null;
+        if (!isStreaming) {
+            beforDeletionIndexFilesMap =
+                    deletionVectors
+                            ? indexFileHandler.scan(
+                                    beforeSnapshot, DELETION_VECTORS_INDEX, beforeFiles.keySet())
+                            : Collections.emptyMap();
+            deletionIndexFilesMap =
+                    deletionVectors
+                            ? indexFileHandler.scan(
+                                    snapshot, DELETION_VECTORS_INDEX, dataFiles.keySet())
+                            : Collections.emptyMap();
+        }
 
         for (Map.Entry<BinaryRow, Set<Integer>> entry : buckets.entrySet()) {
             BinaryRow part = entry.getKey();
             for (Integer bucket : entry.getValue()) {
-                List<DataFileMeta> before =
+                List<ManifestEntry> beforeEntries =
                         beforeFiles
                                 .getOrDefault(part, Collections.emptyMap())
                                 .getOrDefault(bucket, Collections.emptyList());
-                List<DataFileMeta> data =
+                List<ManifestEntry> dataEntries =
                         dataFiles
                                 .getOrDefault(part, Collections.emptyMap())
                                 .getOrDefault(bucket, Collections.emptyList());
 
                 // deduplicate
-                before.removeIf(data::remove);
+                beforeEntries.removeIf(dataEntries::remove);
+
+                Integer totalBuckets = null;
+                if (!dataEntries.isEmpty()) {
+                    totalBuckets = dataEntries.get(0).totalBuckets();
+                } else if (!beforeEntries.isEmpty()) {
+                    totalBuckets = beforeEntries.get(0).totalBuckets();
+                }
+
+                List<DataFileMeta> before =
+                        beforeEntries.stream()
+                                .map(ManifestEntry::file)
+                                .collect(Collectors.toList());
+                List<DataFileMeta> data =
+                        dataEntries.stream().map(ManifestEntry::file).collect(Collectors.toList());
 
                 DataSplit.Builder builder =
                         DataSplit.builder()
                                 .withSnapshot(snapshot.id())
                                 .withPartition(part)
                                 .withBucket(bucket)
+                                .withTotalBuckets(totalBuckets)
                                 .withBeforeFiles(before)
                                 .withDataFiles(data)
                                 .isStreaming(isStreaming)
                                 .withBucketPath(pathFactory.bucketPath(part, bucket).toString());
-                if (deletionVectors) {
+                if (deletionVectors
+                        && beforDeletionIndexFilesMap != null
+                        && deletionIndexFilesMap != null) {
                     builder.withBeforeDeletionFiles(
                             getDeletionFiles(
                                     before,
@@ -472,9 +525,9 @@ public class SnapshotReaderImpl implements SnapshotReader {
     public Plan readIncrementalDiff(Snapshot before) {
         withMode(ScanMode.ALL);
         FileStoreScan.Plan plan = scan.plan();
-        Map<BinaryRow, Map<Integer, List<DataFileMeta>>> dataFiles =
+        Map<BinaryRow, Map<Integer, List<ManifestEntry>>> dataFiles =
                 groupByPartFiles(plan.files(FileKind.ADD));
-        Map<BinaryRow, Map<Integer, List<DataFileMeta>>> beforeFiles =
+        Map<BinaryRow, Map<Integer, List<ManifestEntry>>> beforeFiles =
                 groupByPartFiles(scan.withSnapshot(before).plan().files(FileKind.ADD));
         return toChangesPlan(false, plan, before, beforeFiles, dataFiles);
     }
