@@ -18,6 +18,8 @@
 
 package org.apache.paimon.table.system;
 
+import org.apache.paimon.casting.CastExecutor;
+import org.apache.paimon.casting.CastExecutors;
 import org.apache.paimon.data.BinaryRow;
 import org.apache.paimon.data.BinaryString;
 import org.apache.paimon.data.GenericRow;
@@ -25,6 +27,7 @@ import org.apache.paimon.data.InternalArray;
 import org.apache.paimon.data.InternalRow;
 import org.apache.paimon.data.LazyGenericRow;
 import org.apache.paimon.disk.IOManager;
+import org.apache.paimon.fs.FileIO;
 import org.apache.paimon.io.DataFileMeta;
 import org.apache.paimon.io.DataFilePathFactory;
 import org.apache.paimon.manifest.FileSource;
@@ -136,6 +139,11 @@ public class FilesTable implements ReadonlyTable {
         return Collections.singletonList("file_path");
     }
 
+    @Override
+    public FileIO fileIO() {
+        return storeTable.fileIO();
+    }
+
     @Override
     public InnerTableScan newScan() {
         return new FilesScan(storeTable);
@@ -182,10 +190,10 @@ public class FilesTable implements ReadonlyTable {
             SnapshotReader snapshotReader = fileStoreTable.newSnapshotReader();
             if (partitionPredicate != null && partitionPredicate.function() instanceof Equal) {
                 String partitionStr = partitionPredicate.literals().get(0).toString();
-                if (partitionStr.startsWith("[")) {
+                if (partitionStr.startsWith("{")) {
                     partitionStr = partitionStr.substring(1);
                 }
-                if (partitionStr.endsWith("]")) {
+                if (partitionStr.endsWith("}")) {
                     partitionStr = partitionStr.substring(0, partitionStr.length() - 1);
                 }
                 String[] partFields = partitionStr.split(", ");
@@ -318,8 +326,11 @@ public class FilesTable implements ReadonlyTable {
                     new SimpleStatsEvolutions(
                             sid -> schemaManager.schema(sid).fields(), storeTable.schema().id());
 
-            RowDataToObjectArrayConverter partitionConverter =
-                    new RowDataToObjectArrayConverter(storeTable.schema().logicalPartitionType());
+            @SuppressWarnings("unchecked")
+            CastExecutor<InternalRow, BinaryString> partitionCastExecutor =
+                    (CastExecutor<InternalRow, BinaryString>)
+                            CastExecutors.resolveToString(
+                                    storeTable.schema().logicalPartitionType());
 
             Function<Long, RowDataToObjectArrayConverter> keyConverters =
                     new Function<Long, RowDataToObjectArrayConverter>() {
@@ -349,7 +360,7 @@ public class FilesTable implements ReadonlyTable {
                                 file ->
                                         toRow(
                                                 (DataSplit) dataSplit,
-                                                partitionConverter,
+                                                partitionCastExecutor,
                                                 keyConverters,
                                                 file,
                                                 simpleStatsEvolutions)));
@@ -368,7 +379,7 @@ public class FilesTable implements ReadonlyTable {
 
         private LazyGenericRow toRow(
                 DataSplit dataSplit,
-                RowDataToObjectArrayConverter partitionConverter,
+                CastExecutor<InternalRow, BinaryString> partitionCastExecutor,
                 Function<Long, RowDataToObjectArrayConverter> keyConverters,
                 DataFileMeta file,
                 SimpleStatsEvolutions simpleStatsEvolutions) {
@@ -379,10 +390,7 @@ public class FilesTable implements ReadonlyTable {
                         () ->
                                 dataSplit.partition() == null
                                         ? null
-                                        : BinaryString.fromString(
-                                                Arrays.toString(
-                                                        partitionConverter.convert(
-                                                                dataSplit.partition()))),
+                                        : partitionCastExecutor.cast(dataSplit.partition()),
                         dataSplit::bucket,
                         () ->
                                 BinaryString.fromString(
