@@ -120,6 +120,11 @@ public class TagsTable implements ReadonlyTable {
         return Collections.singletonList(TAG_NAME);
     }
 
+    @Override
+    public FileIO fileIO() {
+        return dataTable.fileIO();
+    }
+
     @Override
     public InnerTableScan newScan() {
         return new TagsScan();
