@@ -18,11 +18,10 @@
 
 package org.apache.paimon.utils;
 
-import org.apache.paimon.Changelog;
 import org.apache.paimon.Snapshot;
 import org.apache.paimon.fs.FileIO;
-import org.apache.paimon.fs.FileStatus;
 import org.apache.paimon.fs.Path;
+import org.apache.paimon.table.Instant;
 
 import org.apache.paimon.shade.caffeine2.com.github.benmanes.caffeine.cache.Cache;
 
@@ -35,8 +34,8 @@ import javax.annotation.Nullable;
 import java.io.FileNotFoundException;
 import java.io.IOException;
 import java.io.Serializable;
+import java.io.UncheckedIOException;
 import java.util.ArrayList;
-import java.util.Arrays;
 import java.util.Collections;
 import java.util.Comparator;
 import java.util.HashSet;
@@ -45,16 +44,12 @@ import java.util.List;
 import java.util.Optional;
 import java.util.Set;
 import java.util.concurrent.ExecutorService;
-import java.util.concurrent.ThreadLocalRandom;
-import java.util.concurrent.TimeUnit;
-import java.util.function.BinaryOperator;
 import java.util.function.Consumer;
 import java.util.function.Function;
 import java.util.function.Predicate;
 import java.util.stream.Collectors;
 import java.util.stream.LongStream;
 
-import static org.apache.paimon.utils.BranchManager.DEFAULT_MAIN_BRANCH;
 import static org.apache.paimon.utils.BranchManager.branchPath;
 import static org.apache.paimon.utils.FileUtils.listVersionedFiles;
 import static org.apache.paimon.utils.ThreadPoolUtils.createCachedThreadPool;
@@ -67,40 +62,35 @@ public class SnapshotManager implements Serializable {
 
     private static final Logger LOG = LoggerFactory.getLogger(SnapshotManager.class);
 
-    private static final String SNAPSHOT_PREFIX = "snapshot-";
-    private static final String CHANGELOG_PREFIX = "changelog-";
-    public static final String EARLIEST = "EARLIEST";
-    public static final String LATEST = "LATEST";
-    private static final int READ_HINT_RETRY_NUM = 3;
-    private static final int READ_HINT_RETRY_INTERVAL = 1;
+    public static final String SNAPSHOT_PREFIX = "snapshot-";
+
+    public static final int EARLIEST_SNAPSHOT_DEFAULT_RETRY_NUM = 3;
 
     private final FileIO fileIO;
     private final Path tablePath;
     private final String branch;
+    @Nullable private final SnapshotLoader snapshotLoader;
     @Nullable private final Cache<Path, Snapshot> cache;
 
-    public SnapshotManager(FileIO fileIO, Path tablePath) {
-        this(fileIO, tablePath, DEFAULT_MAIN_BRANCH);
-    }
-
-    /** Specify the default branch for data writing. */
-    public SnapshotManager(FileIO fileIO, Path tablePath, @Nullable String branchName) {
-        this(fileIO, tablePath, branchName, null);
-    }
-
     public SnapshotManager(
             FileIO fileIO,
             Path tablePath,
             @Nullable String branchName,
+            @Nullable SnapshotLoader snapshotLoader,
             @Nullable Cache<Path, Snapshot> cache) {
         this.fileIO = fileIO;
         this.tablePath = tablePath;
         this.branch = BranchManager.normalizeBranch(branchName);
+        this.snapshotLoader = snapshotLoader;
         this.cache = cache;
     }
 
     public SnapshotManager copyWithBranch(String branchName) {
-        return new SnapshotManager(fileIO, tablePath, branchName);
+        SnapshotLoader newSnapshotLoader = null;
+        if (snapshotLoader != null) {
+            newSnapshotLoader = snapshotLoader.copyWithBranch(branchName);
+        }
+        return new SnapshotManager(fileIO, tablePath, branchName, newSnapshotLoader, cache);
     }
 
     public FileIO fileIO() {
@@ -115,15 +105,6 @@ public class SnapshotManager implements Serializable {
         return branch;
     }
 
-    public Path changelogDirectory() {
-        return new Path(branchPath(tablePath, branch) + "/changelog");
-    }
-
-    public Path longLivedChangelogPath(long snapshotId) {
-        return new Path(
-                branchPath(tablePath, branch) + "/changelog/" + CHANGELOG_PREFIX + snapshotId);
-    }
-
     public Path snapshotPath(long snapshotId) {
         return new Path(
                 branchPath(tablePath, branch) + "/snapshot/" + SNAPSHOT_PREFIX + snapshotId);
@@ -163,15 +144,6 @@ public class SnapshotManager implements Serializable {
         return snapshot;
     }
 
-    public Changelog changelog(long snapshotId) {
-        Path changelogPath = longLivedChangelogPath(snapshotId);
-        return Changelog.fromPath(fileIO, changelogPath);
-    }
-
-    public Changelog longLivedChangelog(long snapshotId) {
-        return Changelog.fromPath(fileIO, longLivedChangelogPath(snapshotId));
-    }
-
     public boolean snapshotExists(long snapshotId) {
         Path path = snapshotPath(snapshotId);
         try {
@@ -191,24 +163,27 @@ public class SnapshotManager implements Serializable {
         fileIO().deleteQuietly(path);
     }
 
-    public boolean longLivedChangelogExists(long snapshotId) {
-        Path path = longLivedChangelogPath(snapshotId);
-        try {
-            return fileIO.exists(path);
-        } catch (IOException e) {
-            throw new RuntimeException(
-                    "Failed to determine if changelog #" + snapshotId + " exists in path " + path,
-                    e);
-        }
-    }
-
     public @Nullable Snapshot latestSnapshot() {
+        if (snapshotLoader != null) {
+            try {
+                return snapshotLoader.load().orElse(null);
+            } catch (UnsupportedOperationException ignored) {
+            } catch (IOException e) {
+                throw new UncheckedIOException(e);
+            }
+        }
         Long snapshotId = latestSnapshotId();
         return snapshotId == null ? null : snapshot(snapshotId);
     }
 
     public @Nullable Long latestSnapshotId() {
         try {
+            if (snapshotLoader != null) {
+                try {
+                    return snapshotLoader.load().map(Snapshot::id).orElse(null);
+                } catch (UnsupportedOperationException ignored) {
+                }
+            }
             return findLatest(snapshotDirectory(), SNAPSHOT_PREFIX, this::snapshotPath);
         } catch (IOException e) {
             throw new RuntimeException("Failed to find latest snapshot id", e);
@@ -216,8 +191,49 @@ public class SnapshotManager implements Serializable {
     }
 
     public @Nullable Snapshot earliestSnapshot() {
+        return earliestSnapshot(null);
+    }
+
+    public void rollback(Instant instant) {
+        if (snapshotLoader != null) {
+            try {
+                snapshotLoader.rollback(instant);
+                return;
+            } catch (IOException e) {
+                throw new UncheckedIOException(e);
+            }
+        }
+        throw new UnsupportedOperationException("rollback is not supported");
+    }
+
+    private @Nullable Snapshot earliestSnapshot(@Nullable Long stopSnapshotId) {
         Long snapshotId = earliestSnapshotId();
-        return snapshotId == null ? null : snapshot(snapshotId);
+        if (snapshotId == null) {
+            return null;
+        }
+
+        if (stopSnapshotId == null) {
+            stopSnapshotId = snapshotId + EARLIEST_SNAPSHOT_DEFAULT_RETRY_NUM;
+        }
+
+        do {
+            try {
+                return tryGetSnapshot(snapshotId);
+            } catch (FileNotFoundException e) {
+                snapshotId++;
+                if (snapshotId > stopSnapshotId) {
+                    return null;
+                }
+                LOG.warn(
+                        "The earliest snapshot or changelog was once identified but disappeared. "
+                                + "It might have been expired by other jobs operating on this table. "
+                                + "Searching for the second earliest snapshot or changelog instead. ");
+            }
+        } while (true);
+    }
+
+    public boolean earliestFileNotExists() {
+        return HintFileUtils.readHint(fileIO, HintFileUtils.EARLIEST, snapshotDirectory()) == null;
     }
 
     public @Nullable Long earliestSnapshotId() {
@@ -228,27 +244,6 @@ public class SnapshotManager implements Serializable {
         }
     }
 
-    public @Nullable Long earliestLongLivedChangelogId() {
-        try {
-            return findEarliest(
-                    changelogDirectory(), CHANGELOG_PREFIX, this::longLivedChangelogPath);
-        } catch (IOException e) {
-            throw new RuntimeException("Failed to find earliest changelog id", e);
-        }
-    }
-
-    public @Nullable Long latestLongLivedChangelogId() {
-        try {
-            return findLatest(changelogDirectory(), CHANGELOG_PREFIX, this::longLivedChangelogPath);
-        } catch (IOException e) {
-            throw new RuntimeException("Failed to find latest changelog id", e);
-        }
-    }
-
-    public @Nullable Long latestChangelogId() {
-        return latestSnapshotId();
-    }
-
     public @Nullable Long pickOrLatest(Predicate<Snapshot> predicate) {
         Long latestId = latestSnapshotId();
         Long earliestId = earliestSnapshotId();
@@ -268,62 +263,22 @@ public class SnapshotManager implements Serializable {
         return latestId;
     }
 
-    private Snapshot changelogOrSnapshot(long snapshotId) {
-        if (longLivedChangelogExists(snapshotId)) {
-            return changelog(snapshotId);
-        } else {
-            return snapshot(snapshotId);
-        }
-    }
-
     /**
-     * Returns the latest snapshot earlier than the timestamp mills. A non-existent snapshot may be
-     * returned if all snapshots are equal to or later than the timestamp mills.
-     */
-    public @Nullable Long earlierThanTimeMills(long timestampMills, boolean startFromChangelog) {
-        Long earliestSnapshot = earliestSnapshotId();
-        Long earliest;
-        if (startFromChangelog) {
-            Long earliestChangelog = earliestLongLivedChangelogId();
-            earliest = earliestChangelog == null ? earliestSnapshot : earliestChangelog;
-        } else {
-            earliest = earliestSnapshot;
-        }
-        Long latest = latestSnapshotId();
-        if (earliest == null || latest == null) {
-            return null;
-        }
-
-        if (changelogOrSnapshot(earliest).timeMillis() >= timestampMills) {
-            return earliest - 1;
-        }
-
-        while (earliest < latest) {
-            long mid = (earliest + latest + 1) / 2;
-            if (changelogOrSnapshot(mid).timeMillis() < timestampMills) {
-                earliest = mid;
-            } else {
-                latest = mid - 1;
-            }
-        }
-        return earliest;
-    }
-
-    /**
-     * Returns a {@link Snapshot} whoes commit time is earlier than or equal to given timestamp
+     * Returns a {@link Snapshot} whose commit time is earlier than or equal to given timestamp
      * mills. If there is no such a snapshot, returns null.
      */
     public @Nullable Snapshot earlierOrEqualTimeMills(long timestampMills) {
-        Long earliest = earliestSnapshotId();
         Long latest = latestSnapshotId();
-        if (earliest == null || latest == null) {
+        if (latest == null) {
             return null;
         }
 
-        Snapshot earliestSnapShot = snapshot(earliest);
-        if (earliestSnapShot.timeMillis() > timestampMills) {
-            return earliestSnapShot;
+        Snapshot earliestSnapShot = earliestSnapshot(latest);
+        if (earliestSnapShot == null || earliestSnapShot.timeMillis() > timestampMills) {
+            return null;
         }
+        long earliest = earliestSnapShot.id();
+
         Snapshot finalSnapshot = null;
         while (earliest <= latest) {
             long mid = earliest + (latest - earliest) / 2; // Avoid overflow
@@ -376,16 +331,22 @@ public class SnapshotManager implements Serializable {
     }
 
     public @Nullable Snapshot earlierOrEqualWatermark(long watermark) {
-        Long earliest = earliestSnapshotId();
         Long latest = latestSnapshotId();
         // If latest == Long.MIN_VALUE don't need next binary search for watermark
         // which can reduce IO cost with snapshot
-        if (earliest == null || latest == null || snapshot(latest).watermark() == Long.MIN_VALUE) {
+        if (latest == null || snapshot(latest).watermark() == Long.MIN_VALUE) {
             return null;
         }
+
+        Snapshot earliestSnapShot = earliestSnapshot(latest);
+        if (earliestSnapShot == null) {
+            return null;
+        }
+        long earliest = earliestSnapShot.id();
+
         Long earliestWatermark = null;
         // find the first snapshot with watermark
-        if ((earliestWatermark = snapshot(earliest).watermark()) == null) {
+        if ((earliestWatermark = earliestSnapShot.watermark()) == null) {
             while (earliest < latest) {
                 earliest++;
                 earliestWatermark = snapshot(earliest).watermark();
@@ -435,16 +396,22 @@ public class SnapshotManager implements Serializable {
     }
 
     public @Nullable Snapshot laterOrEqualWatermark(long watermark) {
-        Long earliest = earliestSnapshotId();
         Long latest = latestSnapshotId();
         // If latest == Long.MIN_VALUE don't need next binary search for watermark
         // which can reduce IO cost with snapshot
-        if (earliest == null || latest == null || snapshot(latest).watermark() == Long.MIN_VALUE) {
+        if (latest == null || snapshot(latest).watermark() == Long.MIN_VALUE) {
             return null;
         }
+
+        Snapshot earliestSnapShot = earliestSnapshot(latest);
+        if (earliestSnapShot == null) {
+            return null;
+        }
+        long earliest = earliestSnapShot.id();
+
         Long earliestWatermark = null;
         // find the first snapshot with watermark
-        if ((earliestWatermark = snapshot(earliest).watermark()) == null) {
+        if ((earliestWatermark = earliestSnapShot.watermark()) == null) {
             while (earliest < latest) {
                 earliest++;
                 earliestWatermark = snapshot(earliest).watermark();
@@ -559,13 +526,6 @@ public class SnapshotManager implements Serializable {
                 .iterator();
     }
 
-    public Iterator<Changelog> changelogs() throws IOException {
-        return listVersionedFiles(fileIO, changelogDirectory(), CHANGELOG_PREFIX)
-                .map(this::changelog)
-                .sorted(Comparator.comparingLong(Changelog::id))
-                .iterator();
-    }
-
     /**
      * If {@link FileNotFoundException} is thrown when reading the snapshot file, this snapshot may
      * be deleted by other processes, so just skip this snapshot.
@@ -590,29 +550,7 @@ public class SnapshotManager implements Serializable {
         return snapshots;
     }
 
-    public List<Changelog> safelyGetAllChangelogs() throws IOException {
-        List<Path> paths =
-                listVersionedFiles(fileIO, changelogDirectory(), CHANGELOG_PREFIX)
-                        .map(this::longLivedChangelogPath)
-                        .collect(Collectors.toList());
-
-        List<Changelog> changelogs = Collections.synchronizedList(new ArrayList<>(paths.size()));
-        collectSnapshots(
-                path -> {
-                    try {
-                        changelogs.add(Changelog.fromJson(fileIO.readFileUtf8(path)));
-                    } catch (IOException e) {
-                        if (!(e instanceof FileNotFoundException)) {
-                            throw new RuntimeException(e);
-                        }
-                    }
-                },
-                paths);
-
-        return changelogs;
-    }
-
-    private void collectSnapshots(Consumer<Path> pathConsumer, List<Path> paths)
+    private static void collectSnapshots(Consumer<Path> pathConsumer, List<Path> paths)
             throws IOException {
         ExecutorService executor =
                 createCachedThreadPool(
@@ -627,54 +565,6 @@ public class SnapshotManager implements Serializable {
         }
     }
 
-    /**
-     * Try to get non snapshot files. If any error occurred, just ignore it and return an empty
-     * result.
-     */
-    public List<Pair<Path, Long>> tryGetNonSnapshotFiles(Predicate<FileStatus> fileStatusFilter) {
-        return listPathWithFilter(snapshotDirectory(), fileStatusFilter, nonSnapshotFileFilter());
-    }
-
-    public List<Pair<Path, Long>> tryGetNonChangelogFiles(Predicate<FileStatus> fileStatusFilter) {
-        return listPathWithFilter(changelogDirectory(), fileStatusFilter, nonChangelogFileFilter());
-    }
-
-    private List<Pair<Path, Long>> listPathWithFilter(
-            Path directory, Predicate<FileStatus> fileStatusFilter, Predicate<Path> fileFilter) {
-        try {
-            FileStatus[] statuses = fileIO.listStatus(directory);
-            if (statuses == null) {
-                return Collections.emptyList();
-            }
-
-            return Arrays.stream(statuses)
-                    .filter(fileStatusFilter)
-                    .filter(status -> fileFilter.test(status.getPath()))
-                    .map(status -> Pair.of(status.getPath(), status.getLen()))
-                    .collect(Collectors.toList());
-        } catch (IOException ignored) {
-            return Collections.emptyList();
-        }
-    }
-
-    private Predicate<Path> nonSnapshotFileFilter() {
-        return path -> {
-            String name = path.getName();
-            return !name.startsWith(SNAPSHOT_PREFIX)
-                    && !name.equals(EARLIEST)
-                    && !name.equals(LATEST);
-        };
-    }
-
-    private Predicate<Path> nonChangelogFileFilter() {
-        return path -> {
-            String name = path.getName();
-            return !name.startsWith(CHANGELOG_PREFIX)
-                    && !name.equals(EARLIEST)
-                    && !name.equals(LATEST);
-        };
-    }
-
     public Optional<Snapshot> latestSnapshotOfUser(String user) {
         Long latestId = latestSnapshotId();
         if (latestId == null) {
@@ -752,10 +642,6 @@ public class SnapshotManager implements Serializable {
         return matchedSnapshots;
     }
 
-    public void commitChangelog(Changelog changelog, long id) throws IOException {
-        fileIO.writeFile(longLivedChangelogPath(id), changelog.toJson(), true);
-    }
-
     /**
      * Traversal snapshots from latest to earliest safely, this is applied on the writer side
      * because the committer may delete obsolete snapshots, which may cause the writer to encounter
@@ -800,53 +686,12 @@ public class SnapshotManager implements Serializable {
 
     private @Nullable Long findLatest(Path dir, String prefix, Function<Long, Path> file)
             throws IOException {
-        Long snapshotId = readHint(LATEST, dir);
-        if (snapshotId != null && snapshotId > 0) {
-            long nextSnapshot = snapshotId + 1;
-            // it is the latest only there is no next one
-            if (!fileIO.exists(file.apply(nextSnapshot))) {
-                return snapshotId;
-            }
-        }
-        return findByListFiles(Math::max, dir, prefix);
+        return HintFileUtils.findLatest(fileIO, dir, prefix, file);
     }
 
     private @Nullable Long findEarliest(Path dir, String prefix, Function<Long, Path> file)
             throws IOException {
-        Long snapshotId = readHint(EARLIEST, dir);
-        // null and it is the earliest only it exists
-        if (snapshotId != null && fileIO.exists(file.apply(snapshotId))) {
-            return snapshotId;
-        }
-
-        return findByListFiles(Math::min, dir, prefix);
-    }
-
-    public Long readHint(String fileName) {
-        return readHint(fileName, snapshotDirectory());
-    }
-
-    public Long readHint(String fileName, Path dir) {
-        Path path = new Path(dir, fileName);
-        int retryNumber = 0;
-        while (retryNumber++ < READ_HINT_RETRY_NUM) {
-            try {
-                return fileIO.readOverwrittenFileUtf8(path).map(Long::parseLong).orElse(null);
-            } catch (Exception ignored) {
-            }
-            try {
-                TimeUnit.MILLISECONDS.sleep(READ_HINT_RETRY_INTERVAL);
-            } catch (InterruptedException e) {
-                Thread.currentThread().interrupt();
-                throw new RuntimeException(e);
-            }
-        }
-        return null;
-    }
-
-    private Long findByListFiles(BinaryOperator<Long> reducer, Path dir, String prefix)
-            throws IOException {
-        return listVersionedFiles(fileIO, dir, prefix).reduce(reducer).orElse(null);
+        return HintFileUtils.findEarliest(fileIO, dir, prefix, file);
     }
 
     public static int findPreviousSnapshot(List<Snapshot> sortedSnapshots, long targetSnapshotId) {
@@ -869,46 +714,14 @@ public class SnapshotManager implements Serializable {
     }
 
     public void deleteLatestHint() throws IOException {
-        Path snapshotDir = snapshotDirectory();
-        Path hintFile = new Path(snapshotDir, LATEST);
-        fileIO.delete(hintFile, false);
+        HintFileUtils.deleteLatestHint(fileIO, snapshotDirectory());
     }
 
     public void commitLatestHint(long snapshotId) throws IOException {
-        commitHint(snapshotId, LATEST, snapshotDirectory());
-    }
-
-    public void commitLongLivedChangelogLatestHint(long snapshotId) throws IOException {
-        commitHint(snapshotId, LATEST, changelogDirectory());
-    }
-
-    public void commitLongLivedChangelogEarliestHint(long snapshotId) throws IOException {
-        commitHint(snapshotId, EARLIEST, changelogDirectory());
+        HintFileUtils.commitLatestHint(fileIO, snapshotId, snapshotDirectory());
     }
 
     public void commitEarliestHint(long snapshotId) throws IOException {
-        commitHint(snapshotId, EARLIEST, snapshotDirectory());
-    }
-
-    private void commitHint(long snapshotId, String fileName, Path dir) throws IOException {
-        Path hintFile = new Path(dir, fileName);
-        int loopTime = 3;
-        while (loopTime-- > 0) {
-            try {
-                fileIO.overwriteFileUtf8(hintFile, String.valueOf(snapshotId));
-                return;
-            } catch (IOException e) {
-                try {
-                    Thread.sleep(ThreadLocalRandom.current().nextInt(1000) + 500);
-                } catch (InterruptedException ex) {
-                    Thread.currentThread().interrupt();
-                    // throw root cause
-                    throw new RuntimeException(e);
-                }
-                if (loopTime == 0) {
-                    throw e;
-                }
-            }
-        }
+        HintFileUtils.commitEarliestHint(fileIO, snapshotId, snapshotDirectory());
     }
 }
