@@ -25,6 +25,7 @@ import org.apache.paimon.fs.FileIOFinder;
 import org.apache.paimon.fs.Path;
 import org.apache.paimon.index.IndexFileMeta;
 import org.apache.paimon.io.DataFileMeta;
+import org.apache.paimon.io.DataFilePathFactory;
 import org.apache.paimon.io.IndexIncrement;
 import org.apache.paimon.manifest.FileEntry;
 import org.apache.paimon.manifest.FileKind;
@@ -56,8 +57,12 @@ import org.apache.paimon.table.sink.CommitMessageImpl;
 import org.apache.paimon.table.source.DataSplit;
 import org.apache.paimon.table.source.ScanMode;
 import org.apache.paimon.types.RowType;
+import org.apache.paimon.utils.ChangelogManager;
 import org.apache.paimon.utils.CommitIncrement;
+import org.apache.paimon.utils.DataFilePathFactories;
 import org.apache.paimon.utils.FileStorePathFactory;
+import org.apache.paimon.utils.HintFileUtils;
+import org.apache.paimon.utils.Pair;
 import org.apache.paimon.utils.RecordWriter;
 import org.apache.paimon.utils.SnapshotManager;
 import org.apache.paimon.utils.TagManager;
@@ -147,17 +152,22 @@ public class TestFileStore extends KeyValueFileStore {
         return new SchemaManager(FileIOFinder.find(new Path(root)), options.path());
     }
 
+    public FileIO fileIO() {
+        return fileIO;
+    }
+
     public AbstractFileStoreWrite<KeyValue> newWrite() {
         return super.newWrite(commitUser);
     }
 
     public FileStoreCommitImpl newCommit() {
-        return super.newCommit(commitUser);
+        return super.newCommit(commitUser, null);
     }
 
     public ExpireSnapshots newExpire(int numRetainedMin, int numRetainedMax, long millisRetained) {
         return new ExpireSnapshotsImpl(
                         snapshotManager(),
+                        changelogManager(),
                         newSnapshotDeletion(),
                         new TagManager(fileIO, options.path()))
                 .config(
@@ -171,6 +181,7 @@ public class TestFileStore extends KeyValueFileStore {
     public ExpireSnapshots newExpire(ExpireConfig expireConfig) {
         return new ExpireSnapshotsImpl(
                         snapshotManager(),
+                        changelogManager(),
                         newSnapshotDeletion(),
                         new TagManager(fileIO, options.path()))
                 .config(expireConfig);
@@ -180,6 +191,7 @@ public class TestFileStore extends KeyValueFileStore {
         ExpireChangelogImpl impl =
                 new ExpireChangelogImpl(
                         snapshotManager(),
+                        changelogManager(),
                         new TagManager(fileIO, options.path()),
                         newChangelogDeletion());
         impl.config(config);
@@ -254,7 +266,7 @@ public class TestFileStore extends KeyValueFileStore {
             snapshotIdBeforeCommit = Snapshot.FIRST_SNAPSHOT_ID - 1;
         }
 
-        try (FileStoreCommit commit = newCommit(commitUser)) {
+        try (FileStoreCommit commit = newCommit(commitUser, null)) {
             commit.dropPartitions(partitions, Long.MAX_VALUE);
         }
 
@@ -332,6 +344,7 @@ public class TestFileStore extends KeyValueFileStore {
                         new CommitMessageImpl(
                                 entryWithPartition.getKey(),
                                 entryWithBucket.getKey(),
+                                options().bucket(),
                                 increment.newFilesIncrement(),
                                 increment.compactIncrement(),
                                 new IndexIncrement(indexFiles)));
@@ -344,7 +357,7 @@ public class TestFileStore extends KeyValueFileStore {
             snapshotIdBeforeCommit = Snapshot.FIRST_SNAPSHOT_ID - 1;
         }
 
-        try (FileStoreCommit commit = newCommit(commitUser)) {
+        try (FileStoreCommit commit = newCommit(commitUser, null)) {
             commitFunction.accept(commit, committable);
         }
 
@@ -490,32 +503,33 @@ public class TestFileStore extends KeyValueFileStore {
         // - latest should < true_latest
         // - earliest should < true_earliest
         SnapshotManager snapshotManager = snapshotManager();
+        ChangelogManager changelogManager = changelogManager();
         Path snapshotDir = snapshotManager.snapshotDirectory();
-        Path earliest = new Path(snapshotDir, SnapshotManager.EARLIEST);
-        Path latest = new Path(snapshotDir, SnapshotManager.LATEST);
+        Path earliest = new Path(snapshotDir, HintFileUtils.EARLIEST);
+        Path latest = new Path(snapshotDir, HintFileUtils.LATEST);
         if (actualFiles.remove(earliest)) {
-            long earliestId = snapshotManager.readHint(SnapshotManager.EARLIEST);
+            long earliestId = snapshotManager.earliestSnapshotId();
             fileIO.delete(earliest, false);
             assertThat(earliestId <= snapshotManager.earliestSnapshotId()).isTrue();
         }
         if (actualFiles.remove(latest)) {
-            long latestId = snapshotManager.readHint(SnapshotManager.LATEST);
+            long latestId = snapshotManager.latestSnapshotId();
             fileIO.delete(latest, false);
             assertThat(latestId <= snapshotManager.latestSnapshotId()).isTrue();
         }
-        Path changelogDir = snapshotManager.changelogDirectory();
-        Path earliestChangelog = new Path(changelogDir, SnapshotManager.EARLIEST);
-        Path latestChangelog = new Path(changelogDir, SnapshotManager.LATEST);
+        Path changelogDir = changelogManager.changelogDirectory();
+        Path earliestChangelog = new Path(changelogDir, HintFileUtils.EARLIEST);
+        Path latestChangelog = new Path(changelogDir, HintFileUtils.LATEST);
 
         if (actualFiles.remove(earliestChangelog)) {
-            long earliestId = snapshotManager.readHint(SnapshotManager.EARLIEST, changelogDir);
+            long earliestId = changelogManager.earliestLongLivedChangelogId();
             fileIO.delete(earliest, false);
-            assertThat(earliestId <= snapshotManager.earliestLongLivedChangelogId()).isTrue();
+            assertThat(earliestId <= changelogManager.earliestLongLivedChangelogId()).isTrue();
         }
         if (actualFiles.remove(latestChangelog)) {
-            long latestId = snapshotManager.readHint(SnapshotManager.LATEST, changelogDir);
+            long latestId = changelogManager.latestLongLivedChangelogId();
             fileIO.delete(latest, false);
-            assertThat(latestId <= snapshotManager.latestLongLivedChangelogId()).isTrue();
+            assertThat(latestId <= changelogManager.latestLongLivedChangelogId()).isTrue();
         }
 
         // for easier debugging
@@ -536,6 +550,7 @@ public class TestFileStore extends KeyValueFileStore {
         schemaManager.listAllIds().forEach(id -> result.add(schemaManager.toSchemaPath(id)));
 
         SnapshotManager snapshotManager = snapshotManager();
+        ChangelogManager changelogManager = changelogManager();
         Long latestSnapshotId = snapshotManager.latestSnapshotId();
 
         if (latestSnapshotId == null) {
@@ -545,7 +560,7 @@ public class TestFileStore extends KeyValueFileStore {
         long firstInUseSnapshotId = Snapshot.FIRST_SNAPSHOT_ID;
         for (long id = latestSnapshotId - 1; id >= Snapshot.FIRST_SNAPSHOT_ID; id--) {
             if (!snapshotManager.snapshotExists(id)
-                    && !snapshotManager.longLivedChangelogExists(id)) {
+                    && !changelogManager.longLivedChangelogExists(id)) {
                 firstInUseSnapshotId = id + 1;
                 break;
             }
@@ -562,6 +577,7 @@ public class TestFileStore extends KeyValueFileStore {
         return getFilesInUse(
                 snapshotId,
                 snapshotManager(),
+                changelogManager(),
                 fileIO,
                 pathFactory(),
                 manifestListFactory().create(),
@@ -571,6 +587,7 @@ public class TestFileStore extends KeyValueFileStore {
     public static Set<Path> getFilesInUse(
             long snapshotId,
             SnapshotManager snapshotManager,
+            ChangelogManager changelogManager,
             FileIO fileIO,
             FileStorePathFactory pathFactory,
             ManifestList manifestList,
@@ -582,16 +599,18 @@ public class TestFileStore extends KeyValueFileStore {
                     getSnapshotFileInUse(
                             snapshotId,
                             snapshotManager,
+                            changelogManager,
                             fileIO,
                             pathFactory,
                             manifestList,
                             manifestFile);
             result.addAll(files);
-        } else if (snapshotManager.longLivedChangelogExists(snapshotId)) {
+        } else if (changelogManager.longLivedChangelogExists(snapshotId)) {
             Set<Path> files =
                     getChangelogFileInUse(
                             snapshotId,
                             snapshotManager,
+                            changelogManager,
                             fileIO,
                             pathFactory,
                             manifestList,
@@ -608,6 +627,7 @@ public class TestFileStore extends KeyValueFileStore {
     private static Set<Path> getSnapshotFileInUse(
             long snapshotId,
             SnapshotManager snapshotManager,
+            ChangelogManager changelogManager,
             FileIO fileIO,
             FileStorePathFactory pathFactory,
             ManifestList manifestList,
@@ -618,7 +638,7 @@ public class TestFileStore extends KeyValueFileStore {
         boolean produceChangelog =
                 options.changelogProducer() != CoreOptions.ChangelogProducer.NONE;
         // The option from the table may not align with the expiration config
-        boolean changelogDecoupled = snapshotManager.earliestLongLivedChangelogId() != null;
+        boolean changelogDecoupled = changelogManager.earliestLongLivedChangelogId() != null;
 
         Path snapshotPath = snapshotManager.snapshotPath(snapshotId);
         Snapshot snapshot = Snapshot.fromPath(fileIO, snapshotPath);
@@ -643,11 +663,12 @@ public class TestFileStore extends KeyValueFileStore {
                         .flatMap(m -> manifestFile.read(m.fileName()).stream())
                         .collect(Collectors.toList());
         entries = new ArrayList<>(FileEntry.mergeEntries(entries));
+        DataFilePathFactories factories = new DataFilePathFactories(pathFactory);
+
         for (ManifestEntry entry : entries) {
-            result.add(
-                    new Path(
-                            pathFactory.bucketPath(entry.partition(), entry.bucket()),
-                            entry.file().fileName()));
+            DataFilePathFactory dataFilePathFactory =
+                    factories.get(entry.partition(), entry.bucket());
+            result.add(dataFilePathFactory.toPath(entry));
         }
 
         // Add 'DELETE' 'APPEND' file in snapshot
@@ -666,10 +687,9 @@ public class TestFileStore extends KeyValueFileStore {
                 if (entry.kind() == FileKind.DELETE
                         && entry.file().fileSource().orElse(FileSource.APPEND)
                                 == FileSource.APPEND) {
-                    result.add(
-                            new Path(
-                                    pathFactory.bucketPath(entry.partition(), entry.bucket()),
-                                    entry.file().fileName()));
+                    DataFilePathFactory dataFilePathFactory =
+                            factories.get(entry.partition(), entry.bucket());
+                    result.add(dataFilePathFactory.toPath(entry));
                 }
             }
         }
@@ -680,6 +700,7 @@ public class TestFileStore extends KeyValueFileStore {
     private static Set<Path> getChangelogFileInUse(
             long changelogId,
             SnapshotManager snapshotManager,
+            ChangelogManager changelogManager,
             FileIO fileIO,
             FileStorePathFactory pathFactory,
             ManifestList manifestList,
@@ -688,12 +709,13 @@ public class TestFileStore extends KeyValueFileStore {
         SchemaManager schemaManager = new SchemaManager(fileIO, snapshotManager.tablePath());
         CoreOptions options = new CoreOptions(schemaManager.latest().get().options());
 
-        Path changelogPath = snapshotManager.longLivedChangelogPath(changelogId);
+        Path changelogPath = changelogManager.longLivedChangelogPath(changelogId);
         Changelog changelog = Changelog.fromPath(fileIO, changelogPath);
 
         // changelog file
         result.add(changelogPath);
 
+        Map<Pair<BinaryRow, Integer>, DataFilePathFactory> dataFilePathFactoryMap = new HashMap<>();
         // data file
         // not all manifests contains useful data file
         // (1) produceChangelog = 'true': data file in changelog manifests
@@ -716,10 +738,14 @@ public class TestFileStore extends KeyValueFileStore {
                             .collect(Collectors.toList());
             for (ManifestEntry entry : files) {
                 if (entry.file().fileSource().orElse(FileSource.APPEND) == FileSource.APPEND) {
-                    result.add(
-                            new Path(
-                                    pathFactory.bucketPath(entry.partition(), entry.bucket()),
-                                    entry.file().fileName()));
+                    Pair<BinaryRow, Integer> bucket = Pair.of(entry.partition(), entry.bucket());
+                    DataFilePathFactory dataFilePathFactory =
+                            dataFilePathFactoryMap.computeIfAbsent(
+                                    bucket,
+                                    b ->
+                                            pathFactory.createDataFilePathFactory(
+                                                    entry.partition(), entry.bucket()));
+                    result.add(dataFilePathFactory.toPath(entry));
                 }
             }
         } else if (changelog.changelogManifestList() != null) {
@@ -731,10 +757,14 @@ public class TestFileStore extends KeyValueFileStore {
                             .flatMap(m -> manifestFile.read(m.fileName()).stream())
                             .collect(Collectors.toList());
             for (ManifestEntry entry : files) {
-                result.add(
-                        new Path(
-                                pathFactory.bucketPath(entry.partition(), entry.bucket()),
-                                entry.file().fileName()));
+                Pair<BinaryRow, Integer> bucket = Pair.of(entry.partition(), entry.bucket());
+                DataFilePathFactory dataFilePathFactory =
+                        dataFilePathFactoryMap.computeIfAbsent(
+                                bucket,
+                                b ->
+                                        pathFactory.createDataFilePathFactory(
+                                                entry.partition(), entry.bucket()));
+                result.add(dataFilePathFactory.toPath(entry));
             }
         }
         return result;
