@@ -93,7 +93,6 @@ public class AppendOnlyWriterTest {
     private static final String PART = "2022-05-01";
     private static final long SCHEMA_ID = 0L;
     private static final int MIN_FILE_NUM = 3;
-    private static final int MAX_FILE_NUM = 4;
 
     @BeforeEach
     public void before() {
@@ -141,6 +140,8 @@ public class AppendOnlyWriterTest {
         assertThat(meta.minSequenceNumber()).isEqualTo(0);
         assertThat(meta.maxSequenceNumber()).isEqualTo(0);
         assertThat(meta.level()).isEqualTo(DataFileMeta.DUMMY_LEVEL);
+        assertThat(meta.fileName().substring(meta.fileName().lastIndexOf(".") + 1))
+                .isEqualTo(CoreOptions.FILE_FORMAT_AVRO);
     }
 
     @Test
@@ -160,8 +161,8 @@ public class AppendOnlyWriterTest {
 
             writer.sync();
             CommitIncrement inc = writer.prepareCommit(true);
-            if (txn > 0 && txn % 3 == 0) {
-                assertThat(inc.compactIncrement().compactBefore()).hasSize(4);
+            if (txn > 0 && txn % 2 == 0) {
+                assertThat(inc.compactIncrement().compactBefore()).hasSize(3);
                 assertThat(inc.compactIncrement().compactAfter()).hasSize(1);
                 DataFileMeta compactAfter = inc.compactIncrement().compactAfter().get(0);
                 assertThat(compactAfter.fileName()).startsWith("compact-");
@@ -267,7 +268,7 @@ public class AppendOnlyWriterTest {
         List<DataFileMeta> compactAfter = secInc.compactIncrement().compactAfter();
         assertThat(compactBefore)
                 .containsExactlyInAnyOrderElementsOf(
-                        firstInc.newFilesIncrement().newFiles().subList(0, 4));
+                        firstInc.newFilesIncrement().newFiles().subList(0, 3));
         assertThat(compactAfter).hasSize(1);
         assertThat(compactBefore.stream().mapToLong(DataFileMeta::fileSize).sum())
                 .isEqualTo(compactAfter.stream().mapToLong(DataFileMeta::fileSize).sum());
@@ -407,6 +408,54 @@ public class AppendOnlyWriterTest {
         writer.close();
     }
 
+    @Test
+    public void tesWriteBufferSpillAutoEnabled() {
+        HashMap<String, String> map = new HashMap<>();
+        // This is the default behavior,no object store and streaming mode.
+        Assertions.assertThat(CoreOptions.fromMap(map).writeBufferSpillable(false, true, false))
+                .isFalse();
+
+        // Using object store.
+        Assertions.assertThat(CoreOptions.fromMap(map).writeBufferSpillable(true, true, false))
+                .isTrue();
+
+        // Batch mode.
+        Assertions.assertThat(CoreOptions.fromMap(map).writeBufferSpillable(false, false, false))
+                .isTrue();
+
+        // Append only table.
+        map.put(CoreOptions.WRITE_BUFFER_SIZE.key(), "200 MB");
+        Assertions.assertThat(CoreOptions.fromMap(map).writeBufferSpillable(false, false, false))
+                .isTrue();
+
+        // Primary key table.
+        map.put(CoreOptions.WRITE_BUFFER_SIZE.key(), "100 MB");
+        Assertions.assertThat(CoreOptions.fromMap(map).writeBufferSpillable(false, false, true))
+                .isTrue();
+
+        // targetFileSize is greater than write buffer size.
+        map.clear();
+        map.put(CoreOptions.TARGET_FILE_SIZE.key(), "2 b");
+        map.put(CoreOptions.WRITE_BUFFER_SIZE.key(), "1 b");
+        Assertions.assertThat(CoreOptions.fromMap(map).writeBufferSpillable(false, true, false))
+                .isTrue();
+
+        // target-file-size is smaller than write-buffer-size.
+        map.clear();
+        map.put(CoreOptions.TARGET_FILE_SIZE.key(), "1 b");
+        map.put(CoreOptions.WRITE_BUFFER_SIZE.key(), "2 b");
+        Assertions.assertThat(CoreOptions.fromMap(map).writeBufferSpillable(false, true, false))
+                .isFalse();
+
+        // Set to false manually.
+        map.clear();
+        map.put(CoreOptions.TARGET_FILE_SIZE.key(), "2 b");
+        map.put(CoreOptions.WRITE_BUFFER_SIZE.key(), "1 b");
+        map.put(CoreOptions.WRITE_BUFFER_SPILLABLE.key(), "false");
+        Assertions.assertThat(CoreOptions.fromMap(map).writeBufferSpillable(false, true, false))
+                .isFalse();
+    }
+
     @Test
     public void testMultipleFlush() throws Exception {
         AppendOnlyWriter writer = createEmptyWriter(Long.MAX_VALUE, true);
@@ -519,11 +568,12 @@ public class AppendOnlyWriterTest {
     private DataFilePathFactory createPathFactory() {
         return new DataFilePathFactory(
                 new Path(tempDir + "/dt=" + PART + "/bucket-0"),
-                CoreOptions.FILE_FORMAT.defaultValue().toString(),
+                CoreOptions.FILE_FORMAT_AVRO,
                 CoreOptions.DATA_FILE_PREFIX.defaultValue(),
                 CoreOptions.CHANGELOG_FILE_PREFIX.defaultValue(),
                 CoreOptions.FILE_SUFFIX_INCLUDE_COMPRESSION.defaultValue(),
-                CoreOptions.FILE_COMPRESSION.defaultValue());
+                CoreOptions.FILE_COMPRESSION.defaultValue(),
+                null);
     }
 
     private AppendOnlyWriter createEmptyWriter(long targetFileSize) {
@@ -601,7 +651,6 @@ public class AppendOnlyWriterTest {
                         toCompact,
                         null,
                         MIN_FILE_NUM,
-                        MAX_FILE_NUM,
                         targetFileSize,
                         compactBefore -> {
                             latch.await();
@@ -631,7 +680,9 @@ public class AppendOnlyWriterTest {
                         CoreOptions.FILE_COMPRESSION.defaultValue(),
                         CompressOptions.defaultOptions(),
                         StatsCollectorFactories.createStatsFactories(
-                                options, AppendOnlyWriterTest.SCHEMA.getFieldNames()),
+                                "truncate(16)",
+                                options,
+                                AppendOnlyWriterTest.SCHEMA.getFieldNames()),
                         MemorySize.MAX_VALUE,
                         new FileIndexOptions(),
                         true,
