@@ -45,6 +45,78 @@ import static org.assertj.core.api.Assertions.assertThat;
 /** Compatibility Test for {@link ManifestCommittableSerializer}. */
 public class ManifestCommittableSerializerCompatibilityTest {
 
+    @Test
+    public void testCompatibilityToV3CommitV7() throws IOException {
+        SimpleStats keyStats =
+                new SimpleStats(
+                        singleColumn("min_key"),
+                        singleColumn("max_key"),
+                        fromLongArray(new Long[] {0L}));
+        SimpleStats valueStats =
+                new SimpleStats(
+                        singleColumn("min_value"),
+                        singleColumn("max_value"),
+                        fromLongArray(new Long[] {0L}));
+        DataFileMeta dataFile =
+                new DataFileMeta(
+                        "my_file",
+                        1024 * 1024,
+                        1024,
+                        singleColumn("min_key"),
+                        singleColumn("max_key"),
+                        keyStats,
+                        valueStats,
+                        15,
+                        200,
+                        5,
+                        3,
+                        Arrays.asList("extra1", "extra2"),
+                        Timestamp.fromLocalDateTime(LocalDateTime.parse("2022-03-02T20:20:12")),
+                        11L,
+                        new byte[] {1, 2, 4},
+                        FileSource.COMPACT,
+                        Arrays.asList("field1", "field2", "field3"),
+                        "hdfs://localhost:9000/path/to/file");
+        List<DataFileMeta> dataFiles = Collections.singletonList(dataFile);
+
+        LinkedHashMap<String, DeletionVectorMeta> dvMetas = new LinkedHashMap<>();
+        dvMetas.put("dv_key1", new DeletionVectorMeta("dv_key1", 1, 2, 3L));
+        dvMetas.put("dv_key2", new DeletionVectorMeta("dv_key2", 3, 4, 5L));
+        IndexFileMeta indexFile =
+                new IndexFileMeta("my_index_type", "my_index_file", 1024 * 100, 1002, dvMetas);
+        List<IndexFileMeta> indexFiles = Collections.singletonList(indexFile);
+
+        CommitMessageImpl commitMessage =
+                new CommitMessageImpl(
+                        singleColumn("my_partition"),
+                        11,
+                        16,
+                        new DataIncrement(dataFiles, dataFiles, dataFiles),
+                        new CompactIncrement(dataFiles, dataFiles, dataFiles),
+                        new IndexIncrement(indexFiles));
+
+        ManifestCommittable manifestCommittable =
+                new ManifestCommittable(
+                        5,
+                        202020L,
+                        Collections.singletonMap(5, 555L),
+                        Collections.singletonList(commitMessage));
+
+        ManifestCommittableSerializer serializer = new ManifestCommittableSerializer();
+        byte[] bytes = serializer.serialize(manifestCommittable);
+        ManifestCommittable deserialized = serializer.deserialize(3, bytes);
+        assertThat(deserialized).isEqualTo(manifestCommittable);
+
+        byte[] oldBytes =
+                IOUtils.readFully(
+                        ManifestCommittableSerializerCompatibilityTest.class
+                                .getClassLoader()
+                                .getResourceAsStream("compatibility/manifest-committable-v7"),
+                        true);
+        deserialized = serializer.deserialize(3, oldBytes);
+        assertThat(deserialized).isEqualTo(manifestCommittable);
+    }
+
     @Test
     public void testCompatibilityToV3CommitV6() throws IOException {
         SimpleStats keyStats =
@@ -90,6 +162,7 @@ public class ManifestCommittableSerializerCompatibilityTest {
                 new CommitMessageImpl(
                         singleColumn("my_partition"),
                         11,
+                        null,
                         new DataIncrement(dataFiles, dataFiles, dataFiles),
                         new CompactIncrement(dataFiles, dataFiles, dataFiles),
                         new IndexIncrement(indexFiles));
@@ -161,6 +234,7 @@ public class ManifestCommittableSerializerCompatibilityTest {
                 new CommitMessageImpl(
                         singleColumn("my_partition"),
                         11,
+                        null,
                         new DataIncrement(dataFiles, dataFiles, dataFiles),
                         new CompactIncrement(dataFiles, dataFiles, dataFiles),
                         new IndexIncrement(indexFiles));
@@ -231,6 +305,7 @@ public class ManifestCommittableSerializerCompatibilityTest {
                 new CommitMessageImpl(
                         singleColumn("my_partition"),
                         11,
+                        null,
                         new DataIncrement(dataFiles, dataFiles, dataFiles),
                         new CompactIncrement(dataFiles, dataFiles, dataFiles),
                         new IndexIncrement(indexFiles));
@@ -302,6 +377,7 @@ public class ManifestCommittableSerializerCompatibilityTest {
                 new CommitMessageImpl(
                         singleColumn("my_partition"),
                         11,
+                        null,
                         new DataIncrement(dataFiles, dataFiles, dataFiles),
                         new CompactIncrement(dataFiles, dataFiles, dataFiles),
                         new IndexIncrement(indexFiles));
@@ -373,6 +449,7 @@ public class ManifestCommittableSerializerCompatibilityTest {
                 new CommitMessageImpl(
                         singleColumn("my_partition"),
                         11,
+                        null,
                         new DataIncrement(dataFiles, dataFiles, dataFiles),
                         new CompactIncrement(dataFiles, dataFiles, dataFiles),
                         new IndexIncrement(indexFiles));
@@ -441,6 +518,7 @@ public class ManifestCommittableSerializerCompatibilityTest {
                 new CommitMessageImpl(
                         singleColumn("my_partition"),
                         11,
+                        null,
                         new DataIncrement(dataFiles, Collections.emptyList(), dataFiles),
                         new CompactIncrement(dataFiles, dataFiles, dataFiles),
                         new IndexIncrement(indexFiles));
