@@ -21,6 +21,7 @@ package org.apache.paimon.mergetree.compact;
 import org.apache.paimon.KeyValue;
 import org.apache.paimon.data.GenericRow;
 import org.apache.paimon.options.Options;
+import org.apache.paimon.types.DataType;
 import org.apache.paimon.types.DataTypes;
 import org.apache.paimon.types.RowKind;
 import org.apache.paimon.types.RowType;
@@ -31,6 +32,7 @@ import org.apache.paimon.shade.guava30.com.google.common.collect.ImmutableList;
 import org.junit.jupiter.api.Test;
 
 import static org.apache.paimon.CoreOptions.FIELDS_DEFAULT_AGG_FUNC;
+import static org.apache.paimon.testutils.assertj.PaimonAssertions.anyCauseMatches;
 import static org.assertj.core.api.Assertions.assertThat;
 import static org.assertj.core.api.Assertions.assertThatThrownBy;
 
@@ -314,80 +316,6 @@ public class PartialUpdateMergeFunctionTest {
                 .hasMessageContaining("is defined repeatedly by multiple groups");
     }
 
-    @Test
-    public void testAdjustProjectionRepeatProject() {
-        Options options = new Options();
-        options.set("fields.f4.sequence-group", "f1,f3");
-        options.set("fields.f5.sequence-group", "f7");
-        RowType rowType =
-                RowType.of(
-                        DataTypes.INT(),
-                        DataTypes.INT(),
-                        DataTypes.INT(),
-                        DataTypes.INT(),
-                        DataTypes.INT(),
-                        DataTypes.INT(),
-                        DataTypes.INT(),
-                        DataTypes.INT());
-        // the field 'f1' is projected twice
-        int[][] projection = new int[][] {{1}, {1}, {3}, {7}};
-        MergeFunctionFactory<KeyValue> factory =
-                PartialUpdateMergeFunction.factory(options, rowType, ImmutableList.of("f0"));
-        MergeFunctionFactory.AdjustedProjection adjustedProjection =
-                factory.adjustProjection(projection);
-
-        validate(adjustedProjection, new int[] {1, 1, 3, 7, 4, 5}, new int[] {0, 1, 2, 3});
-
-        MergeFunction<KeyValue> func = factory.create(adjustedProjection.pushdownProjection);
-        func.reset();
-        add(func, 1, 1, 1, 1, 1, 1);
-        add(func, 2, 2, 6, 2, 2, 2);
-        validate(func, 2, 2, 6, 2, 2, 2);
-
-        // enable field updated by null
-        add(func, 3, 3, null, 7, 4, null);
-        validate(func, 3, 3, null, 2, 4, 2);
-    }
-
-    @Test
-    public void testMultiSequenceFieldsAdjustProjectionRepeatProject() {
-        Options options = new Options();
-        options.set("fields.f2,f4.sequence-group", "f1,f3");
-        options.set("fields.f5,f6.sequence-group", "f7");
-        RowType rowType =
-                RowType.of(
-                        DataTypes.INT(),
-                        DataTypes.INT(),
-                        DataTypes.INT(),
-                        DataTypes.INT(),
-                        DataTypes.INT(),
-                        DataTypes.INT(),
-                        DataTypes.INT(),
-                        DataTypes.INT());
-        // the field 'f1' is projected twice
-        int[][] projection = new int[][] {{1}, {1}, {3}, {7}};
-        MergeFunctionFactory<KeyValue> factory =
-                PartialUpdateMergeFunction.factory(options, rowType, ImmutableList.of("f0"));
-        MergeFunctionFactory.AdjustedProjection adjustedProjection =
-                factory.adjustProjection(projection);
-
-        validate(adjustedProjection, new int[] {1, 1, 3, 7, 2, 4, 5, 6}, new int[] {0, 1, 2, 3});
-
-        MergeFunction<KeyValue> func = factory.create(adjustedProjection.pushdownProjection);
-        func.reset();
-        add(func, 1, 1, 1, 1, 1, 1, 1, 1);
-        add(func, 2, 2, 6, 2, 2, 2, 2, 6);
-        validate(func, 2, 2, 6, 2, 2, 2, 2, 6);
-
-        // update first sequence group
-        add(func, 3, 3, null, 7, 4, null, 1, 8);
-        validate(func, 3, 3, null, 2, 4, null, 2, 6);
-
-        // update second sequence group
-        add(func, 5, 5, 3, 3, 3, 5, 5, 6);
-        validate(func, 5, 3, null, 3, 4, null, 5, 6);
-    }
-
     @Test
     public void testAdjustProjectionSequenceFieldsProject() {
         Options options = new Options();
@@ -894,14 +822,64 @@ public class PartialUpdateMergeFunctionTest {
 
     @Test
     public void testAggregationWithoutSequenceGroup() {
-        Options options = new Options();
-        options.set("fields.f1.aggregate-function", "listagg");
-        RowType rowType = RowType.of(DataTypes.INT(), DataTypes.INT());
+        RowType rowType =
+                RowType.of(
+                        new DataType[] {
+                            DataTypes.INT(),
+                            DataTypes.INT(),
+                            DataTypes.INT(),
+                            DataTypes.INT(),
+                            DataTypes.INT()
+                        },
+                        new String[] {"pk", "f0", "g0", "f1", "g1"});
+
+        Options options1 = new Options();
+        options1.set("fields.f0.aggregate-function", "listagg");
+        options1.set("fields.f1.aggregate-function", "listagg");
         assertThatThrownBy(
                         () ->
                                 PartialUpdateMergeFunction.factory(
-                                        options, rowType, ImmutableList.of("f0")))
-                .hasMessageContaining("Must use sequence group for aggregation functions");
+                                        options1, rowType, ImmutableList.of("pk")))
+                .satisfies(
+                        anyCauseMatches(
+                                IllegalArgumentException.class,
+                                "Must use sequence group for aggregation functions but not found for field f0."));
+
+        Options options2 = new Options(options1.toMap());
+        options2.set("fields.g0.sequence-group", "f0");
+        assertThatThrownBy(
+                        () ->
+                                PartialUpdateMergeFunction.factory(
+                                        options2, rowType, ImmutableList.of("pk")))
+                .satisfies(
+                        anyCauseMatches(
+                                IllegalArgumentException.class,
+                                "Must use sequence group for aggregation functions but not found for field f1."));
+    }
+
+    @Test
+    public void testDeleteReproduceCorrectSequenceNumber() {
+        Options options = new Options();
+        options.set("partial-update.remove-record-on-delete", "true");
+        RowType rowType =
+                RowType.of(
+                        DataTypes.INT(),
+                        DataTypes.INT(),
+                        DataTypes.INT(),
+                        DataTypes.INT(),
+                        DataTypes.INT());
+
+        MergeFunctionFactory<KeyValue> factory =
+                PartialUpdateMergeFunction.factory(options, rowType, ImmutableList.of("f0"));
+
+        MergeFunction<KeyValue> func = factory.create();
+
+        func.reset();
+
+        add(func, RowKind.INSERT, 1, 1, 1, 1, 1);
+        add(func, RowKind.DELETE, 1, 1, 1, 1, 1);
+
+        assertThat(func.getResult().sequenceNumber()).isEqualTo(1);
     }
 
     private void add(MergeFunction<KeyValue> function, Integer... f) {
