@@ -27,6 +27,8 @@ import org.apache.paimon.data.BinaryRow;
 import org.apache.paimon.fs.FileIO;
 import org.apache.paimon.fs.Path;
 import org.apache.paimon.fs.local.LocalFileIO;
+import org.apache.paimon.index.IndexFileHandler;
+import org.apache.paimon.manifest.ExpireFileEntry;
 import org.apache.paimon.manifest.FileKind;
 import org.apache.paimon.manifest.ManifestCommittable;
 import org.apache.paimon.manifest.ManifestEntry;
@@ -38,9 +40,11 @@ import org.apache.paimon.options.ExpireConfig;
 import org.apache.paimon.schema.Schema;
 import org.apache.paimon.schema.SchemaManager;
 import org.apache.paimon.schema.TableSchema;
+import org.apache.paimon.stats.StatsFileHandler;
 import org.apache.paimon.table.ExpireSnapshots;
 import org.apache.paimon.table.ExpireSnapshotsImpl;
 import org.apache.paimon.types.RowType;
+import org.apache.paimon.utils.ChangelogManager;
 import org.apache.paimon.utils.FileStorePathFactory;
 import org.apache.paimon.utils.RecordWriter;
 import org.apache.paimon.utils.SnapshotManager;
@@ -52,25 +56,28 @@ import org.junit.jupiter.api.io.TempDir;
 import org.junit.jupiter.params.ParameterizedTest;
 import org.junit.jupiter.params.provider.ValueSource;
 
+import java.io.IOException;
 import java.nio.file.Files;
 import java.nio.file.Paths;
 import java.time.Duration;
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Collection;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.Set;
 import java.util.concurrent.ThreadLocalRandom;
 import java.util.stream.Collectors;
 
 import static org.apache.paimon.CoreOptions.SNAPSHOT_CLEAN_EMPTY_DIRECTORIES;
 import static org.apache.paimon.operation.FileStoreCommitImpl.mustConflictCheck;
+import static org.apache.paimon.operation.FileStoreTestUtils.assertNFilesExists;
 import static org.apache.paimon.operation.FileStoreTestUtils.assertPathExists;
 import static org.apache.paimon.operation.FileStoreTestUtils.assertPathNotExists;
 import static org.apache.paimon.operation.FileStoreTestUtils.commitData;
 import static org.apache.paimon.operation.FileStoreTestUtils.partitionedData;
-import static org.apache.paimon.utils.BranchManager.DEFAULT_MAIN_BRANCH;
 import static org.assertj.core.api.Assertions.assertThat;
 
 /**
@@ -630,7 +637,7 @@ public class FileDeletionTest {
         List<ManifestFileMeta> newManifests =
                 Arrays.asList(newAddManifests.get(0), newDeleteManifests.get(0));
 
-        String newManifestListName = manifestList.write(newManifests);
+        String newManifestListName = manifestList.write(newManifests).getKey();
 
         fileIO.rename(
                 pathFactory.toManifestListPath(newManifestListName),
@@ -647,6 +654,7 @@ public class FileDeletionTest {
         TestFileStore store = createStore(TestKeyValueGenerator.GeneratorMode.NON_PARTITIONED, 2);
         tagManager = new TagManager(fileIO, store.options().path());
         SnapshotManager snapshotManager = store.snapshotManager();
+        ChangelogManager changelogManager = store.changelogManager();
         TestKeyValueGenerator gen =
                 new TestKeyValueGenerator(TestKeyValueGenerator.GeneratorMode.NON_PARTITIONED);
         BinaryRow partition = gen.getPartition(gen.next());
@@ -674,7 +682,8 @@ public class FileDeletionTest {
         // action: expire snapshot 1 -> delete tag1 -> expire snapshot 2
         // result: exist A & B (because of tag2)
         ExpireSnapshots expireSnapshots =
-                new ExpireSnapshotsImpl(snapshotManager, store.newSnapshotDeletion(), tagManager);
+                new ExpireSnapshotsImpl(
+                        snapshotManager, changelogManager, store.newSnapshotDeletion(), tagManager);
         expireSnapshots
                 .config(
                         ExpireConfig.builder()
@@ -700,6 +709,130 @@ public class FileDeletionTest {
         assertPathExists(fileIO, pathFactory.bucketPath(partition, 1));
     }
 
+    @Test
+    public void testDataFileSkippingSetException() throws Exception {
+        TestFileStore store = createStore(TestKeyValueGenerator.GeneratorMode.NON_PARTITIONED, 2);
+        tagManager = new TagManager(fileIO, store.options().path());
+        SnapshotManager snapshotManager = store.snapshotManager();
+        ChangelogManager changelogManager = store.changelogManager();
+        TestKeyValueGenerator gen =
+                new TestKeyValueGenerator(TestKeyValueGenerator.GeneratorMode.NON_PARTITIONED);
+        BinaryRow partition = gen.getPartition(gen.next());
+
+        // step 1: commit A to bucket 0
+        Map<BinaryRow, Map<Integer, RecordWriter<KeyValue>>> writers = new HashMap<>();
+        List<KeyValue> kvs = partitionedData(5, gen);
+        writeData(store, kvs, partition, 0, writers);
+        commitData(store, commitIdentifier++, writers);
+
+        // step 2: commit B to bucket 1 and create tag2
+        writers.clear();
+        kvs = partitionedData(5, gen);
+        writeData(store, kvs, partition, 1, writers);
+        commitData(store, commitIdentifier++, writers);
+        createTag(snapshotManager.snapshot(2), "tag2", store.options().tagDefaultTimeRetained());
+
+        // step 3: commit -B
+        cleanBucket(store, partition, 1);
+
+        // step 4: commit -A
+        cleanBucket(store, partition, 0);
+
+        // action: expire snapshot 1-3 and throw artificial exception when constructing skipping set
+        // from tag2
+        // result: exist A & B (because of tag2)
+        TestSnapshotDeletion snapshotDeletion =
+                new TestSnapshotDeletion(
+                        store.fileIO(),
+                        store.pathFactory(),
+                        store.manifestFileFactory().create(),
+                        store.manifestListFactory().create(),
+                        store.newIndexFileHandler(),
+                        store.newStatsFileHandler(),
+                        store.options().changelogProducer() != CoreOptions.ChangelogProducer.NONE,
+                        store.options().cleanEmptyDirectories(),
+                        store.options().deleteFileThreadNum());
+
+        ExpireSnapshots expireSnapshots =
+                new ExpireSnapshotsImpl(
+                        snapshotManager, changelogManager, snapshotDeletion, tagManager);
+        snapshotDeletion.readMergedDataFilesThrowException = true;
+        expireSnapshots
+                .config(
+                        ExpireConfig.builder()
+                                .snapshotRetainMax(1)
+                                .snapshotRetainMin(1)
+                                .snapshotTimeRetain(Duration.ofMillis(Long.MAX_VALUE))
+                                .build())
+                .expire();
+
+        FileStorePathFactory pathFactory = store.pathFactory();
+        assertNFilesExists(fileIO, pathFactory.bucketPath(partition, 0), 1);
+        assertNFilesExists(fileIO, pathFactory.bucketPath(partition, 1), 1);
+    }
+
+    @Test
+    public void testManifestFileSkippingSetException() throws Exception {
+        TestFileStore store = createStore(TestKeyValueGenerator.GeneratorMode.NON_PARTITIONED, 2);
+        tagManager = new TagManager(fileIO, store.options().path());
+        SnapshotManager snapshotManager = store.snapshotManager();
+        ChangelogManager changelogManager = store.changelogManager();
+        TestKeyValueGenerator gen =
+                new TestKeyValueGenerator(TestKeyValueGenerator.GeneratorMode.NON_PARTITIONED);
+        BinaryRow partition = gen.getPartition(gen.next());
+
+        // step 1: commit A to bucket 0
+        Map<BinaryRow, Map<Integer, RecordWriter<KeyValue>>> writers = new HashMap<>();
+        List<KeyValue> kvs = partitionedData(5, gen);
+        writeData(store, kvs, partition, 0, writers);
+        commitData(store, commitIdentifier++, writers);
+
+        // step 2: commit B to bucket 1 and create tag2
+        writers.clear();
+        kvs = partitionedData(5, gen);
+        writeData(store, kvs, partition, 1, writers);
+        commitData(store, commitIdentifier++, writers);
+        createTag(snapshotManager.snapshot(2), "tag2", store.options().tagDefaultTimeRetained());
+
+        // step 3: commit -B
+        cleanBucket(store, partition, 1);
+
+        // step 4: commit -A
+        cleanBucket(store, partition, 0);
+
+        Path manifestPath = store.pathFactory().manifestPath();
+        int manifestFileNum = fileIO.listStatus(manifestPath).length;
+
+        // action: expire snapshot 1-3 and throw artificial exception when constructing manifest
+        // skipping set
+        // result: no manifests were deleted (manifestFileNum)
+        TestSnapshotDeletion snapshotDeletion =
+                new TestSnapshotDeletion(
+                        store.fileIO(),
+                        store.pathFactory(),
+                        store.manifestFileFactory().create(),
+                        store.manifestListFactory().create(),
+                        store.newIndexFileHandler(),
+                        store.newStatsFileHandler(),
+                        store.options().changelogProducer() != CoreOptions.ChangelogProducer.NONE,
+                        store.options().cleanEmptyDirectories(),
+                        store.options().deleteFileThreadNum());
+        ExpireSnapshots expireSnapshots =
+                new ExpireSnapshotsImpl(
+                        snapshotManager, changelogManager, snapshotDeletion, tagManager);
+        snapshotDeletion.manifestSkippingSetThrowException = true;
+        expireSnapshots
+                .config(
+                        ExpireConfig.builder()
+                                .snapshotRetainMax(1)
+                                .snapshotRetainMin(1)
+                                .snapshotTimeRetain(Duration.ofMillis(Long.MAX_VALUE))
+                                .build())
+                .expire();
+
+        assertNFilesExists(fileIO, manifestPath, manifestFileNum);
+    }
+
     private TestFileStore createStore(TestKeyValueGenerator.GeneratorMode mode) throws Exception {
         return createStore(mode, 2);
     }
@@ -799,7 +932,6 @@ public class FileDeletionTest {
                     Snapshot.CommitKind.APPEND,
                     store.snapshotManager().latestSnapshot(),
                     mustConflictCheck(),
-                    DEFAULT_MAIN_BRANCH,
                     null);
         }
     }
@@ -807,4 +939,49 @@ public class FileDeletionTest {
     private void createTag(Snapshot snapshot, String tagName, Duration timeRetained) {
         tagManager.createTag(snapshot, tagName, timeRetained, Collections.emptyList(), false);
     }
+
+    private static class TestSnapshotDeletion extends SnapshotDeletion {
+
+        private boolean readMergedDataFilesThrowException = false;
+        private boolean manifestSkippingSetThrowException = false;
+
+        public TestSnapshotDeletion(
+                FileIO fileIO,
+                FileStorePathFactory pathFactory,
+                ManifestFile manifestFile,
+                ManifestList manifestList,
+                IndexFileHandler indexFileHandler,
+                StatsFileHandler statsFileHandler,
+                boolean produceChangelog,
+                boolean cleanEmptyDirectories,
+                int deleteFileThreadNum) {
+            super(
+                    fileIO,
+                    pathFactory,
+                    manifestFile,
+                    manifestList,
+                    indexFileHandler,
+                    statsFileHandler,
+                    produceChangelog,
+                    cleanEmptyDirectories,
+                    deleteFileThreadNum);
+        }
+
+        @Override
+        protected Collection<ExpireFileEntry> readMergedDataFiles(List<ManifestFileMeta> manifests)
+                throws IOException {
+            if (readMergedDataFilesThrowException) {
+                throw new IOException("Throwing exception for test purpose.");
+            }
+            return super.readMergedDataFiles(manifests);
+        }
+
+        @Override
+        public Set<String> manifestSkippingSet(List<Snapshot> skippingSnapshots) {
+            if (manifestSkippingSetThrowException) {
+                throw new RuntimeException("Throwing exception for test purpose.");
+            }
+            return super.manifestSkippingSet(skippingSnapshots);
+        }
+    }
 }
