@@ -45,9 +45,10 @@ import java.util.ArrayList;
 import java.util.Collections;
 import java.util.List;
 
+import static org.apache.paimon.SnapshotTest.newSnapshotManager;
 import static org.apache.paimon.utils.FileStorePathFactoryTest.createNonPartFactory;
 import static org.assertj.core.api.Assertions.assertThat;
-import static org.junit.Assert.assertThrows;
+import static org.junit.jupiter.api.Assertions.assertThrows;
 
 /** Unit tests for {@link ManifestsTable}. */
 public class ManifestsTableTest extends TableTestBase {
@@ -76,7 +77,7 @@ public class ManifestsTableTest extends TableTestBase {
 
         FileIO fileIO = LocalFileIO.create();
         Path tablePath = new Path(String.format("%s/%s.db/%s", warehouse, database, "T"));
-        snapshotManager = new SnapshotManager(fileIO, tablePath);
+        snapshotManager = newSnapshotManager(fileIO, tablePath);
 
         ManifestList.Factory factory =
                 new ManifestList.Factory(
@@ -156,9 +157,9 @@ public class ManifestsTableTest extends TableTestBase {
                         manifestsTable.copy(
                                 Collections.singletonMap(CoreOptions.SCAN_SNAPSHOT_ID.key(), "3"));
         assertThrows(
-                "Specified parameter scan.snapshot-id = 3 is not exist, you can set it in range from 1 to 2",
                 SnapshotNotExistException.class,
-                () -> read(manifestsTable));
+                () -> read(manifestsTable),
+                "Specified parameter scan.snapshot-id = 3 is not exist, you can set it in range from 1 to 2");
     }
 
     private List<InternalRow> getExpectedResult(long snapshotId) {
@@ -180,14 +181,14 @@ public class ManifestsTableTest extends TableTestBase {
                             manifestFileMeta.schemaId(),
                             BinaryString.fromString(
                                     String.format(
-                                            "[%d]",
+                                            "{%d}",
                                             manifestFileMeta
                                                     .partitionStats()
                                                     .minValues()
                                                     .getInt(0))),
                             BinaryString.fromString(
                                     String.format(
-                                            "[%d]",
+                                            "{%d}",
                                             manifestFileMeta
                                                     .partitionStats()
                                                     .maxValues()
