@@ -23,10 +23,15 @@ import org.apache.paimon.Snapshot;
 import org.apache.paimon.fs.FileIO;
 import org.apache.paimon.fs.Path;
 import org.apache.paimon.fs.local.LocalFileIO;
+import org.apache.paimon.table.source.snapshot.TimeTravelUtil;
 
 import org.assertj.core.api.Assertions;
 import org.junit.jupiter.api.Test;
 import org.junit.jupiter.api.io.TempDir;
+import org.junit.jupiter.params.ParameterizedTest;
+import org.junit.jupiter.params.provider.ValueSource;
+
+import javax.annotation.Nullable;
 
 import java.io.IOException;
 import java.util.ArrayList;
@@ -37,6 +42,9 @@ import java.util.concurrent.ThreadLocalRandom;
 import java.util.concurrent.atomic.AtomicReference;
 import java.util.stream.Collectors;
 
+import static org.apache.paimon.SnapshotTest.newChangelogManager;
+import static org.apache.paimon.SnapshotTest.newSnapshotManager;
+import static org.apache.paimon.utils.BranchManager.DEFAULT_MAIN_BRANCH;
 import static org.assertj.core.api.Assertions.assertThat;
 import static org.assertj.core.api.Assertions.fail;
 import static org.junit.jupiter.api.Assertions.assertDoesNotThrow;
@@ -49,15 +57,49 @@ public class SnapshotManagerTest {
     @Test
     public void testSnapshotPath() {
         SnapshotManager snapshotManager =
-                new SnapshotManager(LocalFileIO.create(), new Path(tempDir.toString()));
+                newSnapshotManager(LocalFileIO.create(), new Path(tempDir.toString()));
         for (int i = 0; i < 20; i++) {
             assertThat(snapshotManager.snapshotPath(i))
                     .isEqualTo(new Path(tempDir.toString() + "/snapshot/snapshot-" + i));
         }
     }
 
-    @Test
-    public void testEarlierThanTimeMillis() throws IOException {
+    @ParameterizedTest
+    @ValueSource(booleans = {true, false})
+    public void testEarliestSnapshot(boolean isRaceCondition) throws IOException {
+        long millis = 1684726826L;
+        FileIO localFileIO = LocalFileIO.create();
+        SnapshotManager snapshotManager =
+                new TestSnapshotManager(localFileIO, new Path(tempDir.toString()), isRaceCondition);
+        // create 10 snapshots
+        for (long i = 0; i < 10; i++) {
+            Snapshot snapshot = createSnapshotWithMillis(i, millis + i * 1000);
+            localFileIO.tryToWriteAtomic(snapshotManager.snapshotPath(i), snapshot.toJson());
+        }
+
+        assertThat(snapshotManager.earliestSnapshot().id()).isEqualTo(isRaceCondition ? 1 : 0);
+    }
+
+    @ParameterizedTest
+    @ValueSource(booleans = {true, false})
+    public void testEarlierOrEqualWatermark(boolean isRaceCondition) throws IOException {
+        long millis = 1684726826L;
+        FileIO localFileIO = LocalFileIO.create();
+        SnapshotManager snapshotManager =
+                new TestSnapshotManager(localFileIO, new Path(tempDir.toString()), isRaceCondition);
+        // create 10 snapshots
+        for (long i = 0; i < 10; i++) {
+            Snapshot snapshot = createSnapshotWithMillis(i, millis + i * 1000, millis + i * 1000);
+            localFileIO.tryToWriteAtomic(snapshotManager.snapshotPath(i), snapshot.toJson());
+        }
+
+        assertThat(snapshotManager.earlierOrEqualWatermark(millis + 999).id())
+                .isEqualTo(isRaceCondition ? 1 : 0);
+    }
+
+    @ParameterizedTest
+    @ValueSource(booleans = {true, false})
+    public void testEarlierThanTimeMillis(boolean isRaceCondition) throws IOException {
         long base = System.currentTimeMillis();
         ThreadLocalRandom random = ThreadLocalRandom.current();
 
@@ -70,7 +112,7 @@ public class SnapshotManagerTest {
 
         FileIO localFileIO = LocalFileIO.create();
         SnapshotManager snapshotManager =
-                new SnapshotManager(localFileIO, new Path(tempDir.toString()));
+                new TestSnapshotManager(localFileIO, new Path(tempDir.toString()), isRaceCondition);
         int firstSnapshotId = random.nextInt(1, 100);
         for (int i = 0; i < numSnapshots; i++) {
             Snapshot snapshot = createSnapshotWithMillis(firstSnapshotId + i, millis.get(i));
@@ -87,14 +129,32 @@ public class SnapshotManagerTest {
                 // pick a random time equal to one of the snapshots
                 time = millis.get(random.nextInt(numSnapshots));
             }
-            Long actual = snapshotManager.earlierThanTimeMills(time, false);
+            Long actual =
+                    TimeTravelUtil.earlierThanTimeMills(snapshotManager, null, time, false, false);
 
             if (millis.get(numSnapshots - 1) < time) {
-                assertThat(actual).isEqualTo(firstSnapshotId + numSnapshots - 1);
+                if (isRaceCondition && millis.size() == 1) {
+                    if (tries == 0) {
+                        assertThat(actual).isLessThanOrEqualTo(firstSnapshotId);
+                    } else {
+                        assertThat(actual).isNull();
+                    }
+                } else {
+                    assertThat(actual).isEqualTo(firstSnapshotId + numSnapshots - 1);
+                }
             } else {
                 for (int i = 0; i < numSnapshots; i++) {
                     if (millis.get(i) >= time) {
-                        assertThat(actual).isEqualTo(firstSnapshotId + i - 1);
+                        if (isRaceCondition && i == 0) {
+                            // The first snapshot expired during invocation
+                            if (millis.size() == 1 && tries > 0) {
+                                assertThat(actual).isNull();
+                            } else {
+                                assertThat(actual).isLessThanOrEqualTo(firstSnapshotId);
+                            }
+                        } else {
+                            assertThat(actual).isLessThanOrEqualTo(firstSnapshotId + i - 1);
+                        }
                         break;
                     }
                 }
@@ -102,31 +162,43 @@ public class SnapshotManagerTest {
         }
     }
 
-    @Test
-    public void testEarlierOrEqualTimeMills() throws IOException {
+    @ParameterizedTest
+    @ValueSource(booleans = {true, false})
+    public void testEarlierOrEqualTimeMills(boolean isRaceCondition) throws IOException {
         long millis = 1684726826L;
         FileIO localFileIO = LocalFileIO.create();
         SnapshotManager snapshotManager =
-                new SnapshotManager(localFileIO, new Path(tempDir.toString()));
+                new TestSnapshotManager(localFileIO, new Path(tempDir.toString()), isRaceCondition);
         // create 10 snapshots
         for (long i = 0; i < 10; i++) {
             Snapshot snapshot = createSnapshotWithMillis(i, millis + i * 1000);
             localFileIO.tryToWriteAtomic(snapshotManager.snapshotPath(i), snapshot.toJson());
         }
 
-        // there is no snapshot smaller than "millis - 1L" return the earliest snapshot
-        assertThat(snapshotManager.earlierOrEqualTimeMills(millis - 1L).timeMillis())
-                .isEqualTo(millis);
+        if (isRaceCondition) {
+            // The earliest snapshot has expired, so always return the second snapshot, smaller than
+            // the second snapshot return null
+            assertThat(snapshotManager.earlierOrEqualTimeMills(millis - 1L)).isEqualTo(null);
+            assertThat(snapshotManager.earlierOrEqualTimeMills(millis + 999)).isEqualTo(null);
+            assertThat(snapshotManager.earlierOrEqualTimeMills(millis + 1000).timeMillis())
+                    .isEqualTo(millis + 1000L);
+            assertThat(snapshotManager.earlierOrEqualTimeMills(millis + 1001).timeMillis())
+                    .isEqualTo(millis + 1000L);
+        } else {
+            // there is no snapshot smaller than "millis - 1L" return null
+            assertThat(snapshotManager.earlierOrEqualTimeMills(millis - 1L)).isEqualTo(null);
 
-        // smaller than the second snapshot return the first snapshot
-        assertThat(snapshotManager.earlierOrEqualTimeMills(millis + 999).timeMillis())
-                .isEqualTo(millis);
-        // equal to the second snapshot return the second snapshot
-        assertThat(snapshotManager.earlierOrEqualTimeMills(millis + 1000).timeMillis())
-                .isEqualTo(millis + 1000);
-        // larger than the second snapshot return the second snapshot
-        assertThat(snapshotManager.earlierOrEqualTimeMills(millis + 1001).timeMillis())
-                .isEqualTo(millis + 1000);
+            // smaller than the second snapshot return the first snapshot
+            assertThat(snapshotManager.earlierOrEqualTimeMills(millis + 999).timeMillis())
+                    .isEqualTo(millis);
+
+            // equal to the second snapshot return the second snapshot
+            assertThat(snapshotManager.earlierOrEqualTimeMills(millis + 1000).timeMillis())
+                    .isEqualTo(millis + 1000);
+            // larger than the second snapshot return the second snapshot
+            assertThat(snapshotManager.earlierOrEqualTimeMills(millis + 1001).timeMillis())
+                    .isEqualTo(millis + 1000);
+        }
     }
 
     @Test
@@ -134,7 +206,7 @@ public class SnapshotManagerTest {
         long millis = 1684726826L;
         FileIO localFileIO = LocalFileIO.create();
         SnapshotManager snapshotManager =
-                new SnapshotManager(localFileIO, new Path(tempDir.toString()));
+                newSnapshotManager(localFileIO, new Path(tempDir.toString()));
         // create 10 snapshots
         for (long i = 0; i < 10; i++) {
             Snapshot snapshot = createSnapshotWithMillis(i, millis + i * 1000);
@@ -154,12 +226,13 @@ public class SnapshotManagerTest {
         assertThat(snapshotManager.laterOrEqualTimeMills(millis + 10001)).isNull();
     }
 
-    @Test
-    public void testlaterOrEqualWatermark() throws IOException {
+    @ParameterizedTest
+    @ValueSource(booleans = {true, false})
+    public void testLaterOrEqualWatermark(boolean isRaceCondition) throws IOException {
         long millis = Long.MIN_VALUE;
         FileIO localFileIO = LocalFileIO.create();
         SnapshotManager snapshotManager =
-                new SnapshotManager(localFileIO, new Path(tempDir.toString()));
+                new TestSnapshotManager(localFileIO, new Path(tempDir.toString()), isRaceCondition);
         // create 10 snapshots
         for (long i = 0; i < 10; i++) {
             Snapshot snapshot = createSnapshotWithMillis(i, millis, Long.MIN_VALUE);
@@ -169,7 +242,7 @@ public class SnapshotManagerTest {
         assertThat(snapshotManager.laterOrEqualWatermark(millis + 999)).isNull();
     }
 
-    private Snapshot createSnapshotWithMillis(long id, long millis) {
+    public static Snapshot createSnapshotWithMillis(long id, long millis) {
         return new Snapshot(
                 id,
                 0L,
@@ -178,6 +251,9 @@ public class SnapshotManagerTest {
                 null,
                 null,
                 null,
+                null,
+                null,
+                null,
                 0L,
                 Snapshot.CommitKind.APPEND,
                 millis,
@@ -198,6 +274,9 @@ public class SnapshotManagerTest {
                 null,
                 null,
                 null,
+                null,
+                null,
+                null,
                 0L,
                 Snapshot.CommitKind.APPEND,
                 millis,
@@ -219,6 +298,9 @@ public class SnapshotManagerTest {
                         null,
                         null,
                         null,
+                        null,
+                        null,
+                        null,
                         0L,
                         Snapshot.CommitKind.APPEND,
                         millis,
@@ -234,7 +316,7 @@ public class SnapshotManagerTest {
     public void testLatestSnapshotOfUser() throws IOException, InterruptedException {
         FileIO localFileIO = LocalFileIO.create();
         SnapshotManager snapshotManager =
-                new SnapshotManager(localFileIO, new Path(tempDir.toString()));
+                newSnapshotManager(localFileIO, new Path(tempDir.toString()));
         // create 100 snapshots using user "lastCommitUser"
         for (long i = 0; i < 100; i++) {
             Snapshot snapshot =
@@ -245,6 +327,9 @@ public class SnapshotManagerTest {
                             null,
                             null,
                             null,
+                            null,
+                            null,
+                            null,
                             "lastCommitUser",
                             0L,
                             Snapshot.CommitKind.APPEND,
@@ -283,7 +368,7 @@ public class SnapshotManagerTest {
     public void testTraversalSnapshotsFromLatestSafely() throws IOException, InterruptedException {
         FileIO localFileIO = LocalFileIO.create();
         Path path = new Path(tempDir.toString());
-        SnapshotManager snapshotManager = new SnapshotManager(localFileIO, path);
+        SnapshotManager snapshotManager = newSnapshotManager(localFileIO, path);
         // create 10 snapshots
         for (long i = 0; i < 10; i++) {
             Snapshot snapshot =
@@ -295,6 +380,9 @@ public class SnapshotManagerTest {
                             null,
                             null,
                             null,
+                            null,
+                            null,
+                            null,
                             0L,
                             Snapshot.CommitKind.APPEND,
                             i * 1000,
@@ -379,12 +467,14 @@ public class SnapshotManagerTest {
     public void testLongLivedChangelog() throws Exception {
         FileIO localFileIO = LocalFileIO.create();
         SnapshotManager snapshotManager =
-                new SnapshotManager(localFileIO, new Path(tempDir.toString()));
+                newSnapshotManager(localFileIO, new Path(tempDir.toString()));
+        ChangelogManager changelogManager =
+                newChangelogManager(localFileIO, new Path(tempDir.toString()));
         long millis = 1L;
         for (long i = 1; i <= 5; i++) {
             Changelog changelog = createChangelogWithMillis(i, millis + i * 1000);
             localFileIO.tryToWriteAtomic(
-                    snapshotManager.longLivedChangelogPath(i), changelog.toJson());
+                    changelogManager.longLivedChangelogPath(i), changelog.toJson());
         }
 
         for (long i = 6; i <= 10; i++) {
@@ -392,22 +482,52 @@ public class SnapshotManagerTest {
             localFileIO.tryToWriteAtomic(snapshotManager.snapshotPath(i), snapshot.toJson());
         }
 
-        Assertions.assertThat(snapshotManager.earliestLongLivedChangelogId()).isEqualTo(1);
-        Assertions.assertThat(snapshotManager.latestChangelogId()).isEqualTo(10);
-        Assertions.assertThat(snapshotManager.latestLongLivedChangelogId()).isEqualTo(5);
+        Assertions.assertThat(changelogManager.earliestLongLivedChangelogId()).isEqualTo(1);
+        Assertions.assertThat(changelogManager.latestLongLivedChangelogId()).isEqualTo(5);
         Assertions.assertThat(snapshotManager.earliestSnapshotId()).isEqualTo(6);
         Assertions.assertThat(snapshotManager.latestSnapshotId()).isEqualTo(10);
-        Assertions.assertThat(snapshotManager.changelog(1)).isNotNull();
+        Assertions.assertThat(changelogManager.changelog(1)).isNotNull();
     }
 
     @Test
     public void testCommitChangelogWhenSameChangelogCommitTwice() throws IOException {
         FileIO localFileIO = LocalFileIO.create();
-        SnapshotManager snapshotManager =
-                new SnapshotManager(localFileIO, new Path(tempDir.toString()));
+        ChangelogManager snapshotManager =
+                newChangelogManager(localFileIO, new Path(tempDir.toString()));
         long id = 1L;
         Changelog changelog = createChangelogWithMillis(id, 1L);
         snapshotManager.commitChangelog(changelog, id);
         assertDoesNotThrow(() -> snapshotManager.commitChangelog(changelog, id));
     }
+
+    /**
+     * Test {@link SnapshotManager} to mock situations when there is a race condition, that the
+     * earliest snapshot is deleted by another thread in the middle of the current thread's
+     * invocation.
+     */
+    private static class TestSnapshotManager extends SnapshotManager {
+        private final boolean isRaceCondition;
+
+        private boolean deleteEarliestSnapshot = false;
+
+        public TestSnapshotManager(FileIO fileIO, Path tablePath, boolean isRaceCondition) {
+            super(fileIO, tablePath, DEFAULT_MAIN_BRANCH, null, null);
+            this.isRaceCondition = isRaceCondition;
+        }
+
+        @Override
+        public @Nullable Long earliestSnapshotId() {
+            Long snapshotId = super.earliestSnapshotId();
+            if (isRaceCondition && snapshotId != null && !deleteEarliestSnapshot) {
+                Path snapshotPath = snapshotPath(snapshotId);
+                try {
+                    fileIO().delete(snapshotPath, true);
+                } catch (IOException e) {
+                    throw new RuntimeException(e);
+                }
+                deleteEarliestSnapshot = true;
+            }
+            return snapshotId;
+        }
+    }
 }
