@@ -283,12 +283,20 @@ public class FlinkConnectorOptions {
                     .defaultValue(LookupCacheMode.AUTO)
                     .withDescription("The cache mode of lookup join.");
 
-    public static final ConfigOption<String> LOOKUP_DYNAMIC_PARTITION =
-            ConfigOptions.key("lookup.dynamic-partition")
+    public static final ConfigOption<String> SCAN_PARTITIONS =
+            ConfigOptions.key("scan.partitions")
                     .stringType()
                     .noDefaultValue()
+                    .withFallbackKeys("lookup.dynamic-partition")
                     .withDescription(
-                            "Specific dynamic partition for lookup, only support 'max_pt()' currently.");
+                            "Specify the partitions to scan. "
+                                    + "Partitions should be given in the form of key1=value1,key2=value2. "
+                                    + "Partition keys not specified will be filled with the value of "
+                                    + CoreOptions.PARTITION_DEFAULT_NAME.key()
+                                    + ". Multiple partitions should be separated by semicolon (;). "
+                                    + "This option can support normal source tables and lookup join tables. "
+                                    + "For lookup joins, two special values max_pt() and max_two_pt() are also supported, "
+                                    + "specifying the (two) partition(s) with the largest partition value.");
 
     public static final ConfigOption<Duration> LOOKUP_DYNAMIC_PARTITION_REFRESH_INTERVAL =
             ConfigOptions.key("lookup.dynamic-partition.refresh-interval")
@@ -348,6 +356,12 @@ public class FlinkConnectorOptions {
                     .withDescription(
                             "Allow sink committer and writer operator to be chained together");
 
+    public static final ConfigOption<PartitionMarkDoneActionMode> PARTITION_MARK_DONE_MODE =
+            key("partition.mark-done-action.mode")
+                    .enumType(PartitionMarkDoneActionMode.class)
+                    .defaultValue(PartitionMarkDoneActionMode.PROCESS_TIME)
+                    .withDescription("How to trigger partition mark done action.");
+
     public static final ConfigOption<Duration> PARTITION_IDLE_TIME_TO_DONE =
             key("partition.idle-time-to-done")
                     .durationType()
@@ -364,14 +378,6 @@ public class FlinkConnectorOptions {
                             "You can specify time interval for partition, for example, "
                                     + "daily partition is '1 d', hourly partition is '1 h'.");
 
-    public static final ConfigOption<Duration> PARTITION_IDLE_TIME_TO_REPORT_STATISTIC =
-            key("partition.idle-time-to-report-statistic")
-                    .durationType()
-                    .defaultValue(Duration.ofHours(1))
-                    .withDescription(
-                            "Set a time duration when a partition has no new data after this time duration, "
-                                    + "start to report the partition statistics to hms.");
-
     public static final ConfigOption<String> CLUSTERING_COLUMNS =
             key("sink.clustering.by-columns")
                     .stringType()
@@ -414,14 +420,34 @@ public class FlinkConnectorOptions {
                     .withDescription(
                             "Optional endInput watermark used in case of batch mode or bounded stream.");
 
-    public static final ConfigOption<Boolean> CHANGELOG_PRECOMMIT_COMPACT =
-            key("changelog.precommit-compact")
+    public static final ConfigOption<Boolean> PRECOMMIT_COMPACT =
+            key("precommit-compact")
                     .booleanType()
                     .defaultValue(false)
+                    .withFallbackKeys("changelog.precommit-compact")
                     .withDescription(
-                            "If true, it will add a changelog compact coordinator and worker operator after the writer operator,"
-                                    + "in order to compact several changelog files from the same partition into large ones, "
-                                    + "which can decrease the number of small files. ");
+                            "If true, it will add a compact coordinator and worker operator after the writer operator,"
+                                    + "in order to compact several changelog files (for primary key tables) "
+                                    + "or newly created data files (for unaware bucket tables) "
+                                    + "from the same partition into large ones, "
+                                    + "which can decrease the number of small files.");
+
+    public static final ConfigOption<Integer> CHANGELOG_PRECOMMIT_COMPACT_THREAD_NUM =
+            key("changelog.precommit-compact.thread-num")
+                    .intType()
+                    .noDefaultValue()
+                    .withDescription(
+                            "Maximum number of threads to copy bytes from small changelog files. "
+                                    + "By default is the number of processors available to the Java virtual machine.");
+
+    @ExcludeFromDocumentation("Most users won't need to adjust this config")
+    public static final ConfigOption<MemorySize> CHANGELOG_PRECOMMIT_COMPACT_BUFFER_SIZE =
+            key("changelog.precommit-compact.buffer-size")
+                    .memoryType()
+                    .defaultValue(MemorySize.ofMebiBytes(128))
+                    .withDescription(
+                            "The buffer size for copying bytes from small changelog files. "
+                                    + "The default value is 128 MB.");
 
     public static final ConfigOption<String> SOURCE_OPERATOR_UID_SUFFIX =
             key("source.operator-uid.suffix")
@@ -441,6 +467,21 @@ public class FlinkConnectorOptions {
                                     + "${UID_PREFIX}_${TABLE_NAME}_${USER_UID_SUFFIX}. If the uid suffix is not set, flink will "
                                     + "automatically generate the operator uid, which may be incompatible when the topology changes.");
 
+    public static final ConfigOption<Boolean> SCAN_BOUNDED =
+            key("scan.bounded")
+                    .booleanType()
+                    .noDefaultValue()
+                    .withDescription(
+                            "Bounded mode for Paimon consumer. "
+                                    + "By default, Paimon automatically selects bounded mode based on the mode of the Flink job.");
+
+    public static final ConfigOption<Integer> POSTPONE_DEFAULT_BUCKET_NUM =
+            key("postpone.default-bucket-num")
+                    .intType()
+                    .defaultValue(1)
+                    .withDescription(
+                            "Bucket number for the partitions compacted for the first time in postpone bucket tables.");
+
     public static List<ConfigOption<?>> getOptions() {
         final Field[] fields = FlinkConnectorOptions.class.getFields();
         final List<ConfigOption<?>> list = new ArrayList<>(fields.length);
@@ -526,4 +567,32 @@ public class FlinkConnectorOptions {
             return text(description);
         }
     }
+
+    /** The mode for partition mark done. */
+    public enum PartitionMarkDoneActionMode implements DescribedEnum {
+        PROCESS_TIME(
+                "process-time",
+                "Based on the time of the machine, mark the partition done once the processing time passes period time plus delay."),
+        WATERMARK(
+                "watermark",
+                "Based on the watermark of the input, mark the partition done once the watermark passes period time plus delay.");
+
+        private final String value;
+        private final String description;
+
+        PartitionMarkDoneActionMode(String value, String description) {
+            this.value = value;
+            this.description = description;
+        }
+
+        @Override
+        public String toString() {
+            return value;
+        }
+
+        @Override
+        public InlineElement getDescription() {
+            return text(description);
+        }
+    }
 }
