@@ -21,11 +21,16 @@ package org.apache.paimon.flink;
 import org.apache.paimon.CoreOptions;
 import org.apache.paimon.flink.util.AbstractTestBase;
 import org.apache.paimon.table.FileStoreTable;
+import org.apache.paimon.table.Table;
+import org.apache.paimon.table.source.snapshot.TimeTravelUtil;
 import org.apache.paimon.utils.BlockingIterator;
 import org.apache.paimon.utils.DateTimeUtils;
 import org.apache.paimon.utils.SnapshotNotExistException;
 
+import org.apache.paimon.shade.guava30.com.google.common.collect.ImmutableList;
+
 import org.apache.flink.api.dag.Transformation;
+import org.apache.flink.table.planner.factories.TestValuesTableFactory;
 import org.apache.flink.types.Row;
 import org.apache.flink.types.RowKind;
 import org.apache.flink.util.CloseableIterator;
@@ -35,6 +40,7 @@ import org.junit.jupiter.params.ParameterizedTest;
 import org.junit.jupiter.params.provider.ValueSource;
 
 import java.math.BigDecimal;
+import java.util.Arrays;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.List;
@@ -618,6 +624,22 @@ public class BatchFileStoreITCase extends CatalogITCaseBase {
         validateCount1PushDown(sql);
     }
 
+    private void validateCount1PushDown(String sql) {
+        Transformation<?> transformation = AbstractTestBase.translate(tEnv, sql);
+        while (!transformation.getInputs().isEmpty()) {
+            transformation = transformation.getInputs().get(0);
+        }
+        assertThat(transformation.getDescription()).contains("Count1AggFunction");
+    }
+
+    private void validateCount1NotPushDown(String sql) {
+        Transformation<?> transformation = AbstractTestBase.translate(tEnv, sql);
+        while (!transformation.getInputs().isEmpty()) {
+            transformation = transformation.getInputs().get(0);
+        }
+        assertThat(transformation.getDescription()).doesNotContain("Count1AggFunction");
+    }
+
     @Test
     public void testParquetRowDecimalAndTimestamp() {
         sql(
@@ -636,19 +658,121 @@ public class BatchFileStoreITCase extends CatalogITCaseBase {
                         Row.of(Row.of(DateTimeUtils.toLocalDateTime("2024-11-13 18:00:00", 0))));
     }
 
-    private void validateCount1PushDown(String sql) {
-        Transformation<?> transformation = AbstractTestBase.translate(tEnv, sql);
-        while (!transformation.getInputs().isEmpty()) {
-            transformation = transformation.getInputs().get(0);
+    @Test
+    public void testScanBounded() {
+        sql("INSERT INTO T VALUES (1, 11, 111), (2, 22, 222)");
+        List<Row> result;
+        try (CloseableIterator<Row> iter =
+                sEnv.executeSql("SELECT * FROM T /*+ OPTIONS('scan.bounded'='true') */")
+                        .collect()) {
+            result = ImmutableList.copyOf(iter);
+        } catch (Exception e) {
+            throw new RuntimeException(e);
         }
-        assertThat(transformation.getDescription()).contains("Count1AggFunction");
+        assertThat(result).containsExactlyInAnyOrder(Row.of(1, 11, 111), Row.of(2, 22, 222));
     }
 
-    private void validateCount1NotPushDown(String sql) {
-        Transformation<?> transformation = AbstractTestBase.translate(tEnv, sql);
-        while (!transformation.getInputs().isEmpty()) {
-            transformation = transformation.getInputs().get(0);
+    @Test
+    public void testIncrementTagQueryWithRescaleBucket() throws Exception {
+        sql("CREATE TABLE test (a INT PRIMARY KEY NOT ENFORCED, b INT) WITH ('bucket' = '1')");
+        Table table = paimonTable("test");
+
+        sql("INSERT INTO test VALUES (1, 11), (2, 22)");
+        sql("ALTER TABLE test SET ('bucket' = '2')");
+        sql("INSERT OVERWRITE test SELECT * FROM test");
+        sql("INSERT INTO test VALUES (3, 33)");
+
+        table.createTag("2024-01-01", 1);
+        table.createTag("2024-01-02", 3);
+
+        List<String> incrementalOptions =
+                Arrays.asList(
+                        "'incremental-between'='2024-01-01,2024-01-02'",
+                        "'incremental-to-auto-tag'='2024-01-02'");
+
+        for (String option : incrementalOptions) {
+            assertThatThrownBy(() -> sql("SELECT * FROM test /*+ OPTIONS (%s) */", option))
+                    .satisfies(
+                            anyCauseMatches(
+                                    TimeTravelUtil.InconsistentTagBucketException.class,
+                                    "The bucket number of two snapshots are different (1, 2), which is not supported in incremental diff query."));
         }
-        assertThat(transformation.getDescription()).doesNotContain("Count1AggFunction");
+    }
+
+    @Test
+    public void testAggregationWithNullSequenceField() {
+        sql(
+                "CREATE TABLE test ("
+                        + "  pk INT PRIMARY KEY NOT ENFORCED,"
+                        + "  v STRING,"
+                        + "  s0 INT,"
+                        + "  s1 INT"
+                        + ") WITH ("
+                        + "  'merge-engine' = 'aggregation',"
+                        + "  'sequence.field' = 's0,s1')");
+
+        sql(
+                "INSERT INTO test VALUES (1, 'A1', CAST (NULL AS INT), 1), (1, 'A2', 1, CAST (NULL AS INT))");
+        assertThat(sql("SELECT * FROM test")).containsExactly(Row.of(1, "A2", 1, null));
+
+        sql("INSERT INTO test VALUES (1, 'A3', 1, 0)");
+        assertThat(sql("SELECT * FROM test")).containsExactly(Row.of(1, "A3", 1, 0));
+    }
+
+    @Test
+    public void testScanWithSpecifiedPartitions() {
+        sql("CREATE TABLE P (pt STRING, id INT, v INT) PARTITIONED BY (pt)");
+        sql("CREATE TABLE Q (id INT)");
+        sql(
+                "INSERT INTO P VALUES ('a', 1, 10), ('a', 2, 20), ('b', 1, 11), ('b', 3, 31), ('c', 1, 12), ('c', 2, 22), ('c', 3, 32)");
+        sql("INSERT INTO Q VALUES (1), (2)");
+        String query =
+                "SELECT Q.id, P.v FROM Q INNER JOIN P /*+ OPTIONS('scan.partitions' = 'pt=b;pt=c') */ ON Q.id = P.id ORDER BY Q.id, P.v";
+        assertThat(sql(query)).containsExactly(Row.of(1, 11), Row.of(1, 12), Row.of(2, 22));
+    }
+
+    @Test
+    public void testEmptyTableIncrementalBetweenTimestamp() {
+        assertThat(sql("SELECT * FROM T /*+ OPTIONS('incremental-between-timestamp'='0,1') */"))
+                .isEmpty();
+    }
+
+    @Test
+    public void testIncrementScanMode() throws Exception {
+        sql(
+                "CREATE TABLE test_scan_mode (id INT PRIMARY KEY NOT ENFORCED, v STRING) WITH ('changelog-producer' = 'lookup')");
+
+        // snapshot 1,2
+        sql("INSERT INTO test_scan_mode VALUES (1, 'A')");
+        // snapshot 3,4
+        sql("INSERT INTO test_scan_mode VALUES (2, 'B')");
+
+        // snapshot 5,6
+        String dataId =
+                TestValuesTableFactory.registerData(
+                        Collections.singletonList(Row.ofKind(RowKind.DELETE, 2, "B")));
+        sEnv.executeSql(
+                "CREATE TEMPORARY TABLE source (id INT, v STRING) "
+                        + "WITH ('connector' = 'values', 'bounded' = 'true', 'data-id' = '"
+                        + dataId
+                        + "')");
+        sEnv.executeSql("INSERT INTO test_scan_mode SELECT * FROM source").await();
+
+        //  snapshot 7,8
+        sql("INSERT INTO test_scan_mode VALUES (3, 'C')");
+
+        List<Row> result =
+                sql(
+                        "SELECT * FROM `test_scan_mode$audit_log` "
+                                + "/*+ OPTIONS('incremental-between'='1,8','incremental-between-scan-mode'='diff') */");
+        assertThat(result).containsExactlyInAnyOrder(Row.of("+I", 3, "C"));
+
+        result =
+                sql(
+                        "SELECT * FROM `test_scan_mode$audit_log` "
+                                + "/*+ OPTIONS('incremental-between'='1,8','incremental-between-scan-mode'='delta') */");
+        assertThat(result)
+                .containsExactlyInAnyOrder(
+                        Row.of("+I", 2, "B"), Row.of("-D", 2, "B"), Row.of("+I", 3, "C"));
     }
 }
