@@ -141,6 +141,37 @@ public class BranchSqlITCase extends CatalogITCaseBase {
                         "+I[2, 20, dog]");
     }
 
+    @Test
+    public void testCreateBranchFromAnotherBranch() throws Exception {
+
+        sql(
+                "CREATE TABLE T ("
+                        + " pt INT"
+                        + ", k INT"
+                        + ", v STRING"
+                        + ", PRIMARY KEY (pt, k) NOT ENFORCED"
+                        + " ) PARTITIONED BY (pt) WITH ("
+                        + " 'bucket' = '2'"
+                        + " )");
+
+        // snapshot 1.
+        sql("INSERT INTO T VALUES" + " (1, 10, 'apple')," + " (1, 20, 'banana')");
+        sql("CALL sys.create_tag('default.T', 'tag1', 1)");
+
+        sql("CALL sys.create_branch('default.T', 'branch_A', 'tag1')");
+        FileStoreTable branchTable = paimonTable("T$branch_branch_A");
+        assertThat(branchTable.tagManager().tagExists("tag1")).isEqualTo(true);
+        assertThat(collectResult("SELECT * FROM T$branch_branch_A"))
+                .containsExactlyInAnyOrder("+I[1, 10, apple]", "+I[1, 20, banana]");
+
+        // Create branch_B from branch_A.
+        sql("CALL sys.create_branch('default.T$branch_branch_A', 'branch_B', 'tag1')");
+        FileStoreTable branchTableB = paimonTable("T$branch_branch_B");
+        assertThat(branchTableB.tagManager().tagExists("tag1")).isEqualTo(true);
+        assertThat(collectResult("SELECT * FROM T$branch_branch_B"))
+                .containsExactlyInAnyOrder("+I[1, 10, apple]", "+I[1, 20, banana]");
+    }
+
     @Test
     public void testCreateEmptyBranch() throws Exception {
         sql(
@@ -249,6 +280,12 @@ public class BranchSqlITCase extends CatalogITCaseBase {
                 .containsExactlyInAnyOrder("+I[1, 10, hunter]", "+I[2, 10, hunterX]");
 
         checkSnapshots(snapshotManager, 1, 2);
+
+        sql("alter table T set ('branch'='test')");
+
+        assertThatThrownBy(() -> sql("CALL sys.fast_forward('default.T', 'test')"))
+                .hasMessageContaining(
+                        "Fast-forward from the current branch 'test' is not allowed.");
     }
 
     @Test
@@ -307,6 +344,27 @@ public class BranchSqlITCase extends CatalogITCaseBase {
                         "+I[cat, 20]", "+I[dog, 30]", "+I[lion, 10]", "+I[wolf, 20]");
     }
 
+    @Test
+    public void testCrossPartitionFallbackBranchBatchRead() throws Exception {
+        sql(
+                "CREATE TABLE t ( pk INT PRIMARY KEY NOT ENFORCED, name STRING, dt STRING ) PARTITIONED BY (dt) WITH ( 'bucket' = '-1' )");
+        sql("CALL sys.create_branch('default.t', 'stream')");
+        sql("ALTER TABLE t SET ( 'scan.fallback-branch' = 'stream' )");
+
+        sql(
+                "INSERT INTO t VALUES (1, 'Jack', '20250227'), (1, 'Jackson', '20250227'), (2, 'Sam', '20250228')");
+        sql(
+                "INSERT INTO `t$branch_stream` VALUES (1, 'John Stream', '20250228'), (3, 'Rick Stream', '20250301')");
+        assertThat(collectResult("SELECT pk, name, dt FROM t order by dt"))
+                .containsExactlyInAnyOrder(
+                        "+I[1, Jackson, 20250227]",
+                        "+I[2, Sam, 20250228]",
+                        "+I[3, Rick Stream, 20250301]");
+        assertThat(collectResult("SELECT pk, name, dt FROM `t$branch_stream` order by dt"))
+                .containsExactlyInAnyOrder(
+                        "+I[1, John Stream, 20250228]", "+I[3, Rick Stream, 20250301]");
+    }
+
     @Test
     public void testDifferentRowTypes() throws Exception {
         sql(
@@ -531,15 +589,15 @@ public class BranchSqlITCase extends CatalogITCaseBase {
         sql("INSERT INTO t$branch_b1 VALUES (1, 4, 'S3'), (2, 2, 'S4')");
 
         assertThat(collectResult("SELECT `partition`, record_count, file_count FROM t$partitions"))
-                .containsExactlyInAnyOrder("+I[[1], 3, 3]", "+I[[2], 3, 2]");
+                .containsExactlyInAnyOrder("+I[{1}, 3, 3]", "+I[{2}, 3, 2]");
         assertThat(
                         collectResult(
                                 "SELECT `partition`, record_count, file_count FROM t$branch_b1$partitions"))
-                .containsExactlyInAnyOrder("+I[[1], 2, 2]", "+I[[2], 3, 2]");
+                .containsExactlyInAnyOrder("+I[{1}, 2, 2]", "+I[{2}, 3, 2]");
         assertThat(
                         collectResult(
                                 "SELECT `partition`, record_count, file_count FROM t$partitions /*+ OPTIONS('branch'='b1') */"))
-                .containsExactlyInAnyOrder("+I[[1], 2, 2]", "+I[[2], 3, 2]");
+                .containsExactlyInAnyOrder("+I[{1}, 2, 2]", "+I[{2}, 3, 2]");
     }
 
     @Test
