@@ -158,7 +158,10 @@ public class CatalogTableITCase extends CatalogITCaseBase {
         sql("CREATE TABLE T (a INT, b INT) with ('a.aa.aaa'='val1', 'b.bb.bbb'='val2')");
         sql("ALTER TABLE T SET ('c.cc.ccc' = 'val3')");
 
-        List<Row> result = sql("SELECT * FROM sys.all_table_options");
+        List<Row> result =
+                sql("SELECT * FROM sys.all_table_options").stream()
+                        .filter(row -> !row.getField(2).equals("path"))
+                        .collect(Collectors.toList());
         assertThat(result)
                 .containsExactly(
                         Row.of("default", "T", "a.aa.aaa", "val1"),
@@ -549,14 +552,14 @@ public class CatalogTableITCase extends CatalogITCaseBase {
         assertThatThrownBy(
                         () -> sql("CREATE TABLE T (a INT) WITH ('changelog-producer' = 'input')"))
                 .rootCause()
-                .isInstanceOf(UnsupportedOperationException.class)
+                .isInstanceOf(RuntimeException.class)
                 .hasMessageContaining(
                         "Can not set changelog-producer on table without primary keys");
 
         sql("CREATE TABLE T (a INT)");
         assertThatThrownBy(() -> sql("ALTER TABLE T SET ('changelog-producer'='input')"))
                 .rootCause()
-                .isInstanceOf(UnsupportedOperationException.class)
+                .isInstanceOf(RuntimeException.class)
                 .hasMessageContaining(
                         "Can not set changelog-producer on table without primary keys");
     }
@@ -794,7 +797,7 @@ public class CatalogTableITCase extends CatalogITCaseBase {
         assertThat(getRowStringList(rows1))
                 .containsExactlyInAnyOrder(
                         String.format(
-                                "[2],0,parquet,4,0,2,%s,{a=0, bb=0, dd=0, f=0, p=0},{a=23, bb=24, dd=25, f=26, p=2},{a=27, bb=28, dd=29, f=30, p=2}",
+                                "{2},0,parquet,4,0,2,%s,{a=0, bb=0, dd=0, f=0, p=0},{a=23, bb=24, dd=25, f=26, p=2},{a=27, bb=28, dd=29, f=30, p=2}",
                                 StringUtils.endsWith(tableName, "VALUE_COUNT")
                                         // value count table use all fields as min/max key
                                         ? "[23, 2, 24, 25, 26],[27, 2, 28, 29, 30]"
@@ -804,21 +807,21 @@ public class CatalogTableITCase extends CatalogITCaseBase {
                                                 // with key table use primary key trimmed partition
                                                 : "[23],[27]")),
                         String.format(
-                                "[1],0,parquet,0,0,2,%s,{a=0, bb=0, dd=2, f=2, p=0},{a=1, bb=2, dd=null, f=null, p=1},{a=3, bb=4, dd=null, f=null, p=1}",
+                                "{1},0,parquet,0,0,2,%s,{a=0, bb=0, dd=2, f=2, p=0},{a=1, bb=2, dd=null, f=null, p=1},{a=3, bb=4, dd=null, f=null, p=1}",
                                 StringUtils.endsWith(tableName, "VALUE_COUNT")
                                         ? "[1, 1, 2, S1],[3, 1, 4, S2]"
                                         : (StringUtils.endsWith(tableName, "APPEND_ONLY")
                                                 ? ","
                                                 : "[1],[3]")),
                         String.format(
-                                "[1],0,parquet,1,0,2,%s,{a=0, bb=0, dd=0, f=0, p=0},{a=5, bb=6, dd=7, f=9, p=1},{a=10, bb=11, dd=12, f=14, p=1}",
+                                "{1},0,parquet,1,0,2,%s,{a=0, bb=0, dd=0, f=0, p=0},{a=5, bb=6, dd=7, f=9, p=1},{a=10, bb=11, dd=12, f=14, p=1}",
                                 StringUtils.endsWith(tableName, "VALUE_COUNT")
                                         ? "[5, 1, 6, S3, 7, 8, 9],[10, 1, 11, S4, 12, 13, 14]"
                                         : (StringUtils.endsWith(tableName, "APPEND_ONLY")
                                                 ? ","
                                                 : "[5],[10]")),
                         String.format(
-                                "[1],0,parquet,4,0,2,%s,{a=0, bb=0, dd=0, f=0, p=0},{a=15, bb=16, dd=17, f=18, p=1},{a=19, bb=20, dd=21, f=22, p=1}",
+                                "{1},0,parquet,4,0,2,%s,{a=0, bb=0, dd=0, f=0, p=0},{a=15, bb=16, dd=17, f=18, p=1},{a=19, bb=20, dd=21, f=22, p=1}",
                                 StringUtils.endsWith(tableName, "VALUE_COUNT")
                                         ? "[15, 1, 16, 17, 18],[19, 1, 20, 21, 22]"
                                         : (StringUtils.endsWith(tableName, "APPEND_ONLY")
@@ -839,14 +842,14 @@ public class CatalogTableITCase extends CatalogITCaseBase {
         assertThat(getRowStringList(rows2))
                 .containsExactlyInAnyOrder(
                         String.format(
-                                "[1],0,parquet,0,0,2,%s,{a=0, b=0, c=0, d=2, e=2, f=2, p=0},{a=1, b=2, c=S1, d=null, e=null, f=null, p=1},{a=3, b=4, c=S2, d=null, e=null, f=null, p=1}",
+                                "{1},0,parquet,0,0,2,%s,{a=0, b=0, c=0, d=2, e=2, f=2, p=0},{a=1, b=2, c=S1, d=null, e=null, f=null, p=1},{a=3, b=4, c=S2, d=null, e=null, f=null, p=1}",
                                 StringUtils.endsWith(tableName, "VALUE_COUNT")
                                         ? "[1, 1, 2, S1],[3, 1, 4, S2]"
                                         : (StringUtils.endsWith(tableName, "APPEND_ONLY")
                                                 ? ","
                                                 : "[1],[3]")),
                         String.format(
-                                "[1],0,parquet,1,0,2,%s,{a=0, b=0, c=0, d=0, e=0, f=0, p=0},{a=5, b=6, c=S3, d=7, e=8, f=9, p=1},{a=10, b=11, c=S4, d=12, e=13, f=14, p=1}",
+                                "{1},0,parquet,1,0,2,%s,{a=0, b=0, c=0, d=0, e=0, f=0, p=0},{a=5, b=6, c=S3, d=7, e=8, f=9, p=1},{a=10, b=11, c=S4, d=12, e=13, f=14, p=1}",
                                 StringUtils.endsWith(tableName, "VALUE_COUNT")
                                         ? "[5, 1, 6, S3, 7, 8, 9],[10, 1, 11, S4, 12, 13, 14]"
                                         : (StringUtils.endsWith(tableName, "APPEND_ONLY")
@@ -866,18 +869,18 @@ public class CatalogTableITCase extends CatalogITCaseBase {
 
         assertThat(sql("SELECT `partition`, bucket, level FROM T_WITH_FILTER$files"))
                 .containsExactlyInAnyOrder(
-                        Row.of("[2]", 0, 5), Row.of("[5]", 0, 5), Row.of("[8]", 1, 0));
+                        Row.of("{2}", 0, 5), Row.of("{5}", 0, 5), Row.of("{8}", 1, 0));
 
         assertThat(
                         sql(
-                                "SELECT `partition`, bucket, level FROM T_WITH_FILTER$files WHERE `partition`='[2]'"))
-                .containsExactlyInAnyOrder(Row.of("[2]", 0, 5));
+                                "SELECT `partition`, bucket, level FROM T_WITH_FILTER$files WHERE `partition`='{2}'"))
+                .containsExactlyInAnyOrder(Row.of("{2}", 0, 5));
 
         assertThat(sql("SELECT `partition`, bucket, level FROM T_WITH_FILTER$files WHERE bucket=0"))
-                .containsExactlyInAnyOrder(Row.of("[2]", 0, 5), Row.of("[5]", 0, 5));
+                .containsExactlyInAnyOrder(Row.of("{2}", 0, 5), Row.of("{5}", 0, 5));
 
         assertThat(sql("SELECT `partition`, bucket, level FROM T_WITH_FILTER$files WHERE level=0"))
-                .containsExactlyInAnyOrder(Row.of("[8]", 1, 0));
+                .containsExactlyInAnyOrder(Row.of("{8}", 1, 0));
     }
 
     @Nonnull
@@ -1012,7 +1015,7 @@ public class CatalogTableITCase extends CatalogITCaseBase {
         sql("INSERT INTO %s VALUES (3, 1, 4, 'S3'), (1, 2, 2, 'S4')", table);
         List<Row> result =
                 sql("SELECT `partition`, record_count, file_count FROM %s$partitions", table);
-        assertThat(result).containsExactlyInAnyOrder(Row.of("[1]", 2L, 2L), Row.of("[2]", 3L, 2L));
+        assertThat(result).containsExactlyInAnyOrder(Row.of("{1}", 2L, 2L), Row.of("{2}", 3L, 2L));
 
         // assert new files in partition
         sql("INSERT INTO %s VALUES (3, 4, 4, 'S3'), (1, 3, 2, 'S4')", table);
@@ -1024,10 +1027,10 @@ public class CatalogTableITCase extends CatalogITCaseBase {
                                 table));
         assertThat(result)
                 .containsExactlyInAnyOrder(
-                        Row.of("[1]", 3L, 3L),
-                        Row.of("[2]", 4L, 3L),
-                        Row.of("[3]", 1L, 1L),
-                        Row.of("[4]", 1L, 1L));
+                        Row.of("{1}", 3L, 3L),
+                        Row.of("{2}", 4L, 3L),
+                        Row.of("{3}", 1L, 1L),
+                        Row.of("{4}", 1L, 1L));
 
         // assert delete partitions
         sql("ALTER TABLE %s DROP PARTITION (p = 2)", table);
@@ -1038,7 +1041,7 @@ public class CatalogTableITCase extends CatalogITCaseBase {
                                 table));
         assertThat(result)
                 .containsExactlyInAnyOrder(
-                        Row.of("[1]", 3L, 3L), Row.of("[3]", 1L, 1L), Row.of("[4]", 1L, 1L));
+                        Row.of("{1}", 3L, 3L), Row.of("{3}", 1L, 1L), Row.of("{4}", 1L, 1L));
 
         // add new file to p 2
         sql("INSERT INTO %s VALUES (1, 2, 2, 'S1')", table);
@@ -1049,10 +1052,10 @@ public class CatalogTableITCase extends CatalogITCaseBase {
                                 table));
         assertThat(result)
                 .containsExactlyInAnyOrder(
-                        Row.of("[1]", 3L, 3L),
-                        Row.of("[2]", 1L, 1L),
-                        Row.of("[3]", 1L, 1L),
-                        Row.of("[4]", 1L, 1L));
+                        Row.of("{1}", 3L, 3L),
+                        Row.of("{2}", 1L, 1L),
+                        Row.of("{3}", 1L, 1L),
+                        Row.of("{4}", 1L, 1L));
     }
 
     @Test
@@ -1100,6 +1103,72 @@ public class CatalogTableITCase extends CatalogITCaseBase {
         innerTestReadOptimizedTable();
     }
 
+    @Test
+    public void testBinlogTableStreamRead() throws Exception {
+        sql(
+                "CREATE TABLE T (a INT, b INT, primary key (a) NOT ENFORCED) with ('changelog-producer' = 'lookup', "
+                        + "'bucket' = '2')");
+        BlockingIterator<Row, Row> iterator =
+                streamSqlBlockIter("SELECT * FROM T$binlog /*+ OPTIONS('scan.mode' = 'latest') */");
+        sql("INSERT INTO T VALUES (1, 2)");
+        sql("INSERT INTO T VALUES (1, 3)");
+        sql("INSERT INTO T VALUES (2, 2)");
+        List<Row> rows = iterator.collect(3);
+        assertThat(rows)
+                .containsExactly(
+                        Row.of("+I", new Integer[] {1}, new Integer[] {2}),
+                        Row.of("+U", new Integer[] {1, 1}, new Integer[] {2, 3}),
+                        Row.of("+I", new Integer[] {2}, new Integer[] {2}));
+        iterator.close();
+    }
+
+    @Test
+    public void testBinlogTableBatchRead() throws Exception {
+        sql(
+                "CREATE TABLE T (a INT, b INT, primary key (a) NOT ENFORCED) with ('changelog-producer' = 'lookup', "
+                        + "'bucket' = '2')");
+        sql("INSERT INTO T VALUES (1, 2)");
+        sql("INSERT INTO T VALUES (1, 3)");
+        sql("INSERT INTO T VALUES (2, 2)");
+        List<Row> rows = sql("SELECT * FROM T$binlog /*+ OPTIONS('scan.mode' = 'latest') */");
+        assertThat(rows)
+                .containsExactly(
+                        Row.of("+I", new Integer[] {1}, new Integer[] {3}),
+                        Row.of("+I", new Integer[] {2}, new Integer[] {2}));
+    }
+
+    @Test
+    public void testIndexesTable() {
+        sql(
+                "CREATE TABLE T (pt STRING, a INT, b STRING, PRIMARY KEY (pt, a) NOT ENFORCED)"
+                        + " PARTITIONED BY (pt) with ('deletion-vectors.enabled'='true')");
+        sql(
+                "INSERT INTO T VALUES ('2024-10-01', 1, 'aaaaaaaaaaaaaaaaaaa'), ('2024-10-01', 2, 'b'), ('2024-10-01', 3, 'c')");
+        sql("INSERT INTO T VALUES ('2024-10-01', 1, 'a_new1'), ('2024-10-01', 3, 'c_new1')");
+
+        List<Row> rows = sql("SELECT * FROM `T$table_indexes` WHERE index_type = 'HASH'");
+        assertThat(rows.size()).isEqualTo(1);
+        Row row = rows.get(0);
+        assertThat(row.getField(0)).isEqualTo("{2024-10-01}");
+        assertThat(row.getField(1)).isEqualTo(0);
+        assertThat(row.getField(2)).isEqualTo("HASH");
+        assertThat(row.getField(3).toString().startsWith("index-")).isTrue();
+        assertThat(row.getField(4)).isEqualTo(12L);
+        assertThat(row.getField(5)).isEqualTo(3L);
+        assertThat(row.getField(6)).isNull();
+
+        rows = sql("SELECT * FROM `T$table_indexes` WHERE index_type = 'DELETION_VECTORS'");
+        assertThat(rows.size()).isEqualTo(1);
+        row = rows.get(0);
+        assertThat(row.getField(0)).isEqualTo("{2024-10-01}");
+        assertThat(row.getField(1)).isEqualTo(0);
+        assertThat(row.getField(2)).isEqualTo("DELETION_VECTORS");
+        assertThat(row.getField(3).toString().startsWith("index-")).isTrue();
+        assertThat(row.getField(4)).isEqualTo(33L);
+        assertThat(row.getField(5)).isEqualTo(1L);
+        assertThat(row.getField(6)).isNotNull();
+    }
+
     private void innerTestReadOptimizedTable() {
         // full compaction will always be performed at the end of batch jobs, as long as
         // full-compaction.delta-commits is set, regardless of its value
