@@ -23,9 +23,14 @@ import org.apache.paimon.flink.action.CompactAction;
 import org.apache.paimon.flink.util.AbstractTestBase;
 import org.apache.paimon.fs.Path;
 import org.apache.paimon.fs.local.LocalFileIO;
+import org.apache.paimon.fs.local.LocalFileIOLoader;
 import org.apache.paimon.utils.FailingFileIO;
+import org.apache.paimon.utils.StringUtils;
+import org.apache.paimon.utils.TraceableFileIO;
 
 import org.apache.flink.api.common.JobStatus;
+import org.apache.flink.api.common.RuntimeExecutionMode;
+import org.apache.flink.configuration.ExecutionOptions;
 import org.apache.flink.core.execution.JobClient;
 import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
 import org.apache.flink.table.api.TableEnvironment;
@@ -39,6 +44,8 @@ import org.junit.jupiter.api.Test;
 import org.junit.jupiter.api.Timeout;
 import org.junit.jupiter.params.ParameterizedTest;
 import org.junit.jupiter.params.provider.ValueSource;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 import java.io.IOException;
 import java.nio.file.Files;
@@ -60,16 +67,21 @@ import static org.assertj.core.api.Assertions.assertThatCode;
 public class PrimaryKeyFileStoreTableITCase extends AbstractTestBase {
 
     private static final int TIMEOUT = 180;
+    private static final Logger LOG = LoggerFactory.getLogger(PrimaryKeyFileStoreTableITCase.class);
 
     // ------------------------------------------------------------------------
     //  Test Utilities
     // ------------------------------------------------------------------------
     private String path;
     private Map<String, String> tableDefaultProperties;
+    private String externalPath1;
+    private String externalPath2;
 
     @BeforeEach
     public void before() throws IOException {
         path = getTempDirPath();
+        externalPath1 = getTempDirPath();
+        externalPath2 = getTempDirPath();
 
         ThreadLocalRandom random = ThreadLocalRandom.current();
         tableDefaultProperties = new HashMap<>();
@@ -79,6 +91,11 @@ public class PrimaryKeyFileStoreTableITCase extends AbstractTestBase {
     }
 
     private String createCatalogSql(String catalogName, String warehouse) {
+
+        return createCatalogSql(catalogName, warehouse, "");
+    }
+
+    private String createCatalogSql(String catalogName, String warehouse, String catalogOptions) {
         String defaultPropertyString = "";
         if (!tableDefaultProperties.isEmpty()) {
             defaultPropertyString = ", ";
@@ -91,6 +108,11 @@ public class PrimaryKeyFileStoreTableITCase extends AbstractTestBase {
                                                     e.getKey(), e.getValue()))
                             .collect(Collectors.joining(", "));
         }
+        if (!StringUtils.isNullOrWhitespaceOnly(catalogOptions)) {
+            return String.format(
+                    "CREATE CATALOG `%s` WITH ( 'type' = 'paimon', 'warehouse' = '%s' %s, %s )",
+                    catalogName, warehouse, defaultPropertyString, catalogOptions);
+        }
 
         return String.format(
                 "CREATE CATALOG `%s` WITH ( 'type' = 'paimon', 'warehouse' = '%s' %s )",
@@ -207,6 +229,177 @@ public class PrimaryKeyFileStoreTableITCase extends AbstractTestBase {
         innerTestChangelogProducing(Collections.singletonList("'changelog-producer' = 'lookup'"));
     }
 
+    @Test
+    public void testTableReadWriteWithExternalPathRoundRobin() throws Exception {
+        TableEnvironment sEnv =
+                tableEnvironmentBuilder()
+                        .streamingMode()
+                        .checkpointIntervalMs(ThreadLocalRandom.current().nextInt(900) + 100)
+                        .parallelism(1)
+                        .build();
+
+        sEnv.executeSql(createCatalogSql("testCatalog", path + "/warehouse"));
+        sEnv.executeSql("USE CATALOG testCatalog");
+        String externalPaths =
+                TraceableFileIO.SCHEME
+                        + "://"
+                        + externalPath1.toString()
+                        + ","
+                        + LocalFileIOLoader.SCHEME
+                        + "://"
+                        + externalPath2.toString();
+        sEnv.executeSql(
+                "CREATE TABLE T2 ( k INT, v STRING, PRIMARY KEY (k) NOT ENFORCED ) "
+                        + "WITH ( "
+                        + "'bucket' = '1',"
+                        + "'data-file.external-paths' = '"
+                        + externalPaths
+                        + "',"
+                        + "'data-file.external-paths.strategy' = 'round-robin'"
+                        + ")");
+
+        CloseableIterator<Row> it = collect(sEnv.executeSql("SELECT * FROM T2"));
+
+        // insert data
+        sEnv.executeSql("INSERT INTO T2 VALUES (1, 'A')").await();
+        // read initial data
+        List<String> actual = new ArrayList<>();
+        for (int i = 0; i < 1; i++) {
+            actual.add(it.next().toString());
+        }
+        assertThat(actual).containsExactlyInAnyOrder("+I[1, A]");
+
+        // insert data
+        sEnv.executeSql("INSERT INTO T2 VALUES (2, 'B')").await();
+
+        for (int i = 0; i < 1; i++) {
+            actual.add(it.next().toString());
+        }
+
+        // insert data
+        sEnv.executeSql("INSERT INTO T2 VALUES (3, 'C')").await();
+
+        for (int i = 0; i < 1; i++) {
+            actual.add(it.next().toString());
+        }
+
+        assertThat(actual).containsExactlyInAnyOrder("+I[1, A]", "+I[2, B]", "+I[3, C]");
+    }
+
+    @Test
+    public void testDropTableWithExternalPaths() throws Exception {
+        TableEnvironment sEnv =
+                tableEnvironmentBuilder()
+                        .streamingMode()
+                        .checkpointIntervalMs(ThreadLocalRandom.current().nextInt(900) + 100)
+                        .parallelism(1)
+                        .build();
+
+        sEnv.executeSql(createCatalogSql("testCatalog", path + "/warehouse"));
+        sEnv.executeSql("USE CATALOG testCatalog");
+        String externalPaths =
+                TraceableFileIO.SCHEME
+                        + "://"
+                        + externalPath1
+                        + ","
+                        + LocalFileIOLoader.SCHEME
+                        + "://"
+                        + externalPath2;
+        sEnv.executeSql(
+                "CREATE TABLE T2 ( k INT, v STRING, PRIMARY KEY (k) NOT ENFORCED ) "
+                        + "WITH ( "
+                        + "'bucket' = '1',"
+                        + "'data-file.external-paths' = '"
+                        + externalPaths
+                        + "',"
+                        + "'data-file.external-paths.strategy' = 'round-robin'"
+                        + ")");
+
+        CloseableIterator<Row> it = collect(sEnv.executeSql("SELECT * FROM T2"));
+
+        // insert data
+        sEnv.executeSql("INSERT INTO T2 VALUES (1, 'A')").await();
+        // read initial data
+        List<String> actual = new ArrayList<>();
+        for (int i = 0; i < 1; i++) {
+            actual.add(it.next().toString());
+        }
+        assertThat(actual).containsExactlyInAnyOrder("+I[1, A]");
+
+        // insert data
+        sEnv.executeSql("INSERT INTO T2 VALUES (2, 'B')").await();
+
+        for (int i = 0; i < 1; i++) {
+            actual.add(it.next().toString());
+        }
+
+        // insert data
+        sEnv.executeSql("INSERT INTO T2 VALUES (3, 'C')").await();
+
+        for (int i = 0; i < 1; i++) {
+            actual.add(it.next().toString());
+        }
+
+        assertThat(actual).containsExactlyInAnyOrder("+I[1, A]", "+I[2, B]", "+I[3, C]");
+
+        // drop table
+        sEnv.executeSql("DROP TABLE T2");
+
+        LocalFileIO fileIO = LocalFileIO.create();
+        assertThat(fileIO.exists(new Path(path + "/warehouse" + "/default.db" + "/T2"))).isFalse();
+        assertThat(fileIO.exists(new Path(externalPath1))).isFalse();
+        assertThat(fileIO.exists(new Path(externalPath2))).isFalse();
+    }
+
+    @Test
+    public void testTableReadWriteWithExternalPathSpecificFS() throws Exception {
+        TableEnvironment sEnv =
+                tableEnvironmentBuilder()
+                        .streamingMode()
+                        .checkpointIntervalMs(ThreadLocalRandom.current().nextInt(900) + 100)
+                        .parallelism(1)
+                        .build();
+
+        sEnv.executeSql(createCatalogSql("testCatalog", path + "/warehouse"));
+        sEnv.executeSql("USE CATALOG testCatalog");
+        String externalPaths =
+                TraceableFileIO.SCHEME
+                        + "://"
+                        + externalPath1.toString()
+                        + ","
+                        + "fake://"
+                        + externalPath2.toString();
+        sEnv.executeSql(
+                "CREATE TABLE T2 ( k INT, v STRING, PRIMARY KEY (k) NOT ENFORCED ) "
+                        + "WITH ( "
+                        + "'bucket' = '1',"
+                        + "'data-file.external-paths' = '"
+                        + externalPaths
+                        + "',"
+                        + "'data-file.external-paths.strategy' = 'specific-fs',"
+                        + "'data-file.external-paths.specific-fs' = 'traceable'"
+                        + ")");
+
+        CloseableIterator<Row> it = collect(sEnv.executeSql("SELECT * FROM T2"));
+
+        // insert data
+        sEnv.executeSql("INSERT INTO T2 VALUES (1, 'A')").await();
+        // read initial data
+        List<String> actual = new ArrayList<>();
+        for (int i = 0; i < 1; i++) {
+            actual.add(it.next().toString());
+        }
+        assertThat(actual).containsExactlyInAnyOrder("+I[1, A]");
+
+        // insert data
+        sEnv.executeSql("INSERT INTO T2 VALUES (2, 'B'), (3, 'C')").await();
+
+        for (int i = 0; i < 2; i++) {
+            actual.add(it.next().toString());
+        }
+        assertThat(actual).containsExactlyInAnyOrder("+I[1, A]", "+I[2, B]", "+I[3, C]");
+    }
+
     @Test
     public void testTableReadWriteBranch() throws Exception {
         TableEnvironment sEnv =
@@ -447,6 +640,31 @@ public class PrimaryKeyFileStoreTableITCase extends AbstractTestBase {
                         "The next expected snapshot is too big! Most possible cause might be the table had been recreated.");
     }
 
+    @Test
+    public void testDeleteFallbackBranch() {
+        TableEnvironment bEnv = tableEnvironmentBuilder().batchMode().build();
+        bEnv.executeSql(
+                createCatalogSql("testCatalog", path + "/warehouse", "'cache-enabled' = 'false'"));
+        bEnv.executeSql("USE CATALOG testCatalog");
+        bEnv.executeSql(
+                "CREATE TABLE t ( pt INT, k INT, v INT, PRIMARY KEY (pt, k) NOT ENFORCED ) "
+                        + "PARTITIONED BY (pt) "
+                        + "WITH ("
+                        + "    'bucket' = '2'\n"
+                        + "    ,'continuous.discovery-interval' = '1s'\n"
+                        + ")");
+        bEnv.executeSql("CALL sys.create_branch('default.t', 'branch1')");
+        bEnv.executeSql("ALTER TABLE t SET ('scan.fallback-branch' = 'branch1')");
+        // branch1 is fallback branch, can not be deleted
+        assertThatCode(() -> bEnv.executeSql("CALL sys.delete_branch('default.t', 'branch1')"))
+                .rootCause()
+                .hasMessageContaining("can not delete the fallback branch.");
+
+        // reset scan.fallback-branch
+        bEnv.executeSql("ALTER TABLE t RESET ('scan.fallback-branch')");
+        bEnv.executeSql("CALL sys.delete_branch('default.t', 'branch1')");
+    }
+
     @Test
     @Timeout(TIMEOUT)
     public void testChangelogCompactInBatchWrite() throws Exception {
@@ -461,7 +679,7 @@ public class PrimaryKeyFileStoreTableITCase extends AbstractTestBase {
                         + "WITH ("
                         + "    'bucket' = '10',\n"
                         + "    'changelog-producer' = 'lookup',\n"
-                        + "    'changelog.precommit-compact' = 'true',\n"
+                        + "    'precommit-compact' = 'true',\n"
                         + "    'snapshot.num-retained.min' = '3',\n"
                         + "    'snapshot.num-retained.max' = '3'\n"
                         + ")");
@@ -555,7 +773,7 @@ public class PrimaryKeyFileStoreTableITCase extends AbstractTestBase {
                         + "WITH ("
                         + "    'bucket' = '10',\n"
                         + "    'changelog-producer' = 'lookup',\n"
-                        + "    'changelog.precommit-compact' = 'true'\n"
+                        + "    'precommit-compact' = 'true'\n"
                         + ")");
 
         Path inputPath = new Path(path, "input");
@@ -743,20 +961,17 @@ public class PrimaryKeyFileStoreTableITCase extends AbstractTestBase {
             // Deletion vectors mode not support concurrent write
             numProducers = 1;
         }
-        List<TableResult> results =
-                testRandom(
-                        tEnv,
-                        numProducers,
-                        enableFailure,
-                        "'bucket' = '4',"
-                                + String.format(
-                                        "'deletion-vectors.enabled' = '%s'",
-                                        enableDeletionVectors));
 
-        for (TableResult result : results) {
-            result.await();
-        }
-        checkBatchResult(numProducers);
+        testRandom(
+                tEnv,
+                numProducers,
+                enableFailure,
+                "'bucket' = '4',"
+                        + String.format(
+                                "'deletion-vectors.enabled' = '%s'", enableDeletionVectors));
+
+        // changelog is produced by Flink normalize operator
+        checkChangelogTestResult(numProducers);
     }
 
     private void testFullCompactionChangelogProducerRandom(
@@ -799,7 +1014,7 @@ public class PrimaryKeyFileStoreTableITCase extends AbstractTestBase {
                                 + "'changelog-producer' = 'lookup', "
                                 + "'lookup-wait' = '%s', "
                                 + "'deletion-vectors.enabled' = '%s', "
-                                + "'changelog.precommit-compact' = '%s'",
+                                + "'precommit-compact' = '%s'",
                         random.nextBoolean() ? "4mb" : "8mb",
                         random.nextBoolean(),
                         enableDeletionVectors,
@@ -917,6 +1132,9 @@ public class PrimaryKeyFileStoreTableITCase extends AbstractTestBase {
         try (CloseableIterator<Row> it = collect(sEnv.executeSql("SELECT * FROM T"))) {
             while (it.hasNext()) {
                 Row row = it.next();
+                if (LOG.isDebugEnabled()) {
+                    LOG.debug("Changelog get {}", row);
+                }
                 checker.addChangelog(row);
                 if (((long) row.getField(2)) >= LIMIT) {
                     endCnt++;
@@ -939,7 +1157,7 @@ public class PrimaryKeyFileStoreTableITCase extends AbstractTestBase {
      * <p>All jobs will modify the same set of partitions to emulate conflicting writes. Each job
      * will write its own set of keys for easy result checking.
      */
-    private List<TableResult> testRandom(
+    private void testRandom(
             TableEnvironment tEnv, int numProducers, boolean enableFailure, String tableProperties)
             throws Exception {
         // producers will very quickly produce snapshots,
@@ -968,6 +1186,29 @@ public class PrimaryKeyFileStoreTableITCase extends AbstractTestBase {
         tEnv.getConfig()
                 .getConfiguration()
                 .set(ExecutionConfigOptions.TABLE_EXEC_RESOURCE_DEFAULT_PARALLELISM, 1);
+
+        // We use a large number of rows to mimic unbounded streams because there is a known
+        // consistency issue in bounded streams.
+        //
+        // For bounded streams, if COMPACT snapshot fails to commit when the stream ends (due to
+        // conflict or whatever reasons), we have no chance to modify the compaction result, so the
+        // changelogs produced by compaction will not be committed.
+        //
+        // If it happens in production, users can run another job to compact the table, or run
+        // another job to write more data into the table. These remaining changelogs will be
+        // produced again.
+        int factor;
+        RuntimeExecutionMode mode =
+                tEnv.getConfig().getConfiguration().get(ExecutionOptions.RUNTIME_MODE);
+        if (mode == RuntimeExecutionMode.BATCH) {
+            factor = 1;
+        } else if (mode == RuntimeExecutionMode.STREAMING) {
+            factor = 10;
+        } else {
+            throw new UnsupportedOperationException(
+                    "Unknown runtime execution mode " + mode.name());
+        }
+        int usefulNumRows = LIMIT + NUM_PARTS * NUM_KEYS;
         tEnv.executeSql(
                         "CREATE TABLE `default_catalog`.`default_database`.`S` ("
                                 + "  i INT"
@@ -976,10 +1217,10 @@ public class PrimaryKeyFileStoreTableITCase extends AbstractTestBase {
                                 + "  'fields.i.kind' = 'sequence',"
                                 + "  'fields.i.start' = '0',"
                                 + "  'fields.i.end' = '"
-                                + (LIMIT + NUM_PARTS * NUM_KEYS - 1)
+                                + (usefulNumRows - 1) * factor
                                 + "',"
                                 + "  'number-of-rows' = '"
-                                + (LIMIT + NUM_PARTS * NUM_KEYS)
+                                + usefulNumRows * factor
                                 + "',"
                                 + "  'rows-per-second' = '"
                                 + (LIMIT / 20 + ThreadLocalRandom.current().nextInt(LIMIT / 20))
@@ -987,8 +1228,6 @@ public class PrimaryKeyFileStoreTableITCase extends AbstractTestBase {
                                 + ")")
                 .await();
 
-        List<TableResult> results = new ArrayList<>();
-
         if (enableFailure) {
             FailingFileIO.reset(failingName, 2, 10000);
         }
@@ -1009,21 +1248,19 @@ public class PrimaryKeyFileStoreTableITCase extends AbstractTestBase {
             String v2Sql = "CAST(i AS STRING) || '.str' AS v2";
             tEnv.executeSql(
                     String.format(
-                            "CREATE TEMPORARY VIEW myView%d AS SELECT %s, %s, %s, %s FROM `default_catalog`.`default_database`.`S`",
+                            "CREATE TEMPORARY VIEW myView%d AS SELECT %s, %s, %s, %s, i FROM `default_catalog`.`default_database`.`S`",
                             i, ptSql, kSql, v1Sql, v2Sql));
 
             // run test SQL
             int idx = i;
-            TableResult result =
-                    FailingFileIO.retryArtificialException(
-                            () ->
-                                    tEnv.executeSql(
-                                            "INSERT INTO T /*+ OPTIONS('sink.parallelism' = '2') */ SELECT * FROM myView"
-                                                    + idx));
-            results.add(result);
+            FailingFileIO.retryArtificialException(
+                    () ->
+                            tEnv.executeSql(
+                                    "INSERT INTO T /*+ OPTIONS('sink.parallelism' = '2') */ SELECT pt, k, v1, v2 FROM myView"
+                                            + idx
+                                            + " WHERE i < "
+                                            + usefulNumRows));
         }
-
-        return results;
     }
 
     private void checkBatchResult(int numProducers) throws Exception {
