@@ -25,6 +25,8 @@ import org.apache.paimon.catalog.Identifier;
 import org.apache.paimon.client.ClientPool;
 import org.apache.paimon.options.CatalogOptions;
 import org.apache.paimon.options.Options;
+import org.apache.paimon.partition.Partition;
+import org.apache.paimon.partition.PartitionStatistics;
 import org.apache.paimon.schema.Schema;
 import org.apache.paimon.schema.SchemaChange;
 import org.apache.paimon.types.DataField;
@@ -55,9 +57,12 @@ import java.util.UUID;
 import java.util.concurrent.atomic.AtomicBoolean;
 
 import static org.apache.hadoop.hive.conf.HiveConf.ConfVars.METASTORECONNECTURLKEY;
+import static org.apache.paimon.CoreOptions.METASTORE_PARTITIONED_TABLE;
+import static org.apache.paimon.CoreOptions.METASTORE_TAG_TO_PARTITION;
 import static org.apache.paimon.hive.HiveCatalog.PAIMON_TABLE_IDENTIFIER;
 import static org.apache.paimon.hive.HiveCatalog.TABLE_TYPE_PROP;
 import static org.assertj.core.api.Assertions.assertThat;
+import static org.assertj.core.api.Assertions.assertThatExceptionOfType;
 import static org.assertj.core.api.Assertions.assertThatThrownBy;
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.fail;
@@ -72,7 +77,10 @@ public class HiveCatalogTest extends CatalogTestBase {
         String jdoConnectionURL = "jdbc:derby:memory:" + UUID.randomUUID();
         hiveConf.setVar(METASTORECONNECTURLKEY, jdoConnectionURL + ";create=true");
         String metastoreClientClass = "org.apache.hadoop.hive.metastore.HiveMetaStoreClient";
-        catalog = new HiveCatalog(fileIO, hiveConf, metastoreClientClass, warehouse);
+        Options catalogOptions = new Options();
+        catalogOptions.set(CatalogOptions.SYNC_ALL_PROPERTIES.key(), "false");
+        catalog =
+                new HiveCatalog(fileIO, hiveConf, metastoreClientClass, catalogOptions, warehouse);
     }
 
     @Test
@@ -169,11 +177,6 @@ public class HiveCatalogTest extends CatalogTestBase {
         assertThat(hiveConf.get("hive.metastore.uris")).isEqualTo("dummy-hms");
     }
 
-    @Test
-    public void testAlterDatabase() throws Exception {
-        this.alterDatabaseWhenSupportAlter();
-    }
-
     @Test
     public void testAddHiveTableParameters() {
         try {
@@ -308,8 +311,6 @@ public class HiveCatalogTest extends CatalogTestBase {
             Thread thread1 =
                     new Thread(
                             () -> {
-                                System.out.println(
-                                        "First thread started at " + System.currentTimeMillis());
                                 try {
                                     tables1.addAll(catalog.listTables(databaseName));
                                 } catch (Catalog.DatabaseNotExistException e) {
@@ -319,8 +320,6 @@ public class HiveCatalogTest extends CatalogTestBase {
             Thread thread2 =
                     new Thread(
                             () -> {
-                                System.out.println(
-                                        "Second thread started at " + System.currentTimeMillis());
                                 try {
                                     tables2.addAll(catalog.listTables(databaseName));
                                 } catch (Catalog.DatabaseNotExistException e) {
@@ -378,8 +377,6 @@ public class HiveCatalogTest extends CatalogTestBase {
         try (HiveCatalog customCatalog =
                 new HiveCatalog(fileIO, hiveConf, metastoreClientClass, warehouse)) {
             customBatchTables = customCatalog.listTables(databaseName);
-        } catch (Exception e) {
-            throw e;
         }
         assertEquals(defaultBatchTables.size(), customBatchTables.size());
         defaultBatchTables.sort(String::compareTo);
@@ -395,8 +392,6 @@ public class HiveCatalogTest extends CatalogTestBase {
         try (HiveCatalog invalidBatchSizeCatalog =
                 new HiveCatalog(fileIO, invalidHiveConf, metastoreClientClass, warehouse)) {
             invalidBatchSizeTables = invalidBatchSizeCatalog.listTables(databaseName);
-        } catch (Exception e) {
-            throw e;
         }
         assertEquals(defaultBatchTables.size(), invalidBatchSizeTables.size());
         invalidBatchSizeTables.sort(String::compareTo);
@@ -412,11 +407,22 @@ public class HiveCatalogTest extends CatalogTestBase {
         return true;
     }
 
+    @Override
+    protected boolean supportPartitions() {
+        return true;
+    }
+
     @Override
     protected boolean supportsFormatTable() {
         return true;
     }
 
+    @Override
+    protected void checkPartition(Partition expected, Partition actual) {
+        assertThat(actual.recordCount()).isEqualTo(expected.recordCount());
+        assertThat(actual.lastFileCreationTime()).isEqualTo(expected.lastFileCreationTime() / 1000);
+    }
+
     @Test
     public void testCreateExternalTableWithLocation(@TempDir java.nio.file.Path tempDir)
             throws Exception {
@@ -448,4 +454,89 @@ public class HiveCatalogTest extends CatalogTestBase {
 
         externalWarehouseCatalog.close();
     }
+
+    @Test
+    public void testTagToPartitionTable() throws Exception {
+        String databaseName = "testTagToPartitionTable";
+        catalog.dropDatabase(databaseName, true, true);
+        catalog.createDatabase(databaseName, true);
+        Identifier identifier = Identifier.create(databaseName, "table");
+        catalog.createTable(
+                identifier,
+                Schema.newBuilder()
+                        .option(METASTORE_TAG_TO_PARTITION.key(), "dt")
+                        .column("col", DataTypes.INT())
+                        .column("dt", DataTypes.STRING())
+                        .build(),
+                true);
+
+        catalog.createPartitions(
+                identifier,
+                Arrays.asList(
+                        Collections.singletonMap("dt", "20250101"),
+                        Collections.singletonMap("dt", "20250102")));
+        assertThat(catalog.listPartitions(identifier).stream().map(Partition::spec))
+                .containsExactlyInAnyOrder(
+                        Collections.singletonMap("dt", "20250102"),
+                        Collections.singletonMap("dt", "20250101"));
+    }
+
+    @Test
+    public void testAlterPartitions() throws Exception {
+        if (!supportPartitions()) {
+            return;
+        }
+        String databaseName = "testAlterPartitionTable";
+        catalog.dropDatabase(databaseName, true, true);
+        catalog.createDatabase(databaseName, true);
+        Identifier alterIdentifier = Identifier.create(databaseName, "alert_partitions");
+        catalog.createTable(
+                alterIdentifier,
+                Schema.newBuilder()
+                        .option(METASTORE_PARTITIONED_TABLE.key(), "true")
+                        .option(METASTORE_TAG_TO_PARTITION.key(), "dt")
+                        .column("col", DataTypes.INT())
+                        .column("dt", DataTypes.STRING())
+                        .partitionKeys("dt")
+                        .build(),
+                true);
+        catalog.createPartitions(
+                alterIdentifier,
+                Collections.singletonList(Collections.singletonMap("dt", "20250101")));
+        assertThat(catalog.listPartitions(alterIdentifier).stream().map(Partition::spec))
+                .containsExactlyInAnyOrder(Collections.singletonMap("dt", "20250101"));
+        long fileCreationTime = System.currentTimeMillis();
+        PartitionStatistics partition =
+                new PartitionStatistics(
+                        Collections.singletonMap("dt", "20250101"), 1, 2, 3, fileCreationTime);
+        catalog.alterPartitions(alterIdentifier, Collections.singletonList(partition));
+        Partition partitionFromServer = catalog.listPartitions(alterIdentifier).get(0);
+        checkPartition(
+                new Partition(
+                        Collections.singletonMap("dt", "20250101"),
+                        1,
+                        2,
+                        3,
+                        fileCreationTime,
+                        false),
+                partitionFromServer);
+
+        // Test when table does not exist
+        assertThatExceptionOfType(Catalog.TableNotExistException.class)
+                .isThrownBy(
+                        () ->
+                                catalog.alterPartitions(
+                                        Identifier.create(databaseName, "non_existing_table"),
+                                        Collections.singletonList(partition)));
+    }
+
+    @Override
+    protected boolean supportsAlterDatabase() {
+        return true;
+    }
+
+    @Override
+    protected boolean supportsViewDialects() {
+        return false;
+    }
 }
