@@ -21,6 +21,7 @@ package org.apache.paimon.spark.procedure
 import org.apache.paimon.spark.PaimonSparkTestBase
 
 import org.apache.spark.sql.Row
+import org.assertj.core.api.Assertions.assertThat
 
 class PurgeFilesProcedureTest extends PaimonSparkTestBase {
 
@@ -35,6 +36,7 @@ class PurgeFilesProcedureTest extends PaimonSparkTestBase {
 
     spark.sql("CALL paimon.sys.purge_files(table => 'test.T')")
     checkAnswer(spark.sql("select * from test.T"), Nil)
+    assertThat(spark.sql("select snapshot_id from test.`T$snapshots`").collect()).hasSize(1)
 
     spark.sql("insert into T select '2', 'aa'");
     checkAnswer(spark.sql("select * from test.T"), Row("2", "aa") :: Nil)
