@@ -160,7 +160,6 @@ class DeletionVectorTest extends PaimonSparkTestBase with AdaptiveSparkPlanHelpe
                        |CREATE TABLE T (id INT, name STRING)
                        |TBLPROPERTIES (
                        |  'deletion-vectors.enabled' = 'true',
-                       |  'compaction.max.file-num' = '50',
                        |  'bucket' = '$bucket' $bucketKey)
                        |""".stripMargin)
 
@@ -685,7 +684,7 @@ class DeletionVectorTest extends PaimonSparkTestBase with AdaptiveSparkPlanHelpe
     partitions.flatMap {
       partition =>
         dvMaintainerFactory
-          .createOrRestore(table.snapshotManager().latestSnapshotId(), partition)
+          .createOrRestore(table.snapshotManager().latestSnapshot(), partition)
           .deletionVectors()
           .asScala
     }.toMap
